@@ -131,6 +131,7 @@ ACTIVE_TASK::ACTIVE_TASK() {
     strcpy(web_graphics_url, "");
     strcpy(remote_desktop_addr, "");
     async_copy = NULL;
+    finish_file_time = 0;
 }
 
 // preempt this task;
@@ -187,6 +188,8 @@ int ACTIVE_TASK::preempt(int preempt_type) {
     return 0;
 }
 
+#ifndef SIM
+
 // called when a process has exited
 //
 void ACTIVE_TASK::cleanup_task() {
@@ -231,11 +234,11 @@ void ACTIVE_TASK::cleanup_task() {
 #endif
 
     if (config.exit_after_finish) {
+        gstate.write_state_file();
         exit(0);
     }
 }
 
-#ifndef SIM
 int ACTIVE_TASK::init(RESULT* rp) {
     result = rp;
     wup = rp->wup;
@@ -300,16 +303,19 @@ void ACTIVE_TASK_SET::get_memory_usage() {
     int retval;
     static bool first = true;
     static double last_cpu_time;
+    double diff=0;
 
-    double diff = gstate.now - last_mem_time;
-    if (diff < 0 || diff > MEMORY_USAGE_PERIOD + 10) {
-        // user has changed system clock,
-        // or there has been a long system sleep
-        //
-        last_mem_time = gstate.now;
-        return;
+    if (!first) {
+        diff = gstate.now - last_mem_time;
+        if (diff < 0 || diff > MEMORY_USAGE_PERIOD + 10) {
+            // user has changed system clock,
+            // or there has been a long system sleep
+            //
+            last_mem_time = gstate.now;
+            return;
+        }
+        if (diff < MEMORY_USAGE_PERIOD) return;
     }
-    if (diff < MEMORY_USAGE_PERIOD) return;
 
     last_mem_time = gstate.now;
     PROC_MAP pm;
@@ -345,18 +351,20 @@ void ACTIVE_TASK_SET::get_memory_usage() {
         procinfo_app(pi, v, pm, atp->app_version->graphics_exec_file);
         pi.working_set_size_smoothed = .5*(pi.working_set_size_smoothed + pi.working_set_size);
 
-        int pf = pi.page_fault_count - last_page_fault_count;
-        pi.page_fault_rate = pf/diff;
-        if (log_flags.mem_usage_debug) {
-            msg_printf(atp->result->project, MSG_INFO,
-                "[mem_usage] %s: WS %.2fMB, smoothed %.2fMB, page %.2fMB, %.2f page faults/sec, user CPU %.3f, kernel CPU %.3f",
-                atp->result->name,
-                pi.working_set_size/MEGA,
-                pi.working_set_size_smoothed/MEGA,
-                pi.swap_size/MEGA,
-                pi.page_fault_rate,
-                pi.user_time, pi.kernel_time
-            );
+        if (!first) {
+            int pf = pi.page_fault_count - last_page_fault_count;
+            pi.page_fault_rate = pf/diff;
+            if (log_flags.mem_usage_debug) {
+                msg_printf(atp->result->project, MSG_INFO,
+                    "[mem_usage] %s: WS %.2fMB, smoothed %.2fMB, page %.2fMB, %.2f page faults/sec, user CPU %.3f, kernel CPU %.3f",
+                    atp->result->name,
+                    pi.working_set_size/MEGA,
+                    pi.working_set_size_smoothed/MEGA,
+                    pi.swap_size/MEGA,
+                    pi.page_fault_rate,
+                    pi.user_time, pi.kernel_time
+                );
+            }
         }
     }
 
@@ -391,9 +399,7 @@ void ACTIVE_TASK_SET::get_memory_usage() {
         );
     }
     double new_cpu_time = pi.user_time + pi.kernel_time;
-    if (first) {
-        first = false;
-    } else {
+    if (!first) {
         non_boinc_cpu_usage = (new_cpu_time - last_cpu_time)/(diff*gstate.host_info.p_ncpus);
         // processes might have exited in the last 10 sec,
         // causing this to be negative.
@@ -405,6 +411,7 @@ void ACTIVE_TASK_SET::get_memory_usage() {
         }
     }
     last_cpu_time = new_cpu_time;
+    first = false;
 }
 
 #endif
@@ -413,7 +420,7 @@ void ACTIVE_TASK_SET::get_memory_usage() {
 // Move it from slot dir to project dir
 //
 int ACTIVE_TASK::move_trickle_file() {
-    char project_dir[256], new_path[1024], old_path[1024];
+    char project_dir[MAXPATHLEN], new_path[MAXPATHLEN], old_path[MAXPATHLEN];
     int retval;
 
     get_project_dir(result->project, project_dir, sizeof(project_dir));
@@ -440,7 +447,7 @@ int ACTIVE_TASK::current_disk_usage(double& size) {
     unsigned int i;
     int retval;
     FILE_INFO* fip;
-    char path[1024];
+    char path[MAXPATHLEN];
 
     retval = dir_size(slot_dir, size);
     if (retval) return retval;
@@ -464,7 +471,7 @@ bool ACTIVE_TASK_SET::is_slot_in_use(int slot) {
 }
 
 bool ACTIVE_TASK_SET::is_slot_dir_in_use(char* dir) {
-    char path[1024];
+    char path[MAXPATHLEN];
     unsigned int i;
     for (i=0; i<active_tasks.size(); i++) {
         get_slot_dir(active_tasks[i]->slot, path, sizeof(path));
@@ -479,7 +486,7 @@ bool ACTIVE_TASK_SET::is_slot_dir_in_use(char* dir) {
 void ACTIVE_TASK::get_free_slot(RESULT* rp) {
 #ifndef SIM
     int j, retval;
-    char path[1024];
+    char path[MAXPATHLEN];
 
     for (j=0; ; j++) {
         if (gstate.active_tasks.is_slot_in_use(j)) continue;
@@ -886,7 +893,7 @@ void ACTIVE_TASK_SET::report_overdue() {
 //
 int ACTIVE_TASK::handle_upload_files() {
     std::string filename;
-    char buf[256], path[1024];
+    char buf[MAXPATHLEN], path[MAXPATHLEN];
     int retval;
 
     DirScanner dirscan(slot_dir);
@@ -942,7 +949,7 @@ void ACTIVE_TASK_SET::network_available() {
 }
 
 void ACTIVE_TASK::upload_notify_app(const FILE_INFO* fip, const FILE_REF* frp) {
-    char path[256];
+    char path[MAXPATHLEN];
     sprintf(path, "%s/%s%s", slot_dir, UPLOAD_FILE_STATUS_PREFIX, frp->open_name);
     FILE* f = boinc_fopen(path, "w");
     if (!f) return;
