@@ -85,7 +85,7 @@ bool ACTIVE_TASK_SET::poll() {
     bool action;
     unsigned int i;
     static double last_time = 0;
-    if (gstate.now - last_time < TASK_POLL_PERIOD) return false;
+    if (!gstate.clock_change && gstate.now - last_time < TASK_POLL_PERIOD) return false;
     last_time = gstate.now;
 
     action = check_app_exited();
@@ -98,16 +98,45 @@ bool ACTIVE_TASK_SET::poll() {
         ACTIVE_TASK* atp = active_tasks[i];
         if (atp->task_state() == PROCESS_ABORT_PENDING) {
             if (gstate.now > atp->abort_time + ABORT_TIMEOUT) {
+                if (log_flags.task_debug) {
+                    msg_printf(atp->result->project, MSG_INFO,
+                        "[task] abort request timed out, killing task %s",
+                        atp->result->name
+                    );
+                }
                 atp->kill_task(false);
             }
         }
         if (atp->task_state() == PROCESS_QUIT_PENDING) {
             if (gstate.now > atp->quit_time + QUIT_TIMEOUT) {
+                if (log_flags.task_debug) {
+                    msg_printf(atp->result->project, MSG_INFO,
+                        "[task] quit request timed out, killing task %s",
+                        atp->result->name
+                    );
+                }
                 atp->kill_task(true);
             }
         }
     }
 
+    // Check for finish files every 10 sec.
+    // If we already found a finish file, kill the app;
+    // it must be hung somewhere in boinc_finish();
+    //
+    static double last_finish_check_time = 0;
+    if (gstate.clock_change || gstate.now - last_finish_check_time > 10) {
+        last_finish_check_time = gstate.now;
+        for (i=0; i<active_tasks.size(); i++) {
+            ACTIVE_TASK* atp = active_tasks[i];
+            if (atp->task_state() == PROCESS_UNINITIALIZED) continue;
+            if (atp->finish_file_time) {
+                atp->kill_task(false);
+            } else if (atp->finish_file_present()) {
+                atp->finish_file_time = gstate.now;
+            }
+        }
+    }
     if (action) {
         gstate.set_client_state_dirty("ACTIVE_TASK_SET::poll");
     }
@@ -223,7 +252,7 @@ int ACTIVE_TASK::kill_task(bool will_restart) {
 }
 
 // We have sent a quit request to the process; see if it's exited.
-// This is called when the core client exits,
+// This is called when the client exits,
 // or when a project is detached or reset
 //
 bool ACTIVE_TASK::has_task_exited() {
@@ -391,8 +420,9 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             strcpy(buf, "");
             if (temporary_exit_file_present(x, buf)) {
                 handle_temporary_exit(will_restart, x, buf);
+            } else {
+                handle_premature_exit(will_restart);
             }
-            handle_premature_exit(will_restart);
             break;
         case 0xc000013a:        // control-C??
         case 0x40010004:        // vista shutdown?? can someone explain this?
@@ -508,10 +538,6 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 
     cleanup_task();
 
-    if (config.exit_after_finish) {
-        exit(0);
-    }
-
     if (!will_restart) {
         copy_output_files();
         int retval = read_stderr_file();
@@ -529,13 +555,13 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 }
 
 bool ACTIVE_TASK::finish_file_present() {
-    char path[256];
+    char path[MAXPATHLEN];
     sprintf(path, "%s/%s", slot_dir, BOINC_FINISH_CALLED_FILE);
     return (boinc_file_exists(path) != 0);
 }
 
 bool ACTIVE_TASK::temporary_exit_file_present(double& x, char* buf) {
-    char path[256];
+    char path[MAXPATHLEN];
     sprintf(path, "%s/%s", slot_dir, TEMPORARY_EXIT_FILE);
     FILE* f = fopen(path, "r");
     if (!f) return false;
@@ -609,6 +635,8 @@ void ACTIVE_TASK_SET::send_heartbeats() {
     }
 }
 
+// send queued process-control messages; check for timeout
+//
 void ACTIVE_TASK_SET::process_control_poll() {
     unsigned int i;
     ACTIVE_TASK* atp;
@@ -682,7 +710,7 @@ bool ACTIVE_TASK_SET::check_app_exited() {
             // if we're running benchmarks, exited process
             // is probably a benchmark process; don't show error
             //
-            if (!gstate.are_cpu_benchmarks_running() && log_flags.task_debug) {
+            if (!gstate.benchmarks_running && log_flags.task_debug) {
                 msg_printf(NULL, MSG_INTERNAL_ERROR,
                     "Process %d not found\n", pid
                 );
@@ -740,7 +768,7 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
     //
     double min_interval = gstate.global_prefs.disk_interval;
     if (min_interval < 300) min_interval = 300;
-    if (gstate.now > last_disk_check_time + min_interval) {
+    if (gstate.clock_change || gstate.now > last_disk_check_time + min_interval) {
         do_disk_check = true;
     }
     for (i=0; i<active_tasks.size(); i++) {
@@ -805,7 +833,7 @@ int ACTIVE_TASK::abort_task(int exit_status, const char* msg) {
 //
 int ACTIVE_TASK::read_stderr_file() {
     char* buf1, *buf2;
-    char path[256];
+    char path[MAXPATHLEN];
 
     // truncate stderr output to the last 63KB;
     // it's unlikely that more than that will be useful
@@ -905,7 +933,7 @@ void ACTIVE_TASK_SET::request_reread_app_info() {
 // (or all tasks, if proj==0).
 // If they don't exit in 5 seconds,
 // send them a kill signal and wait up to 5 more seconds to exit.
-// This is called when the core client exits,
+// This is called when the client exits,
 // or when a project is detached or reset
 //
 int ACTIVE_TASK_SET::exit_tasks(PROJECT* proj) {
@@ -1005,11 +1033,11 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
         if (atp->task_state() != PROCESS_EXECUTING) continue;
         switch (reason) {
         case SUSPEND_REASON_CPU_THROTTLE:
-            // if we're doing CPU throttling, don't bother suspending apps
-            // that don't use a full CPU
-            //
             if (atp->result->dont_throttle()) continue;
-            if (atp->app_version->avg_ncpus < 1) continue;
+            // if we're doing CPU throttling,
+            // don't suspend CPU apps that use < 1 CPU
+            //
+            if (!atp->result->uses_coprocs() && atp->app_version->avg_ncpus < 1) continue;
             atp->preempt(REMOVE_NEVER);
             break;
         case SUSPEND_REASON_BENCHMARKS:
@@ -1069,7 +1097,7 @@ bool ACTIVE_TASK_SET::is_task_executing() {
 }
 
 // Send quit message to all app processes
-// This is called when the core client exits,
+// This is called when the client exits,
 // or when a project is detached or reset
 //
 void ACTIVE_TASK_SET::request_tasks_exit(PROJECT* proj) {
@@ -1283,7 +1311,7 @@ void ACTIVE_TASK_SET::get_msgs() {
     double old_time;
     static double last_time=0;
     double delta_t;
-    if (last_time) {
+    if (!gstate.clock_change && last_time) {
         delta_t = gstate.now - last_time;
 
         // Normally this is called every second.
@@ -1341,7 +1369,7 @@ void ACTIVE_TASK_SET::get_msgs() {
 // (this avoids rewriting the state file on each checkpoint)
 //
 void ACTIVE_TASK::write_task_state_file() {
-    char path[1024];
+    char path[MAXPATHLEN];
     sprintf(path, "%s/%s", slot_dir, TASK_STATE_FILENAME);
     FILE* f = fopen(path, "w");
     if (!f) return;
@@ -1366,7 +1394,7 @@ void ACTIVE_TASK::write_task_state_file() {
 // then the main state file
 //
 void ACTIVE_TASK::read_task_state_file() {
-    char buf[4096], path[1024], s[1024];
+    char buf[4096], path[MAXPATHLEN], s[1024];
     sprintf(path, "%s/%s", slot_dir, TASK_STATE_FILENAME);
     FILE* f = fopen(path, "r");
     if (!f) return;
