@@ -83,8 +83,6 @@ using std::string;
 #include "file_names.h"
 #include "result.h"
 #include "sandbox.h"
-#include "unix_util.h"
-
 
 #ifdef _WIN32
 #include "run_app_windows.h"
@@ -106,7 +104,7 @@ typedef BOOL (WINAPI *tDEB)(LPVOID lpEnvironment);
 
 #endif
 
-// Goes through an array of strings, and prints each string
+// print each string in an array
 //
 #ifndef _WIN32
 static void debug_print_argv(char** argv) {
@@ -160,7 +158,7 @@ int ACTIVE_TASK::get_shmem_seg_name() {
     if (!shm_handle) return ERR_SHMGET;
     sprintf(shmem_seg_name, "boinc_%d", i);
 #else
-    char init_data_path[256];
+    char init_data_path[MAXPATHLEN];
 #ifndef __EMX__
     // shmem_seg_name is not used with mmap() shared memory
     if (app_version->api_major_version() >= 6) {
@@ -195,7 +193,7 @@ int ACTIVE_TASK::get_shmem_seg_name() {
 }
 
 void ACTIVE_TASK::init_app_init_data(APP_INIT_DATA& aid) {
-    char project_dir[256], project_path[256];
+    char project_dir[256], project_path[MAXPATHLEN];
 
     aid.major_version = BOINC_MAJOR_VERSION;
     aid.minor_version = BOINC_MINOR_VERSION;
@@ -220,6 +218,17 @@ void ACTIVE_TASK::init_app_init_data(APP_INIT_DATA& aid) {
     relative_to_absolute("", aid.boinc_dir);
     strcpy(aid.authenticator, wup->project->authenticator);
     aid.slot = slot;
+#ifdef _WIN32
+    if (strstr(gstate.host_info.os_name, "Windows 2000")) {
+        // Win2K immediately reuses PIDs, so can't use this mechanism
+        //
+        aid.client_pid = 0;
+    } else {
+        aid.client_pid = GetCurrentProcessId();
+    }
+#else
+    aid.client_pid = getpid();
+#endif
     strcpy(aid.wu_name, wup->name);
     strcpy(aid.result_name, result->name);
     aid.user_total_credit = wup->project->user_total_credit;
@@ -262,6 +271,7 @@ void ACTIVE_TASK::init_app_init_data(APP_INIT_DATA& aid) {
 
     }
     aid.ncpus = app_version->avg_ncpus;
+    aid.vbox_window = config.vbox_window;
     aid.checkpoint_period = gstate.global_prefs.disk_interval;
     aid.fraction_done_start = 0;
     aid.fraction_done_end = 1;
@@ -279,7 +289,7 @@ void ACTIVE_TASK::init_app_init_data(APP_INIT_DATA& aid) {
 //
 int ACTIVE_TASK::write_app_init_file(APP_INIT_DATA& aid) {
     FILE *f;
-    char init_data_path[256];
+    char init_data_path[MAXPATHLEN];
 
     sprintf(init_data_path, "%s/%s", slot_dir, INIT_DATA_FILE);
 
@@ -308,7 +318,7 @@ static int create_dirs_for_logical_name(
     const char* name, const char* slot_dir
 ) {
     char buf[1024];
-    char dir_path[1024];
+    char dir_path[MAXPATHLEN];
     int retval;
 
     strcpy(buf, name);
@@ -353,7 +363,7 @@ bool ACTIVE_TASK::must_copy_file(FILE_REF& fref, bool is_io_file) {
 int ACTIVE_TASK::setup_file(
     FILE_INFO* fip, FILE_REF& fref, char* file_path, bool input, bool is_io_file
 ) {
-    char link_path[256], rel_file_path[256], open_name[256];
+    char link_path[MAXPATHLEN], rel_file_path[MAXPATHLEN], open_name[256];
     int retval;
     PROJECT* project = result->project;
 
@@ -426,7 +436,7 @@ int ACTIVE_TASK::link_user_files() {
     unsigned int i;
     FILE_REF fref;
     FILE_INFO* fip;
-    char file_path[1024];
+    char file_path[MAXPATHLEN];
 
     for (i=0; i<project->user_files.size(); i++) {
         fref = project->user_files[i];
@@ -481,7 +491,7 @@ int ACTIVE_TASK::copy_output_files() {
 //   ACTIVE_TASK::task_state is PROCESS_EXECUTING
 //
 int ACTIVE_TASK::start() {
-    char exec_name[256], file_path[256], buf[256], exec_path[256];
+    char exec_name[256], file_path[MAXPATHLEN], buf[256], exec_path[MAXPATHLEN];
     char cmdline[80000];    // 64KB plus some extra
     unsigned int i;
     FILE_REF fref;
@@ -509,7 +519,8 @@ int ACTIVE_TASK::start() {
             if (fip) {
                 snprintf(
                     buf, sizeof(buf),
-                    "Input file %s missing or invalid: %d", fip->name, retval
+                    "Input file %s missing or invalid: %s",
+                    fip->name, boincerror(retval)
                 );
             } else {
                 strcpy(buf, "Input file missing or invalid");
@@ -544,7 +555,7 @@ int ACTIVE_TASK::start() {
     init_app_init_data(aid);
     retval = write_app_init_file(aid);
     if (retval) {
-        sprintf(buf, "Can't write init file: %d", retval);
+        sprintf(buf, "Can't write init file: %s", boincerror(retval));
         goto error;
     }
 
@@ -628,7 +639,7 @@ int ACTIVE_TASK::start() {
     PROCESS_INFORMATION process_info;
     STARTUPINFO startup_info;
     LPVOID environment_block = NULL;
-    char slotdirpath[256];
+    char slotdirpath[MAXPATHLEN];
     char error_msg[1024];
     char error_msg2[1024];
 
@@ -643,7 +654,7 @@ int ACTIVE_TASK::start() {
     app_client_shm.reset_msgs();
 
     if (config.run_apps_manually) {
-        // fill in core client's PID so we won't think app has exited
+        // fill in client's PID so we won't think app has exited
         //
         pid = GetCurrentProcessId();
         process_handle = GetCurrentProcess();
@@ -685,7 +696,7 @@ int ACTIVE_TASK::start() {
 
             if (!pCEB(&environment_block, sandbox_account_service_token, FALSE)) {
                 if (log_flags.task) {
-                    windows_error_string(error_msg, sizeof(error_msg));
+                    windows_format_error_string(GetLastError(), error_msg, sizeof(error_msg));
                     msg_printf(wup->project, MSG_INFO,
                         "Process environment block creation failed: %s", error_msg
                     );
@@ -708,7 +719,7 @@ int ACTIVE_TASK::start() {
                 success = true;
                 break;
             } else {
-                windows_error_string(error_msg, sizeof(error_msg));
+                windows_format_error_string(GetLastError(), error_msg, sizeof(error_msg));
                 msg_printf(wup->project, MSG_INTERNAL_ERROR,
                     "Process creation failed: %s", error_msg
                 );
@@ -716,7 +727,7 @@ int ACTIVE_TASK::start() {
 
             if (!pDEB(environment_block)) {
                 if (log_flags.task) {
-                    windows_error_string(error_msg, sizeof(error_msg2));
+                    windows_format_error_string(GetLastError(), error_msg, sizeof(error_msg2));
                     msg_printf(wup->project, MSG_INFO,
                         "Process environment block cleanup failed: %s",
                         error_msg2
@@ -746,7 +757,7 @@ int ACTIVE_TASK::start() {
                 success = true;
                 break;
             } else {
-                windows_error_string(error_msg, sizeof(error_msg));
+                windows_format_error_string(GetLastError(), error_msg, sizeof(error_msg));
                 msg_printf(wup->project, MSG_INTERNAL_ERROR,
                     "Process creation failed: %s", error_msg
                 );
@@ -787,7 +798,7 @@ int ACTIVE_TASK::start() {
     //
     retval = chdir(slot_dir);
     if (retval) {
-        sprintf(buf, "Can't change directory: %s", slot_dir, boincerror(retval));
+        sprintf(buf, "Can't change directory to %s: %s", slot_dir, boincerror(retval));
         goto error;
     }
 
@@ -809,7 +820,7 @@ int ACTIVE_TASK::start() {
     sprintf(buf, "../../%s", exec_path );
     pid = spawnv(P_NOWAIT, buf, argv);
     if (pid == -1) {
-        sprintf(buf, "Process creation failed: %s\n", buf, boincerror(retval));
+        sprintf(buf, "Process creation failed: %s\n", boincerror(retval));
         chdir(current_dir);
         retval = ERR_EXEC;
         goto error;
@@ -911,7 +922,7 @@ int ACTIVE_TASK::start() {
     if (pid == 0) {
         // from here on we're running in a new process.
         // If an error happens,
-        // exit nonzero so that the core client knows there was a problem.
+        // exit nonzero so that the client knows there was a problem.
 
         // don't pass stdout to the app
         //
@@ -1004,7 +1015,7 @@ int ACTIVE_TASK::start() {
         }
         sprintf(buf, "../../%s", exec_path);
         if (g_use_sandbox) {
-            char switcher_path[100];
+            char switcher_path[MAXPATHLEN];
             sprintf(switcher_path, "../../%s/%s",
                 SWITCHER_DIR, SWITCHER_FILE_NAME
             );
@@ -1053,7 +1064,7 @@ error:
     // Verify it to trigger another download.
     //
     gstate.input_files_available(result, true);
-    gstate.report_result_error(*result, "couldn't start %s: %d", buf, retval);
+    gstate.report_result_error(*result, "couldn't start app: %s", buf);
     if (log_flags.task_debug) {
         msg_printf(wup->project, MSG_INFO,
             "[task] couldn't start app: %s", buf
