@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-// boinccmd: command-line interface to a BOINC core client,
+// boinccmd: command-line interface to a BOINC client,
 // using GUI RPCs.
 //
 // usage: boinccmd [--host hostname] [--passwd passwd] command
@@ -77,23 +77,23 @@ Commands:\n\
  --lookup_account URL email passwd\n\
  --network_available                retry deferred network communication\n\
  --project URL op                   project operation\n\
-   op = reset | detach | update | suspend | resume | nomorework | allowmorework\n\
+   op = reset | detach | update | suspend | resume | nomorework | allowmorework | detach_when_done | dont_detach_when_done\n\
  --project_attach URL auth          attach to project\n\
  --quit                             tell client to exit\n\
  --quit_acct_mgr                    quit current account manager\n\
  --read_cc_config\n\
  --read_global_prefs_override\n\
  --run_benchmarks\n\
- --set_debts URL1 std1 ltd1 [URL2 std2 ltd2 ...]\n\
  --set_gpu_mode mode duration       set GPU run mode for given duration\n\
    mode = always | auto | never\n\
- --set_network_mode mode duration\n\
+ --set_network_mode mode duration   set network mode for given duration\n\
+   mode = always | auto | never\n\
  --set_proxy_settings\n\
  --set_run_mode mode duration       set run mode for given duration\n\
    mode = always | auto | never\n\
  --task url task_name op            task operation\n\
    op = suspend | resume | abort\n\
- --version, -V                      show core client version\n\
+ --version, -V                      show client version\n\
 "
 );
     exit(1);
