@@ -43,7 +43,7 @@ struct MESSAGE_DESC {
 //
 struct MESSAGE_DESCS {
     std::deque<MESSAGE_DESC*> msgs;
-    void insert(struct PROJ_AM *p, int priority, int now, char* msg);
+    void insert(PROJ_AM *p, int priority, int now, char* msg);
     void write(int seqno, class MIOFILE&, bool translatable);
     int highest_seqno();
     void cleanup();
@@ -60,7 +60,7 @@ extern MESSAGE_DESCS message_descs;
 
 // Show a message, preceded by timestamp and project name
 
-extern void msg_printf(struct PROJ_AM *p, int priority, const char *fmt, ...)
+extern void msg_printf(PROJ_AM *p, int priority, const char *fmt, ...)
     __attribute__ ((format (printf, 3, 4)))
 ;
 
@@ -69,12 +69,10 @@ extern void msg_printf(struct PROJ_AM *p, int priority, const char *fmt, ...)
 // is_html: true if message body contains HTML tags
 // link: URL for "more..." link
 //
-extern void msg_printf_notice(struct PROJ_AM *p, bool is_html, const char* link, const char *fmt, ...)
+extern void msg_printf_notice(PROJ_AM *p, bool is_html, const char* link, const char *fmt, ...)
     __attribute__ ((format (printf, 4, 5)))
 ;
 
-extern void strip_translation(char*);
-
 #define _(x) "_(\""x"\")"
 
 #endif
