@@ -26,6 +26,7 @@
 #include <ctime>
 #include <cstdarg>
 #include <cstring>
+#include <math.h>
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -47,9 +48,11 @@
 #include "run_app_windows.h"
 #endif
 
+#include "app_config.h"
 #include "async_file.h"
 #include "client_msgs.h"
 #include "cs_notice.h"
+#include "cs_proxy.h"
 #include "cs_trickle.h"
 #include "file_names.h"
 #include "hostinfo.h"
@@ -81,6 +84,7 @@ CLIENT_STATE::CLIENT_STATE()
     scheduler_op = new SCHEDULER_OP(http_ops);
 #endif
     client_state_dirty = false;
+    clock_change = false;
     check_all_logins = false;
     cmdline_gui_rpc_port = 0;
     run_cpu_benchmarks = false;
@@ -128,8 +132,9 @@ CLIENT_STATE::CLIENT_STATE()
     redirect_io = false;
     disable_graphics = false;
     cant_write_state_file = false;
+    benchmarks_running = false;
 
-    debt_interval_start = 0;
+    rec_interval_start = 0;
     retry_shmem_time = 0;
     must_schedule_cpus = true;
     no_gui_rpc = false;
@@ -142,6 +147,9 @@ CLIENT_STATE::CLIENT_STATE()
     launched_by_manager = false;
     initialized = false;
     last_wakeup_time = dtime();
+#ifdef _WIN32
+    have_sysmon_msg = false;
+#endif
 }
 
 void CLIENT_STATE::show_host_info() {
@@ -155,12 +163,14 @@ void CLIENT_STATE::show_host_info() {
     if (ncpus != host_info.p_ncpus) {
         msg_printf(NULL, MSG_INFO, "Using %d CPUs", ncpus);
     }
+#if 0
     if (host_info.m_cache > 0) {
         msg_printf(NULL, MSG_INFO,
             "Processor: %s cache",
             buf
         );
     }
+#endif
     msg_printf(NULL, MSG_INFO,
         "Processor features: %s", host_info.p_features
     );
@@ -202,6 +212,7 @@ void CLIENT_STATE::show_host_info() {
 
 int rsc_index(const char* name) {
     const char* nm = strcmp(name, "CUDA")?name:GPU_TYPE_NVIDIA;
+        // handle old state files
     for (int i=0; i<coprocs.n_rsc; i++) {
         if (!strcmp(nm, coprocs.coprocs[i].type)) {
             return i;
@@ -269,7 +280,9 @@ void CLIENT_STATE::set_now() {
 
     // if time went backward significantly, clear delays
     //
+    clock_change = false;
     if (x < (now-60)) {
+        clock_change = true;
         clear_absolute_times();
     }
 
@@ -298,11 +311,11 @@ int CLIENT_STATE::init() {
 
     srand((unsigned int)time(0));
     now = dtime();
-    client_start_time = now;
     scheduler_op->url_random = drand();
 
     notices.init();
     daily_xfer_history.init();
+    time_stats.init();
 
     detect_platforms();
     time_stats.start();
@@ -368,8 +381,7 @@ int CLIENT_STATE::init() {
         vector<string> descs;
         vector<string> warnings;
         coprocs.get(
-            config.use_all_gpus, descs, warnings,
-            config.ignore_nvidia_dev, config.ignore_ati_dev
+            config.use_all_gpus, descs, warnings, config.ignore_gpu_instance
         );
         for (i=0; i<descs.size(); i++) {
             msg_printf(NULL, MSG_INFO, "%s", descs[i].c_str());
@@ -386,6 +398,10 @@ int CLIENT_STATE::init() {
 #if 0
         msg_printf(NULL, MSG_INFO, "Faking an ATI GPU");
         coprocs.ati.fake(512*MEGA, 256*MEGA, 2);
+#endif
+#if 0
+        msg_printf(NULL, MSG_INFO, "Faking an Intel GPU");
+        coprocs.intel_gpu.fake(512*MEGA, 256*MEGA, 2);
 #endif
     }
 
@@ -403,7 +419,14 @@ int CLIENT_STATE::init() {
             coprocs.add(coprocs.ati);
         }
     }
-    host_info._coprocs = coprocs;
+    if (coprocs.have_intel_gpu()) {
+        if (rsc_index(GPU_TYPE_INTEL)>0) {
+            msg_printf(NULL, MSG_INFO, "INTEL GPU info taken from cc_config.xml");
+        } else {
+            coprocs.add(coprocs.intel_gpu);
+        }
+    }
+    host_info.coprocs = coprocs;
     
     if (coprocs.none() ) {
         msg_printf(NULL, MSG_INFO, "No usable GPUs found");
@@ -429,6 +452,10 @@ int CLIENT_STATE::init() {
     //
     parse_state_file();
 
+    // check for app_config.xml files in project dirs
+    //
+    check_app_config();
+
     // this needs to go after parse_state_file because
     // GPU exclusions refer to projects
     //
@@ -463,10 +490,10 @@ int CLIENT_STATE::init() {
             avp->flops = avp->avg_ncpus * host_info.p_fpops;
 
             // for GPU apps, use conservative estimate:
-            // assume app will run at peak CPU speed, not peak GPU
+            // assume GPU runs at 10X peak CPU speed
             //
             if (avp->gpu_usage.rsc_type) {
-                avp->flops += avp->gpu_usage.usage * host_info.p_fpops;
+                avp->flops += avp->gpu_usage.usage * 10 * host_info.p_fpops;
             }
         }
     }
@@ -522,10 +549,12 @@ int CLIENT_STATE::init() {
     }
     // if new version of client,
     // - run CPU benchmarks
+    // - get new project list
     // - contact reference site (or some project) to trigger firewall alert
     //
     if (new_client) {
         run_cpu_benchmarks = true;
+        all_projects_list_check_time = 0;
         if (config.dont_contact_ref_site) {
             if (projects.size() > 0) {
                 projects[0]->master_url_fetch_pending = true;
@@ -544,7 +573,7 @@ int CLIENT_STATE::init() {
     request_schedule_cpus("Startup");
     request_work_fetch("Startup");
     work_fetch.init();
-    debt_interval_start = now;
+    rec_interval_start = now;
 
     // set up the project and slot directories
     //
@@ -561,12 +590,13 @@ int CLIENT_STATE::init() {
     //
     set_client_state_dirty("init");
 
-    // initialize GUI RPC data structures before we start accepting
-    // GUI RPC's.
+    // check for initialization files
     //
     acct_mgr_info.init();
     project_init.init();
 
+    // set up for handling GUI RPCs
+    //
     if (!no_gui_rpc) {
         // When we're running at boot time,
         // it may be a few seconds before we can socket/bind/listen.
@@ -599,6 +629,24 @@ int CLIENT_STATE::init() {
 #endif
 
     http_ops->cleanup_temp_files();
+    
+    // must parse env vars after parsing state file
+    // otherwise items will get overwritten with state file info
+    //
+    parse_env_vars();
+
+    // do this after parsing env vars
+    //
+    proxy_info_startup();
+
+    if (gstate.projects.size() == 0) {
+        msg_printf(NULL, MSG_INFO,
+            "This computer is not attached to any projects"
+        );
+        msg_printf(NULL, MSG_INFO,
+            "Visit http://boinc.berkeley.edu for instructions"
+        );
+    }
 
     // get list of BOINC projects occasionally,
     // and initialize notice RSS feeds
@@ -723,11 +771,18 @@ bool CLIENT_STATE::poll_slow_events() {
         last_wakeup_time = now;
     }
 
-    if (should_run_cpu_benchmarks() && !are_cpu_benchmarks_running()) {
+    if (should_run_cpu_benchmarks() && !benchmarks_running) {
         run_cpu_benchmarks = false;
         start_cpu_benchmarks();
     }
 
+#ifdef _WIN32
+    if (have_sysmon_msg) {
+        msg_printf(NULL, MSG_INFO, sysmon_msg);
+        have_sysmon_msg = false;
+    }
+#endif
+
     bool old_user_active = user_active;
     user_active = !host_info.users_idle(
         check_all_logins, global_prefs.idle_time_to_run
@@ -796,12 +851,15 @@ bool CLIENT_STATE::poll_slow_events() {
         //
         first = false;
         if (suspend_reason) {
-            print_suspend_tasks_message(suspend_reason);
+            msg_printf(NULL, MSG_INFO,
+                "Suspending computation - %s",
+                suspend_reason_string(suspend_reason)
+            );
         }
     }
     tasks_suspended = (suspend_reason != 0);
 
-    if (suspend_reason & SUSPEND_REASON_BENCHMARKS) {
+    if (benchmarks_running) {
         cpu_benchmarks_poll();
     }
 
@@ -1059,7 +1117,7 @@ int CLIENT_STATE::link_app_version(PROJECT* p, APP_VERSION* avp) {
         }
 
         if (!strcmp(file_ref.open_name, GRAPHICS_APP_FILENAME)) {
-            char relpath[512], path[512];
+            char relpath[MAXPATHLEN], path[MAXPATHLEN];
             get_pathname(fip, relpath, sizeof(relpath));
             relative_to_absolute(relpath, path);
             strlcpy(avp->graphics_exec_path, path, sizeof(avp->graphics_exec_path));
@@ -1214,7 +1272,7 @@ bool CLIENT_STATE::abort_unstarted_late_jobs() {
 bool CLIENT_STATE::garbage_collect() {
     bool action;
     static double last_time=0;
-    if (gstate.now - last_time < GARBAGE_COLLECT_PERIOD) return false;
+    if (!clock_change && now - last_time < GARBAGE_COLLECT_PERIOD) return false;
     last_time = gstate.now;
 
     action = abort_unstarted_late_jobs();
@@ -1415,7 +1473,8 @@ bool CLIENT_STATE::garbage_collect_always() {
 
     // go through APP_VERSIONs;
     // delete any not referenced by any WORKUNIT
-    // and superceded by a more recent version.
+    // and superceded by a more recent version
+    // for the same platform and plan class
     //
     avp_iter = app_versions.begin();
     while (avp_iter != app_versions.end()) {
@@ -1424,9 +1483,10 @@ bool CLIENT_STATE::garbage_collect_always() {
             found = false;
             for (j=0; j<app_versions.size(); j++) {
                 avp2 = app_versions[j];
-                if (avp2->app==avp->app
+                if (avp2->app == avp->app
+                    && avp2->version_num > avp->version_num
                     && (!strcmp(avp2->plan_class, avp->plan_class))
-                    && avp2->version_num>avp->version_num
+                    && (!strcmp(avp2->platform, avp->platform))
                 ) {
                     found = true;
                     break;
@@ -1520,8 +1580,8 @@ bool CLIENT_STATE::update_results() {
     static double last_time=0;
     int retval;
 
-    if (gstate.now - last_time < UPDATE_RESULTS_PERIOD) return false;
-    last_time = gstate.now;
+    if (!clock_change && now - last_time < UPDATE_RESULTS_PERIOD) return false;
+    last_time = now;
 
     result_iter = results.begin();
     while (result_iter != results.end()) {
@@ -1649,9 +1709,9 @@ int CLIENT_STATE::report_result_error(RESULT& res, const char* format, ...) {
     vsnprintf(err_msg, sizeof(err_msg), format, va);
     va_end(va);
 
-    sprintf(buf, "Unrecoverable error for task %s (%s)", res.name, err_msg);
+    sprintf(buf, "Unrecoverable error for task %s", res.name);
 #ifndef SIM
-    scheduler_op->backoff(res.project, buf);
+    scheduler_op->project_rpc_backoff(res.project, buf);
 #endif
 
     sprintf( buf, "<message>\n%s\n</message>\n", err_msg);
@@ -1722,12 +1782,11 @@ int CLIENT_STATE::report_result_error(RESULT& res, const char* format, ...) {
 // - stop all active tasks
 // - stop all file transfers
 // - stop scheduler RPC if any
-// - delete all workunits and results
-// - delete all apps and app_versions
+// - delete workunits and results
+// - delete apps and app_versions
 // - garbage collect to delete unneeded files
-// - clear debts and backoffs
+// - clear backoffs
 //
-// Note: does NOT delete persistent files or user-supplied files;
 // does not delete project dir
 //
 int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
@@ -1780,6 +1839,15 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
     project->user_files.clear();
     project->project_files.clear();
 
+    // clear flags so that sticky files get deleted
+    //
+    for (i=0; i<file_infos.size(); i++) {
+        FILE_INFO* fip = file_infos[i];
+        if (fip->project == project) {
+            fip->sticky = false;
+        }
+    }
+
     garbage_collect_always();
 
     // remove apps and app_versions (but not if anonymous platform)
@@ -1809,6 +1877,10 @@ int CLIENT_STATE::reset_project(PROJECT* project, bool detaching) {
         garbage_collect_always();
     }
 
+    // force refresh of scheduler URLs
+    //
+    project->scheduler_urls.clear();
+
     project->duration_correction_factor = 1;
     project->ams_resource_share = -1;
     project->min_rpc_time = 0;
@@ -1831,7 +1903,7 @@ int CLIENT_STATE::detach_project(PROJECT* project) {
     vector<FILE_INFO*>::iterator fi_iter;
     FILE_INFO* fip;
     PROJECT* p;
-    char path[256];
+    char path[MAXPATHLEN];
     int retval;
 
     reset_project(project, true);
@@ -1924,7 +1996,7 @@ int CLIENT_STATE::detach_project(PROJECT* project) {
 // e.g. flush buffers, but why bother)
 //
 int CLIENT_STATE::quit_activities() {
-    // calculate long-term debts (for state file)
+    // calculate REC (for state file)
     //
     adjust_rec();
 
@@ -1943,7 +2015,7 @@ int CLIENT_STATE::quit_activities() {
 
 #endif
 
-// See if a timestamp in the client state file
+// Called at startup to see if a timestamp in the client state file
 // is later than the current time.
 // If so, the user must have decremented the system clock.
 //
@@ -1965,6 +2037,8 @@ void CLIENT_STATE::clear_absolute_times() {
         "System clock was turned backwards; clearing timeouts"
     );
 
+    exclusive_app_running = 0;
+    exclusive_gpu_app_running = 0;
     new_version_check_time = now;
     all_projects_list_check_time = now;
     retry_shmem_time = 0;
