@@ -92,7 +92,7 @@ struct CLIENT_STATE {
     GET_PROJECT_LIST_OP get_project_list_op;
     ACCT_MGR_OP acct_mgr_op;
 
-    TIME_STATS time_stats;
+    CLIENT_TIME_STATS time_stats;
     GLOBAL_PREFS global_prefs;
     NET_STATS net_stats;
     ACTIVE_TASK_SET active_tasks;
@@ -140,6 +140,12 @@ struct CLIENT_STATE {
 #ifndef _WIN32
     gid_t boinc_project_gid;
 #endif
+#ifdef _WIN32
+    // vars so that the sysmon thread can write messages
+    //
+    bool have_sysmon_msg;
+    char sysmon_msg[256];
+#endif
 
     // backoff-related variables
     //
@@ -182,10 +188,9 @@ struct CLIENT_STATE {
         // not be displayable, so GUIs shouldn't offer graphics.
     bool detach_console;
     bool launched_by_manager;
-        // this affects auto-update
     bool run_by_updater;
     double now;
-    double client_start_time;
+    bool clock_change;      // system clock was recently decreased
     double last_wakeup_time;
     bool initialized;
     bool cant_write_state_file;
@@ -270,14 +275,14 @@ struct CLIENT_STATE {
     double potentially_runnable_resource_share();
     double nearly_runnable_resource_share();
     double fetchable_resource_share();
-    double debt_interval_start;
-    double total_cpu_time_this_debt_interval;
+    double rec_interval_start;
+    double total_cpu_time_this_rec_interval;
     bool must_enforce_cpu_schedule;
     bool must_schedule_cpus;
     bool must_check_work_fetch;
     void assign_results_to_projects();
-    RESULT* largest_debt_project_best_result();
-    void reset_debt_accounting();
+    RESULT* highest_prio_project_best_result();
+    void reset_rec_accounting();
     bool schedule_cpus();
     void make_run_list(vector<RESULT*>&);
     bool enforce_run_list(vector<RESULT*>&);
@@ -346,11 +351,12 @@ struct CLIENT_STATE {
     ACTIVE_TASK* get_task(RESULT*);
 
 // --------------- cs_benchmark.cpp:
+    bool benchmarks_running;
+
     bool should_run_cpu_benchmarks();
     void start_cpu_benchmarks();
     bool cpu_benchmarks_poll();
     void abort_cpu_benchmarks();
-    bool are_cpu_benchmarks_running();
     bool cpu_benchmarks_done();
     void cpu_benchmarks_set_defaults();
     void print_benchmark_results();
@@ -379,6 +385,7 @@ struct CLIENT_STATE {
         // disk usage not counting projects
         // computed by get_disk_usages()
     double total_disk_usage;
+        // client plus projects
     int get_disk_usages();
     void get_disk_shares();
     double allowed_disk_usage(double boinc_total);
@@ -465,6 +472,7 @@ struct CLIENT_STATE {
     int proj_min_results(PROJECT*, double);
     void check_project_timeout();
     double overall_cpu_frac();
+    double overall_cpu_and_network_frac();
     double overall_gpu_frac();
     double time_until_work_done(PROJECT*, int, double);
     bool compute_work_requests();
@@ -494,9 +502,6 @@ extern double calculate_exponential_backoff(
     int n, double MIN, double MAX
 );
 
-extern void print_suspend_tasks_message(int);
-
-
 //////// TIME-RELATED CONSTANTS ////////////
 
 //////// CLIENT INTERNAL
@@ -542,19 +547,23 @@ extern void print_suspend_tasks_message(int);
     // we'll find out about it within a day.
 
 #define WF_DEFER_INTERVAL   300
-    // if a project is uploading, and the last upload started within this interval,
+    // if a project is uploading,
+    // and the last upload started within this interval,
     // don't fetch work from it.
     // This allows the work fetch to be merged with the reporting of the
     // jobs that are currently uploading.
 
+#define RESULT_REPORT_IF_AT_LEAST_N 64
+    // If a project has at least this many ready-to-report tasks, report them.
+
 //////// CPU SCHEDULING
 
 #define CPU_SCHED_PERIOD    60
     // do CPU schedule at least this often
 
-#define DEBT_ADJUST_PERIOD CPU_SCHED_PERIOD
-    // debt is adjusted at least this often,
-    // since adjust_debts() is called from enforce_schedule()
+#define REC_ADJUST_PERIOD CPU_SCHED_PERIOD
+    // REC is adjusted at least this often,
+    // since adjust_rec() is called from enforce_schedule()
 
 #define DEADLINE_CUSHION    0
     // try to finish jobs this much in advance of their deadline
@@ -582,9 +591,6 @@ extern void print_suspend_tasks_message(int);
 
 #define DAILY_XFER_HISTORY_PERIOD   60
 
-#define MAX_STD   (86400)
-    // maximum short-term debt
-
 #define ACCT_MGR_MIN_BACKOFF    600
 #define ACCT_MGR_MAX_BACKOFF    86400
     // min/max account manager RPC backoff
