@@ -511,7 +511,7 @@ int FILE_INFO::write_gui(MIOFILE& out) {
 // delete physical underlying file associated with FILE_INFO
 //
 int FILE_INFO::delete_file() {
-    char path[1024];
+    char path[MAXPATHLEN];
 
     get_pathname(this, path, sizeof(path));
     int retval = delete_project_owned_file(path, true);
@@ -653,7 +653,7 @@ void FILE_INFO::failure_message(string& s) {
 #define BUFSIZE 16384
 int FILE_INFO::gzip() {
     char buf[BUFSIZE];
-    char inpath[1024], outpath[1024];
+    char inpath[MAXPATHLEN], outpath[MAXPATHLEN];
 
     get_pathname(this, inpath, sizeof(inpath));
     strcpy(outpath, inpath);
@@ -682,7 +682,7 @@ int FILE_INFO::gzip() {
 //
 int FILE_INFO::gunzip(char* md5_buf) {
     unsigned char buf[BUFSIZE];
-    char inpath[1024], outpath[1024], tmppath[1024];
+    char inpath[MAXPATHLEN], outpath[MAXPATHLEN], tmppath[MAXPATHLEN];
     md5_state_t md5_state;
 
     md5_init(&md5_state);
@@ -811,6 +811,9 @@ int APP_VERSION::parse(XML_PARSER& xp) {
             if (!retval) {
                 rt = rsc_index(cp.type);
                 if (rt <= 0) {
+                    msg_printf(0, MSG_INFO,
+                        "app version refers to missing GPU type %s", cp.type
+                    );
                     missing_coproc = true;
                     missing_coproc_usage = cp.count;
                     strcpy(missing_coproc_name, cp.type);
@@ -874,14 +877,12 @@ int APP_VERSION::write(MIOFILE& out, bool write_file_info) {
         }
     }
     if (gpu_usage.rsc_type) {
-        const char* p = rsc_name(gpu_usage.rsc_type);
-        if (!strcmp(p, "NVIDIA")) p = "CUDA";
         out.printf(
             "    <coproc>\n"
             "        <type>%s</type>\n"
             "        <count>%f</count>\n"
             "    </coproc>\n",
-            p,
+            rsc_name(gpu_usage.rsc_type),
             gpu_usage.usage
         );
     }
