@@ -28,15 +28,17 @@
 #if !defined(_WIN32) || defined(__CYGWIN32__)
 #include <cstdio>
 #include <sys/time.h>
+#include <sys/param.h>
 #endif
 
-#include "md5_file.h"
-#include "cert_sig.h"
-#include "hostinfo.h"
-#include "coproc.h"
-#include "miofile.h"
-#include "common_defs.h"
 #include "cc_config.h"
+#include "common_defs.h"
+#include "coproc.h"
+#include "cert_sig.h"
+#include "filesys.h"
+#include "hostinfo.h"
+#include "md5_file.h"
+#include "miofile.h"
 
 #include "cs_notice.h"
 #include "cs_trickle.h"
@@ -91,19 +93,9 @@ struct URL_LIST {
     }
 };
 
-// Values of FILE_INFO::status.
-// If the status is neither of these two,
-// it's an error code indicating an unrecoverable error
-// in the transfer of the file,
-// or that the file was too big and was deleted.
-//
-#define FILE_NOT_PRESENT    0
-#define FILE_PRESENT        1
-#define FILE_VERIFY_PENDING	2
-
 struct FILE_INFO {
     char name[256];
-    char md5_cksum[33];
+    char md5_cksum[MD5_LEN];
     double max_nbytes;
     double nbytes;
     double gzipped_nbytes;  // defined if download_gzipped is true
@@ -252,6 +244,13 @@ struct APP {
     char user_friendly_name[256];
     bool non_cpu_intensive;
     PROJECT* project;
+    int max_concurrent;
+        // Limit on # of concurrent jobs of this app; 0 if none
+        // Specified in app_config.xml
+    int n_concurrent;
+        // temp during job scheduling, to enforce max_concurrent
+    int non_excluded_instances[MAX_RSC];
+        // for each resources type, the non-excluded instances
 #ifdef SIM
     double latency_bound;
     double fpops_est;
@@ -260,9 +259,9 @@ struct APP {
     double working_set;
     double weight;
     bool ignore;
-    APP() {memset(this, 0, sizeof(APP));}
 #endif
 
+    APP() {memset(this, 0, sizeof(APP));}
     int parse(XML_PARSER&);
     int write(MIOFILE&);
 };
@@ -294,7 +293,7 @@ struct APP_VERSION {
     PROJECT* project;
     std::vector<FILE_REF> app_files;
     int ref_cnt;
-    char graphics_exec_path[512];
+    char graphics_exec_path[MAXPATHLEN];
     char graphics_exec_file[256];
     double max_working_set_size;
         // max working set of tasks using this app version.
