@@ -117,7 +117,6 @@ struct BENCHMARK_DESC {
 };
 
 static BENCHMARK_DESC* benchmark_descs=0;
-static bool benchmarks_running=false;    // at least 1 benchmark thread running
 static double cpu_benchmarks_start;
 static int bm_ncpus;
     // user might change ncpus during benchmarks.
@@ -359,14 +358,14 @@ bool CLIENT_STATE::cpu_benchmarks_poll() {
     static double last_time = 0;
     if (!benchmarks_running) return false;
 
-    if (now < last_time + BENCHMARK_POLL_PERIOD) return false;
+    if (!clock_change && now < last_time + BENCHMARK_POLL_PERIOD) return false;
     last_time = now;
 
     active_tasks.send_heartbeats();
 
     // if active tasks don't quit after 10 sec, give up on benchmark
     //
-    if (now >= (cpu_benchmarks_start + 10.0) && active_tasks.is_task_executing()) {
+    if (gstate.clock_change || now >= (cpu_benchmarks_start + 10.0) && active_tasks.is_task_executing()) {
         msg_printf(NULL, MSG_INTERNAL_ERROR,
             "Failed to stop applications; aborting CPU benchmarks"
         );
@@ -560,10 +559,3 @@ void CLIENT_STATE::cpu_benchmarks_set_defaults() {
     if (!host_info.p_membw) host_info.p_membw = DEFAULT_MEMBW;
     if (!host_info.m_cache) host_info.m_cache = DEFAULT_CACHE;
 }
-
-// return true if any CPU benchmark thread/process is running
-//
-bool CLIENT_STATE::are_cpu_benchmarks_running() {
-    return benchmarks_running;
-}
-
