@@ -90,12 +90,12 @@ int CLIENT_STATE::make_project_dirs() {
 // Is app signed by one of the Application Certifiers?
 //
 bool FILE_INFO::verify_file_certs() {
-    char file[256];
+    char file[MAXPATHLEN];
     bool retval = false;
 
     if (!is_dir(CERTIFICATE_DIRECTORY)) return false;
     DIRREF dir = dir_open(CERTIFICATE_DIRECTORY);
-    while (dir_scan(file, dir, sizeof(file))) {
+    while (!dir_scan(file, dir, sizeof(file))) {
         if (cert_verify_file(cert_sigs, file, CERTIFICATE_DIRECTORY)) {
             msg_printf(project, MSG_INFO,
                 "Signature verified using certificate %s", file
@@ -156,7 +156,7 @@ bool FILE_INFO::verify_file_certs() {
 int FILE_INFO::verify_file(
     bool verify_contents, bool show_errors, bool allow_async
 ) {
-    char cksum[64], pathname[256];
+    char cksum[64], pathname[MAXPATHLEN];
     bool verified;
     int retval;
     double size, local_nbytes;
@@ -176,7 +176,7 @@ int FILE_INFO::verify_file(
     // see if we need to unzip it
     //
     if (download_gzipped && !boinc_file_exists(pathname)) {
-        char gzpath[256];
+        char gzpath[MAXPATHLEN];
         sprintf(gzpath, "%s.gz", pathname);
         if (boinc_file_exists(gzpath) ) {
 			if (allow_async && nbytes > ASYNC_FILE_THRESHOLD) {
@@ -348,7 +348,7 @@ bool CLIENT_STATE::create_and_delete_pers_file_xfers() {
     int retval;
     static double last_time;
 
-    if (now - last_time < PERS_FILE_XFER_START_PERIOD) return false;
+    if (!clock_change && now - last_time < PERS_FILE_XFER_START_PERIOD) return false;
     last_time = now;
 
     // Look for FILE_INFOs for which we should start a transfer,
@@ -401,7 +401,7 @@ bool CLIENT_STATE::create_and_delete_pers_file_xfers() {
                 // If this was a compressed download, rename .gzt to .gz
                 //
                 if (fip->download_gzipped) {
-                    char path[256], from_path[256], to_path[256];
+                    char path[MAXPATHLEN], from_path[MAXPATHLEN], to_path[MAXPATHLEN];
                     get_pathname(fip, path, sizeof(path));
                     sprintf(from_path, "%s.gzt", path);
                     sprintf(to_path, "%s.gz", path);
@@ -454,23 +454,39 @@ bool CLIENT_STATE::create_and_delete_pers_file_xfers() {
 }
 #endif
 
-// called at startup to ensure that if the core client
+// called at startup to ensure that if the client
 // thinks a file is there, it actually is, and is the right size
 //
 void CLIENT_STATE::check_file_existence() {
     unsigned int i;
-    char path[1024];
+    char path[MAXPATHLEN];
 
     for (i=0; i<file_infos.size(); i++) {
         FILE_INFO* fip = file_infos[i];
+        if (fip->status < 0 && fip->downloadable()) {
+            // file had an error; reset it so that we download again
+            get_pathname(fip, path, sizeof(path));
+            msg_printf(NULL, MSG_INFO, "Resetting file %s: %s", path, boincerror(fip->status));
+            fip->reset();
+            continue;
+        }
+        if (config.dont_check_file_sizes) continue;
         if (fip->status == FILE_PRESENT) {
             get_pathname(fip, path, sizeof(path));
             double size;
             int retval = file_size(path, size);
-            if (retval || (fip->nbytes && (size != fip->nbytes))) {
+            if (retval) {
                 delete_project_owned_file(path, true);
                 fip->status = FILE_NOT_PRESENT;
-                msg_printf(NULL, MSG_INFO, "file %s not found", path);
+                msg_printf(NULL, MSG_INFO, "File %s not found", path);
+            } else if (fip->nbytes && (size != fip->nbytes)) {
+                if (gstate.global_prefs.dont_verify_images && is_image_file(path)) continue;
+                delete_project_owned_file(path, true);
+                fip->status = FILE_NOT_PRESENT;
+                msg_printf(NULL, MSG_INFO,
+                    "File %s has wrong size: expected %.0f, got %.0f",
+                    path, fip->nbytes, size
+                );
             }
         }
     }
