@@ -96,8 +96,6 @@ void CLIENT_STATE::detect_platforms() {
     }
     add_platform("windows_intelx86");
 #endif
-#elif defined(ANDROID)
-    add_platform("arm-android");
 
 #elif defined(__APPLE__)
 
@@ -318,20 +316,21 @@ void CLIENT_STATE::detect_platforms() {
 // write XML list of supported platforms
 //
 void CLIENT_STATE::write_platforms(PROJECT* p, MIOFILE& mf) {
-
-    mf.printf(
-        "    <platform_name>%s</platform_name>\n",
-        p->anonymous_platform ? "anonymous" : get_primary_platform()
-    );
-
-    for (unsigned int i=1; i<platforms.size(); i++) {
-        PLATFORM& platform = platforms[i];
+    if (p->anonymous_platform) {
+        mf.printf("    <platform_name>anonymous</platform_name>\n");
+    } else {
         mf.printf(
-            "    <alt_platform>\n"
-            "        <name>%s</name>\n"
-            "    </alt_platform>\n",
-            platform.name.c_str()
+            "    <platform_name>%s</platform_name>\n", get_primary_platform()
         );
+        for (unsigned int i=1; i<platforms.size(); i++) {
+            PLATFORM& platform = platforms[i];
+            mf.printf(
+                "    <alt_platform>\n"
+                "        <name>%s</name>\n"
+                "    </alt_platform>\n",
+                platform.name.c_str()
+            );
+        }
     }
 }
 
