@@ -80,7 +80,7 @@ void GUI_RPC_CONN::handle_auth1(MIOFILE& fout) {
 }
 
 int GUI_RPC_CONN::handle_auth2(char* buf, MIOFILE& fout) {
-    char nonce_hash[256], nonce_hash_correct[256], buf2[256];
+    char nonce_hash[256], nonce_hash_correct[256], buf2[512];
     if (!parse_str(buf, "<nonce_hash>", nonce_hash, 256)) {
         auth_failure(fout);
         return ERR_AUTHENTICATOR;
@@ -137,7 +137,15 @@ static void handle_get_disk_usage(GUI_RPC_CONN& grc) {
     double size, boinc_non_project, d_allowed, boinc_total;
 
     grc.mfout.printf("<disk_usage_summary>\n");
-    get_filesystem_info(gstate.host_info.d_total, gstate.host_info.d_free);
+    int retval = get_filesystem_info(
+        gstate.host_info.d_total, gstate.host_info.d_free
+    );
+    if (retval) {
+        msg_printf(0, MSG_INTERNAL_ERROR,
+            "get_filesystem_info(): %s", boincerror(retval)
+        );
+    }
+
     dir_size(".", boinc_non_project, false);
     dir_size("locale", size, false);
     boinc_non_project += size;
@@ -146,7 +154,7 @@ static void handle_get_disk_usage(GUI_RPC_CONN& grc) {
         // If launched by Manager, get Manager's size on disk
         ProcessSerialNumber managerPSN;
         FSRef ourFSRef;
-        char path[1024];
+        char path[MAXPATHLEN];
         double manager_size = 0.0;
         OSStatus err;
         err = GetProcessForPID(getppid(), &managerPSN);
@@ -828,7 +836,7 @@ static void handle_project_attach(GUI_RPC_CONN& grc) {
 
     // if project_init.xml refers to this project,
     // delete the file, otherwise we'll just
-    // reattach the next time the core client starts
+    // reattach the next time the client starts
     //
     if (!strcmp(url.c_str(), gstate.project_init.url)) {
         retval = gstate.project_init.remove();
@@ -1097,6 +1105,11 @@ static void handle_read_cc_config(GUI_RPC_CONN& grc) {
     log_flags.show();
     gstate.set_ncpus();
     process_gpu_exclusions();
+
+    // also reread app_config.xml files
+    //
+    check_app_config();
+
     gstate.request_schedule_cpus("Core client configuration");
     gstate.request_work_fetch("Core client configuration");
     set_no_rsc_config();
