@@ -50,11 +50,44 @@
 #include "parse.h"
 #include "util.h"
 #include "file_names.h"
-#include "client_msgs.h"
 #include "error_numbers.h"
 
+#include "client_msgs.h"
+
 #include "hostinfo.h"
 
+#ifdef ANDROID
+// Returns TRUE if host is currently using a wifi connection
+// used on Android devices to prevent usage of data plans.
+// if value cant be read, default return false
+//
+bool HOST_INFO::host_wifi_online() {
+    char wifi_state[64];
+
+    strcpy(wifi_state, "");
+
+    // location in Android 2.3
+    FILE *f = fopen("/sys/class/net/eth0/operstate", "r");
+    if (!f) {
+        // location in Android 4
+        f = fopen("/sys/class/net/wlan0/operstate", "r");
+    }
+
+    if (f) {
+        fgets(wifi_state, 64, f);
+        fclose(f);
+    } else {
+        msg_printf(0, MSG_INFO, "HOST_INFO::host_wifi_online(): wifi adapter not found!\n");
+        return false;
+    }
+
+    if (strstr(wifi_state,"up")) {
+        return true;
+    }
+    return false;
+}
+#endif //ANDROID
+
 // get domain name and IP address of this host
 //
 int HOST_INFO::get_local_network_info() {
@@ -93,9 +126,15 @@ int HOST_INFO::get_local_network_info() {
 void HOST_INFO::make_random_string(const char* salt, char* out) {
     char buf[1024];
 
+#ifdef ANDROID
+    sprintf(buf, "%f%s%s%f%s",
+        dtime(), domain_name, ip_addr, d_free, salt
+    );
+#else
     sprintf(buf, "%d%.15e%s%s%f%s",
         getpid(), dtime(), domain_name, ip_addr, d_free, salt
     );
+#endif
     md5_block((const unsigned char*) buf, (int)strlen(buf), out);
 }
 
