@@ -89,6 +89,9 @@
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#if HAVE_SYS_SENSORS_H
+#include <sys/sensors.h>
+#endif
 
 #ifdef __EMX__
 #define INCL_DOSMISC
@@ -97,6 +100,7 @@
 #endif
 
 #include "error_numbers.h"
+#include "common_defs.h"
 #include "filesys.h"
 #include "str_util.h"
 #include "str_replace.h"
@@ -150,10 +154,15 @@ mach_port_t gEventHandle = NULL;
 #define _SC_PAGESIZE _SC_PAGE_SIZE
 #endif
 
+#if HAVE_DPMS
+#include <X11/extensions/dpms.h>
+#endif
+
 #if HAVE_XSS
 #include <X11/extensions/scrnsaver.h>
 #endif
 
+
 // The following is intended to be true both on Linux
 // and Debian GNU/kFreeBSD (see trac #521)
 //
@@ -212,6 +221,34 @@ bool HOST_INFO::host_is_running_on_batteries() {
     CFRelease(list);
     return retval;
 
+#elif ANDROID
+    // using /sys/class/power_supply/*/online
+    // power supplies are both ac and usb!
+    //
+    static FILE *fsysac, *fsysusb;
+    int aconline = 0;
+    int usbonline = 0;
+    bool power_supply_online = false;
+
+    fsysac = fopen("/sys/class/power_supply/ac/online", "r");
+    fsysusb = fopen("/sys/class/power_supply/usb/online", "r");
+
+    if (fsysac) {
+        (void) fscanf(fsysac, "%d", &aconline);
+        fclose(fsysac);
+    }
+
+    if (fsysusb) {
+        (void) fscanf(fsysusb, "%d", &usbonline);
+        fclose(fsysusb);
+    }
+
+    if ((aconline == 1) || (usbonline == 1)) {
+        power_supply_online = true;
+    }
+
+    return !power_supply_online;
+
 #elif LINUX_LIKE_SYSTEM
     static enum {
       Detect,
@@ -354,11 +391,173 @@ bool HOST_INFO::host_is_running_on_batteries() {
          // so we say we aren't
         return false;
     }
+#elif defined(__OpenBSD__)
+    static int mib[] = {CTL_HW, HW_SENSORS, 0, 0, 0};
+    static int devn = -1;
+    struct sensor s;
+    size_t slen = sizeof(struct sensor);
+
+    if (devn == -1) {
+        struct sensordev snsrdev;
+        size_t sdlen = sizeof(struct sensordev);
+        for (devn = 0;; devn++) {
+            mib[2] = devn;
+            if (sysctl(mib, 3, &snsrdev, &sdlen, NULL, 0) == -1) {
+                if (errno == ENXIO)
+                    continue;
+                if (errno == ENOENT)
+                    break;
+            }
+            if (!strcmp("acpiac0", snsrdev.xname)) {
+                break;
+            }
+        }
+        mib[3] = 9;
+        mib[4] = 0;
+    }
+
+    if (sysctl(mib, 5, &s, &slen, NULL, 0) != -1) {
+        if (s.value)
+            // AC present
+            return false;
+        else
+            return true;
+    }
+
+    return false;
+#elif defined(__FreeBSD__)
+    int ac;
+    size_t len = sizeof(ac);
+
+    if (sysctlbyname("hw.acpi.acline", &ac, &len, NULL, 0) != -1) {
+        if (ac)
+            // AC present
+            return false;
+        else
+            return true;
+    }
+
+    return false;
 #else
     return false;
 #endif
 }
 
+#ifdef ANDROID
+
+// Get battery state, charge percentage, and temperature
+//
+
+static const char* battery_dirs[] = {
+    "battery",
+    "BAT0",
+    "bq27520",
+    "bq27200-0",
+    "max17042-0",
+    "ds2760-battery.0",
+    NULL
+};
+
+void HOST_INFO::get_battery_status() {
+    static bool first = true;
+    static FILE *fcap, *fhealth, *fstatus, *ftemp;
+    FILE *f;
+    battery_charge_pct = -1;
+    battery_temperature_celsius = 0;
+    char buf[256];
+    char health[256];
+    char status[256];
+
+    strcpy(health, "");
+    strcpy(status, "");
+
+    if (first) {
+        first = false;
+        int i = 0;
+
+        // Find the battery location for this device.
+        // matszpk has already collected a bunch of battery locations.
+        //
+        for (i = 0; battery_dirs[i] != NULL; i++) {
+            snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/present",
+                battery_dirs[i]
+            );
+            f = fopen(buf, "r");
+            if (f != NULL) {
+                fclose(f);
+                break;
+            }
+        }
+
+        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/capacity",
+            battery_dirs[i]
+        );
+        fcap = fopen(buf, "r");
+        if (fcap) setbuf(fcap, NULL);
+
+        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/health",
+            battery_dirs[i]
+        );
+        fhealth = fopen(buf, "r");
+        if (fhealth) setbuf(fhealth, NULL);
+
+        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/status",
+            battery_dirs[i]
+        );
+        fstatus = fopen(buf, "r");
+        if (fstatus) setbuf(fstatus, NULL);
+
+        snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/temp",
+            battery_dirs[i]
+        );
+        ftemp = fopen(buf, "r");
+        if (ftemp) {
+            setbuf(ftemp, NULL);
+        } else {
+            snprintf(buf, sizeof(buf), "/sys/class/power_supply/%s/batt_temp",
+                battery_dirs[i]
+            );
+            ftemp = fopen(buf, "r");
+            if (ftemp) setbuf(ftemp, NULL);
+        }
+    }
+
+    if (fcap) {
+        rewind(fcap);
+        fscanf(fcap, "%d", &battery_charge_pct);
+    }
+
+    if (fhealth) {
+        rewind(fhealth);
+        fgets(health, sizeof(health), fhealth);
+    }
+
+    if (fstatus) {
+        rewind(fstatus);
+        fgets(status, sizeof(status), fstatus);
+    }
+
+    battery_state = BATTERY_STATE_UNKNOWN;
+    if (strstr(health, "Overheat")) {
+        battery_state = BATTERY_STATE_OVERHEATED;
+    } else if (strstr(status, "Not charging")) {
+        battery_state = BATTERY_STATE_DISCHARGING;
+    } else if (strstr(status, "Charging")) {
+        battery_state = BATTERY_STATE_CHARGING;
+    } else if (strstr(status, "Full")) {
+        battery_state = BATTERY_STATE_FULL;
+    }
+
+    if (ftemp) {
+        int x;
+        rewind(ftemp);
+        if (fscanf(ftemp, "%d", &x) == 1) {
+            battery_temperature_celsius = x/10.;
+        }
+    }
+}
+#endif
+
 #if LINUX_LIKE_SYSTEM
 static void parse_meminfo_linux(HOST_INFO& host) {
     char buf[256];
@@ -422,6 +621,9 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
 #elif __ia64__
     strcpy(host.p_model, "IA-64 ");
     model_hack = true;
+#elif __arm__
+    strcpy(host.p_vendor, "ARM");
+    vendor_hack = vendor_found = true;
 #endif
 
     host.m_cache=-1;
@@ -460,6 +662,8 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
             strstr(buf, "family     : ") || strstr(buf, "model name : ")
 #elif __powerpc__ || __sparc__
             strstr(buf, "cpu\t\t: ")
+#elif __arm__
+            strstr(buf, "Processor\t: ")
 #else
             strstr(buf, "model name\t: ") || strstr(buf, "cpu model\t\t: ")
 #endif
@@ -545,6 +749,8 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
                 strlcpy(features, strchr(buf, ':') + 2, sizeof(features));
             } else if ((strstr(buf, "features   : ") == buf)) {    /* ia64 */
                 strlcpy(features, strchr(buf, ':') + 2, sizeof(features));
+            } else if ((strstr(buf, "Features\t: ") == buf)) { /* arm */
+               strlcpy(features, strchr(buf, ':') + 2, sizeof(features));
             }
             if (strlen(features)) {
                 features_found = true;
@@ -584,9 +790,10 @@ static void parse_cpuinfo_linux(HOST_INFO& host) {
 
 void use_cpuid(HOST_INFO& host) {
     u_int p[4];
-    int hasMMX, hasSSE, hasSSE2, hasSSE3, has3DNow, has3DNowExt = 0;
+    int hasMMX, hasSSE, hasSSE2, hasSSE3, has3DNow, has3DNowExt;
     char capabilities[256];
 
+    hasMMX = hasSSE = hasSSE2 = hasSSE3 = has3DNow = has3DNowExt = 0;
     do_cpuid(0x0, p);
 
     if (p[0] >= 0x1) {
@@ -608,12 +815,12 @@ void use_cpuid(HOST_INFO& host) {
     }
 
     capabilities[0] = '\0';
-    if (hasSSE) strncat(capabilities, "sse ", 4);
-    if (hasSSE2) strncat(capabilities, "sse2 ", 5);
-    if (hasSSE3) strncat(capabilities, "sse3 ", 5);
-    if (has3DNow) strncat(capabilities, "3dnow ", 6);
-    if (has3DNowExt) strncat(capabilities, "3dnowext ", 9);
-    if (hasMMX) strncat(capabilities, "mmx ", 4);
+    if (hasSSE) strcat(capabilities, "sse ");
+    if (hasSSE2) strcat(capabilities, "sse2 ");
+    if (hasSSE3) strcat(capabilities, "sse3 ");
+    if (has3DNow) strcat(capabilities, "3dnow ");
+    if (has3DNowExt) strcat(capabilities, "3dnowext ");
+    if (hasMMX) strcat(capabilities, "mmx ");
     strip_whitespace(capabilities);
     char buf[1024];
     snprintf(buf, sizeof(buf), "%s [] [%s]",
@@ -1111,6 +1318,15 @@ int get_max_cpu_temperature() {
 }
 
 #endif
+
+// Is this a dual GPU MacBook with automatic GPU switching?
+bool isDualGPUMacBook() {
+    io_service_t service = IO_OBJECT_NULL;
+    
+    service = IOServiceGetMatchingService(kIOMasterPortDefault, IOServiceMatching("AppleGraphicsControl"));
+    return (service != IO_OBJECT_NULL);
+}
+
 #endif  // __APPLE__
 
 // see if Virtualbox is installed
@@ -1142,6 +1358,9 @@ int HOST_INFO::get_virtualbox_version() {
 
     if (boinc_file_exists(path)) {
 #if LINUX_LIKE_SYSTEM
+        if (access(path, X_OK)) {
+            return 0;
+        }
         safe_strcpy(cmd, path);
         safe_strcat(cmd, " --version");
 #elif defined( __APPLE__)
@@ -1169,7 +1388,12 @@ int HOST_INFO::get_virtualbox_version() {
 // - only one level of #if
 //
 int HOST_INFO::get_host_info() {
-    get_filesystem_info(d_total, d_free);
+    int retval = get_filesystem_info(d_total, d_free);
+    if (retval) {
+        msg_printf(0, MSG_INTERNAL_ERROR,
+            "get_filesystem_info() failed: %s", boincerror(retval)
+        );
+    }
     get_virtualbox_version();
 
 ///////////// p_vendor, p_model, p_features /////////////////
@@ -1285,7 +1509,7 @@ int HOST_INFO::get_host_info() {
     m_nbytes = (double)sysconf(_SC_PAGESIZE) * (double)sysconf(_SC_PHYS_PAGES);
     if (m_nbytes < 0) {
         msg_printf(NULL, MSG_INTERNAL_ERROR,
-            "RAM size not measured correctly: page size %d, #pages %d",
+            "RAM size not measured correctly: page size %ld, #pages %ld",
             sysconf(_SC_PAGESIZE), sysconf(_SC_PHYS_PAGES)
         );
     }
@@ -1312,15 +1536,8 @@ int HOST_INFO::get_host_info() {
     getsysinfo( GSI_PHYSMEM, (caddr_t) &mem_size, sizeof( mem_size));
     m_nbytes = 1024.* (double)mem_size;
 #elif defined(HW_PHYSMEM) 
-    // for OpenBSD
-    mib[0] = CTL_HW; 
-    int mem_size; 
-    mib[1] = HW_PHYSMEM; 
-    len = sizeof(mem_size); 
-    sysctl(mib, 2, &mem_size, &len, NULL, 0); 
-    m_nbytes = mem_size; 
-#elif defined(__FreeBSD__)
-    unsigned int mem_size;
+    // for OpenBSD & NetBSD & FreeBSD
+    int mem_size;
     mib[0] = CTL_HW;
     mib[1] = HW_PHYSMEM;
     len = sizeof(mem_size);
@@ -1409,7 +1626,11 @@ int HOST_INFO::get_host_info() {
 #if HAVE_SYS_UTSNAME_H
     struct utsname u;
     uname(&u);
+#ifdef ANDROID
+    safe_strcpy(os_name, "Android");
+#else
     safe_strcpy(os_name, u.sysname);
+#endif //ANDROID
 #if defined(__EMX__) // OS2: version is in u.version
     safe_strcpy(os_version, u.version);
 #elif defined(__HAIKU__)
@@ -1704,6 +1925,7 @@ bool HOST_INFO::users_idle(
 
 #if LINUX_LIKE_SYSTEM
 bool interrupts_idle(time_t t) {
+    // This method doesn't really work reliably on USB keyboards and mice.
     static FILE *ifp = NULL;
     static long irq_count[256];
     static time_t last_irq = time(NULL);
@@ -1747,7 +1969,51 @@ bool xss_idle(long idle_treshold) {
     
     if(disp != NULL) {
         XScreenSaverQueryInfo(disp, DefaultRootWindow(disp), xssInfo);
-        idle_time = xssInfo->idle / 1000; // xssInfo->idle is in ms
+
+        idle_time = xssInfo->idle;
+
+#if HAVE_DPMS
+        int dummy;
+        CARD16 standby, suspend, off;
+        CARD16 state;
+        BOOL onoff;
+
+        if (DPMSQueryExtension(disp, &dummy, &dummy)) {
+            if (DPMSCapable(disp)) {
+                DPMSGetTimeouts(disp, &standby, &suspend, &off);
+                DPMSInfo(disp, &state, &onoff);
+
+                if (onoff) {
+                    switch (state) {
+                      case DPMSModeStandby:
+                          /* this check is a littlebit paranoid, but be sure */
+                          if (idle_time < (unsigned) (standby * 1000)) {
+                              idle_time += (standby * 1000);
+                          }
+                          break;
+                      case DPMSModeSuspend:
+                          if (idle_time < (unsigned) ((suspend + standby) * 1000)) {
+                              idle_time += ((suspend + standby) * 1000);
+                          }
+                          break;
+                      case DPMSModeOff:
+                          if (idle_time < (unsigned) ((off + suspend + standby) * 1000)) {
+                              idle_time += ((off + suspend + standby) * 1000);
+                          }
+                          break;
+                      case DPMSModeOn:
+                        default:
+                          break;
+                    }
+                }
+
+            } 
+        }
+#endif
+
+        // convert from milliseconds to seconds
+        idle_time = idle_time / 1000;
+
     } else {
         disp = XOpenDisplay(NULL);
         // XOpenDisplay may return NULL if there is no running X
@@ -1779,10 +2045,17 @@ bool HOST_INFO::users_idle(bool check_all_logins, double idle_time_to_run) {
 
 #if LINUX_LIKE_SYSTEM
     // Check /proc/interrupts to detect keyboard or mouse activity.
+    // this ignores USB keyboards/mice.  They don't use the keyboard
+    // and mouse interrupts.
     if (!interrupts_idle(idle_time)) {
         return false;
     }
 
+    // Lets at least check the dev entries which should be correct for
+    // USB mice.  The tty check will catch keyboards if they are entering
+    // data into a tty.
+    if (!device_idle(idle_time, "/dev/input/mice")) return false;
+
 #if HAVE_XSS
     if (!xss_idle((long)(idle_time_to_run * 60))) {
         return false;
