@@ -28,6 +28,7 @@
 #endif
 
 #include "error_numbers.h"
+#include "common_defs.h"
 #include "filesys.h"
 #include "str_util.h"
 #include "str_replace.h"
@@ -292,8 +293,57 @@
 #ifndef PRODUCT_ENTERPRISE_N_EVALUATION
 #define PRODUCT_ENTERPRISE_N_EVALUATION             0x00000054
 #endif
-
-
+#ifndef PRODUCT_EMBEDDED_AUTOMOTIVE
+#define PRODUCT_EMBEDDED_AUTOMOTIVE					0x00000055
+#endif
+#ifndef PRODUCT_EMBEDDED_INDUSTRY_A
+#define PRODUCT_EMBEDDED_INDUSTRY_A					0x00000056
+#endif
+#ifndef PRODUCT_THINPC								
+#define PRODUCT_THINPC								0x00000057
+#endif
+#ifndef PRODUCT_EMBEDDED_A
+#define PRODUCT_EMBEDDED_A							0x00000058
+#endif
+#ifndef PRODUCT_EMBEDDED_INDUSTRY
+#define PRODUCT_EMBEDDED_INDUSTRY					0x00000059
+#endif
+#ifndef PRODUCT_EMBEDDED_E
+#define PRODUCT_EMBEDDED_E                          0x0000005A
+#endif
+#ifndef PRODUCT_EMBEDDED_INDUSTRY_E
+#define PRODUCT_EMBEDDED_INDUSTRY_E                 0x0000005B
+#endif
+#ifndef PRODUCT_EMBEDDED_INDUSTRY_A_E
+#define PRODUCT_EMBEDDED_INDUSTRY_A_E               0x0000005C
+#endif
+#ifndef PRODUCT_STORAGE_WORKGROUP_EVALUATION_SERVER
+#define PRODUCT_STORAGE_WORKGROUP_EVALUATION_SERVER 0x0000005F
+#endif
+#ifndef PRODUCT_STORAGE_STANDARD_EVALUATION_SERVER
+#define PRODUCT_STORAGE_STANDARD_EVALUATION_SERVER  0x00000060
+#endif
+#ifndef PRODUCT_CORE_ARM                            
+#define PRODUCT_CORE_ARM                            0x00000061
+#endif
+#ifndef PRODUCT_CORE_N                              
+#define PRODUCT_CORE_N                              0x00000062
+#endif
+#ifndef PRODUCT_CORE_COUNTRYSPECIFIC               
+#define PRODUCT_CORE_COUNTRYSPECIFIC                0x00000063
+#endif
+#ifndef PRODUCT_CORE_SINGLELANGUAGE                 
+#define PRODUCT_CORE_SINGLELANGUAGE                 0x00000064
+#endif
+#ifndef PRODUCT_CORE                                
+#define PRODUCT_CORE                                0x00000065
+#endif
+#ifndef PRODUCT_PROFESSIONAL_WMC                    
+#define PRODUCT_PROFESSIONAL_WMC                    0x00000067
+#endif
+#ifndef PRODUCT_MOBILE_CORE
+#define PRODUCT_MOBILE_CORE                         0x00000068
+#endif
 // Newer suite types than what is currently defined in
 //   Visual Studio 2005
 #ifndef VER_SUITE_WH_SERVER
@@ -387,11 +437,20 @@ int get_os_information(
     switch (osvi.dwPlatformId) {
         case VER_PLATFORM_WIN32_NT:
 
+			if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 3 ) {
+                if( osvi.wProductType == VER_NT_WORKSTATION ) {
+                    strcat(os_name, "Windows Blue");
+                } else {
+                    strcat(os_name, "Windows Blue Server");
+                }
+                pGPI( 6, 3, 0, 0, &dwType);
+            }
+
             if ( osvi.dwMajorVersion == 6 && osvi.dwMinorVersion == 2 ) {
                 if( osvi.wProductType == VER_NT_WORKSTATION ) {
                     strcat(os_name, "Windows 8");
                 } else {
-                    strcat(os_name, "Windows 8 Server ");
+                    strcat(os_name, "Windows Server 2012");
                 }
                 pGPI( 6, 2, 0, 0, &dwType);
             }
@@ -434,8 +493,7 @@ int get_os_information(
                 strcat(os_name, "Windows 2000");
             }
 
-            if ( osvi.dwMajorVersion <= 4 )
-            {
+            if ( osvi.dwMajorVersion <= 4 ) {
                 strcat(os_name, "Windows NT");
             }
 
@@ -541,6 +599,25 @@ int get_os_information(
                             case PRODUCT_PRERELEASE_N:
                                 strcat(szSKU, "Developer Preview N ");
                                 break;
+							case PRODUCT_EMBEDDED:
+								strcat(szSKU, "Embedded Standard ");
+								break;
+							case PRODUCT_THINPC:    
+								strcat(szSKU, "ThinPC ");
+								break;
+							case PRODUCT_CORE:
+								strcat(szSKU, "Core ");
+								break;
+							case PRODUCT_CORE_N:
+								strcat(szSKU, "Core N ");
+								break;
+							case PRODUCT_CORE_ARM:
+								strcat(szSKU, "Core "); // just Core because Boinc will add ARM later
+								break;
+							case PRODUCT_PROFESSIONAL_WMC:
+								strcat(szSKU, "Professional with Media Center ");
+								break;
+							
                        }
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2) ) {
                         if( osvi.wSuiteMask & VER_SUITE_PERSONAL ) {
@@ -567,7 +644,7 @@ int get_os_information(
                     }
                 }
 
-                // Test for the server type.
+				// Test for the server type.
                 else if ( (osvi.wProductType == VER_NT_SERVER) || (osvi.wProductType == VER_NT_DOMAIN_CONTROLLER) ) {
                     if( (osvi.dwMajorVersion == 6) ) {
 
@@ -989,12 +1066,17 @@ int get_processor_features(char* vendor, char* features, int features_size) {
     FEATURE_TEST(std_supported, (std_edx & (1 << 28)), "htt ");
     FEATURE_TEST(std_supported, (std_edx & (1 << 29)), "tm ");
 
-    FEATURE_TEST(std_supported, (std_ecx & (1 << 0)), "pni ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 0)), "pni ");		// should be named like it is definded - SSE3 and not PNI !
     FEATURE_TEST(std_supported, (std_ecx & (1 << 9)), "ssse3 ");
-    FEATURE_TEST(std_supported, (std_ecx & (1 << 13)), "cx16 ");
+	FEATURE_TEST(std_supported, (std_ecx & (1 << 12)), "fma ");		// removed from Intel only because AMD Family 15h & 16h support it too
+	FEATURE_TEST(std_supported, (std_ecx & (1 << 13)), "cx16 ");
     FEATURE_TEST(std_supported, (std_ecx & (1 << 19)), "sse4_1 ");
     FEATURE_TEST(std_supported, (std_ecx & (1 << 20)), "sse4_2 ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 22)), "movebe ");	// removed from Intel only because AMD Family 16h support it too	
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 23)), "popcnt ");	// removed from Intel only because AMD Family 10h/11h/12h/14h/15h/16h support it too
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 25)), "aes ");		// removed from Intel only because AMD Family 15h & 16h support it too
 
+	
     FEATURE_TEST(ext_supported, (ext_edx & (1 << 11)), "syscall ");
     FEATURE_TEST(ext_supported, (ext_edx & (1 << 20)), "nx ");
     FEATURE_TEST(ext_supported, (ext_edx & (1 << 29)), "lm ");
@@ -1004,11 +1086,7 @@ int get_processor_features(char* vendor, char* features, int features_size) {
         FEATURE_TEST(std_supported, (std_ecx & (1 << 5)), "vmx ");
         FEATURE_TEST(std_supported, (std_ecx & (1 << 6)), "smx ");
         FEATURE_TEST(std_supported, (std_ecx & (1 << 8)), "tm2 ");
-        FEATURE_TEST(std_supported, (std_ecx & (1 << 12)), "fma ");
         FEATURE_TEST(std_supported, (std_ecx & (1 << 18)), "dca ");
-        FEATURE_TEST(std_supported, (std_ecx & (1 << 22)), "movebe ");
-        FEATURE_TEST(std_supported, (std_ecx & (1 << 23)), "popcnt ");
-        FEATURE_TEST(std_supported, (std_ecx & (1 << 25)), "aes ");
 
         FEATURE_TEST(std_supported, (std_edx & (1 << 31)), "pbe ");
     }
@@ -1024,7 +1102,10 @@ int get_processor_features(char* vendor, char* features, int features_size) {
         FEATURE_TEST(ext_supported, (ext_ecx & (1 << 13)), "wdt ");
         FEATURE_TEST(ext_supported, (ext_ecx & (1 << 15)), "lwp ");
         FEATURE_TEST(ext_supported, (ext_ecx & (1 << 16)), "fma4 ");
-        FEATURE_TEST(ext_supported, (ext_ecx & (1 << 18)), "cvt16 ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 17)), "tce ");		// new - translation cache extension
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 18)), "cvt16 ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 21)), "tbm ");		// new - trailing bit manipulation instruction
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 22)), "topx ");	// new - topology extensions
 
         FEATURE_TEST(ext_supported, (ext_edx & (1 << 26)), "page1gb ");
         FEATURE_TEST(ext_supported, (ext_edx & (1 << 27)), "rdtscp ");
@@ -1198,7 +1279,12 @@ int HOST_INFO::get_virtualbox_version() {
 //
 int HOST_INFO::get_host_info() {
     get_timezone(timezone);
-    get_filesystem_info(d_total, d_free);
+    int retval = get_filesystem_info(d_total, d_free);
+    if (retval) {
+        msg_printf(0, MSG_INTERNAL_ERROR,
+            "get_filesystem_info(): %s", boincerror(retval)
+        );
+    }
     get_memory_info(m_nbytes, m_swap);
     get_os_information(
         os_name, sizeof(os_name), os_version, sizeof(os_version)
@@ -1219,9 +1305,9 @@ int HOST_INFO::get_host_info() {
 }
 
 bool HOST_INFO::host_is_running_on_batteries() {
-    SYSTEM_POWER_STATUS pStatus;
-    ZeroMemory(&pStatus, sizeof(SYSTEM_POWER_STATUS));
-    if (!GetSystemPowerStatus(&pStatus)) {
+    SYSTEM_POWER_STATUS Status;
+    ZeroMemory(&Status, sizeof(SYSTEM_POWER_STATUS));
+    if (!GetSystemPowerStatus(&Status)) {
         return false;
     }
 
@@ -1229,13 +1315,48 @@ bool HOST_INFO::host_is_running_on_batteries() {
     //   undocumented value, so lets check to see if the
     //   battery is charging or missing and make that part
     //   of the decision.
-    bool bIsOnBatteryPower  = (pStatus.ACLineStatus != 1);
-    bool bIsBatteryCharging = ((pStatus.BatteryFlag & 8) == 8);
-    bool bIsBatteryMissing = ((pStatus.BatteryFlag & 128) == 128);
+    bool bIsOnBatteryPower  = (Status.ACLineStatus != 1);
+    bool bIsBatteryCharging = ((Status.BatteryFlag & 8) == 8);
+    bool bIsBatteryMissing = ((Status.BatteryFlag & 128) == 128);
 
     return (bIsOnBatteryPower && !bIsBatteryCharging && !bIsBatteryMissing);
 }
 
+int HOST_INFO::get_host_battery_charge() {
+    SYSTEM_POWER_STATUS Status;
+    ZeroMemory(&Status, sizeof(SYSTEM_POWER_STATUS));
+    if (!GetSystemPowerStatus(&Status)) {
+        return false;
+    }
+
+    if (((int)Status.BatteryLifePercent) == 255) return 0;
+    return ((int)Status.BatteryLifePercent);
+}
+
+int HOST_INFO::get_host_battery_state() {
+    SYSTEM_POWER_STATUS Status;
+    ZeroMemory(&Status, sizeof(SYSTEM_POWER_STATUS));
+    if (!GetSystemPowerStatus(&Status)) {
+        return false;
+    }
+
+    // Sometimes the system reports the ACLineStatus as an
+    //   undocumented value, so lets check to see if the
+    //   battery is charging or missing and make that part
+    //   of the decision.
+    bool bIsOnBatteryPower  = (Status.ACLineStatus != 1);
+    bool bIsBatteryCharging = ((Status.BatteryFlag & 8) == 8);
+
+    if        (bIsOnBatteryPower && !bIsBatteryCharging) {
+        return BATTERY_STATE_DISCHARGING;
+    } else if (((int)Status.BatteryLifePercent) == 100) {
+        return BATTERY_STATE_FULL;
+    } else if (bIsBatteryCharging) {
+        return BATTERY_STATE_CHARGING;
+    }
+    return BATTERY_STATE_UNKNOWN;
+}
+
 bool HOST_INFO::users_idle(bool /*check_all_logins*/, double idle_time_to_run) {
     double seconds_idle = get_idle_tick_count() / 1000;
     double seconds_time_to_run = 60 * idle_time_to_run;
