@@ -42,6 +42,7 @@
 #include "file_names.h"
 #include "project.h"
 #include "result.h"
+#include "sandbox.h"
 
 using std::string;
 
@@ -80,7 +81,6 @@ void LOG_FLAGS::show() {
     show_flag(buf, cpu_sched_debug, "cpu_sched_debug");
     show_flag(buf, cpu_sched_status, "cpu_sched_status");
     show_flag(buf, dcf_debug, "dcf_debug");
-    show_flag(buf, priority_debug, "priority_debug");
     show_flag(buf, file_xfer_debug, "file_xfer_debug");
     show_flag(buf, gui_rpc_debug, "gui_rpc_debug");
     show_flag(buf, heartbeat_debug, "heartbeat_debug");
@@ -88,7 +88,9 @@ void LOG_FLAGS::show() {
     show_flag(buf, http_xfer_debug, "http_xfer_debug");
     show_flag(buf, mem_usage_debug, "mem_usage_debug");
     show_flag(buf, network_status_debug, "network_status_debug");
+    show_flag(buf, notice_debug, "notice_debug");
     show_flag(buf, poll_debug, "poll_debug");
+    show_flag(buf, priority_debug, "priority_debug");
     show_flag(buf, proxy_debug, "proxy_debug");
     show_flag(buf, rr_simulation, "rr_simulation");
     show_flag(buf, sched_op_debug, "sched_op_debug");
@@ -100,16 +102,17 @@ void LOG_FLAGS::show() {
     show_flag(buf, time_debug, "time_debug");
     show_flag(buf, unparsed_xml, "unparsed_xml");
     show_flag(buf, work_fetch_debug, "work_fetch_debug");
-    show_flag(buf, notice_debug, "notice_debug");
 
     if (strlen(buf)) {
         msg_printf(NULL, MSG_INFO, "log flags: %s", buf);
     }
 }
 
-static void show_gpu_ignore(vector<int>& devs, const char* name) {
+static void show_gpu_ignore(vector<int>& devs, int rt) {
     for (unsigned int i=0; i<devs.size(); i++) {
-        msg_printf(NULL, MSG_INFO, "Config: ignoring %s GPU %d", name, devs[i]);
+        msg_printf(NULL, MSG_INFO,
+            "Config: ignoring %s %d", proc_type_name(rt), devs[i]
+        );
     }
 }
 
@@ -188,8 +191,12 @@ void CONFIG::show() {
     if (fetch_minimal_work) {
         msg_printf(NULL, MSG_INFO, "Config: fetch minimal work");
     }
-    show_gpu_ignore(ignore_nvidia_dev, GPU_TYPE_NVIDIA);
-    show_gpu_ignore(ignore_ati_dev, GPU_TYPE_ATI);
+    if (fetch_on_update) {
+        msg_printf(NULL, MSG_INFO, "Config: fetch on update");
+    }
+    for (int j=1; j<NPROC_TYPES; j++) {
+        show_gpu_ignore(ignore_gpu_instance[j], j);
+    }
     for (i=0; i<exclude_gpus.size(); i++) {
         show_exclude_gpu(exclude_gpus[i]);
     }
@@ -213,26 +220,40 @@ void CONFIG::show() {
     FILE* f = fopen(REMOTEHOST_FILE_NAME, "r");
     if (f) {
         msg_printf(NULL, MSG_INFO,
-            "Config: GUI RPC allowed from:"
+            "Config: GUI RPCs allowed from:"
         );
         char buf[256];
         while (fgets(buf, 256, f)) {
             strip_whitespace(buf);
             if (!(buf[0] =='#' || buf[0] == ';') && strlen(buf) > 0 ) {
                 msg_printf(NULL, MSG_INFO,
-                    "Config:   %s", buf
+                    "    %s", buf
                 );
             }
         }
         fclose(f);
     }
+    if (vbox_window) {
+        msg_printf(NULL, MSG_INFO,
+            "Config: open console window for VirtualBox applications"
+        );
+        if (g_use_sandbox) {
+            msg_printf(NULL, MSG_INFO,
+                "    NOTE: the client is running in protected mode,"
+            );
+            msg_printf(NULL, MSG_INFO,
+                "    so VirtualBox console windows cannot be opened."
+            );
+        }
+    }
 }
 
 // This is used by the BOINC client.
 // KEEP IN SYNCH WITH CONFIG::parse_options()!!
+// (It's separate so that we can write messages in it)
 
 int CONFIG::parse_options_client(XML_PARSER& xp) {
-    char path[256];
+    char path[MAXPATHLEN];
     string s;
     int n, retval;
 
@@ -244,8 +265,9 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
     alt_platforms.clear();
     exclusive_apps.clear();
     exclusive_gpu_apps.clear();
-    ignore_nvidia_dev.clear();
-    ignore_ati_dev.clear();
+    for (int i=1; i<NPROC_TYPES; i++) {
+        ignore_gpu_instance[i].clear();
+    }
 
     while (!xp.get_tag()) {
         if (!xp.is_tag) {
@@ -334,6 +356,7 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
             continue;
         }
         if (xp.parse_bool("fetch_minimal_work", fetch_minimal_work)) continue;
+        if (xp.parse_bool("fetch_on_update", fetch_on_update)) continue;
         if (xp.parse_string("force_auth", force_auth)) {
             downcase_string(force_auth);
             continue;
@@ -342,11 +365,15 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
         if (xp.parse_int("http_transfer_timeout", http_transfer_timeout)) continue;
         if (xp.parse_int("http_transfer_timeout_bps", http_transfer_timeout_bps)) continue;
         if (xp.parse_int("ignore_cuda_dev", n)||xp.parse_int("ignore_nvidia_dev", n)) {
-            ignore_nvidia_dev.push_back(n);
+            ignore_gpu_instance[PROC_TYPE_NVIDIA_GPU].push_back(n);
             continue;
         }
         if (xp.parse_int("ignore_ati_dev", n)) {
-            ignore_ati_dev.push_back(n);
+            ignore_gpu_instance[PROC_TYPE_AMD_GPU].push_back(n);
+            continue;
+        }
+        if (xp.parse_int("ignore_intel_dev", n)) {
+            ignore_gpu_instance[PROC_TYPE_INTEL_GPU].push_back(n);
             continue;
         }
         if (xp.parse_int("max_file_xfers", max_file_xfers)) continue;
@@ -366,7 +393,7 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
         if (xp.parse_bool("os_random_only", os_random_only)) continue;
 #ifndef SIM
         if (xp.match_tag("proxy_info")) {
-            retval = config_proxy_info.parse_config(xp);
+            retval = proxy_info.parse_config(xp);
             if (retval) {
                 msg_printf_notice(NULL, false, NULL,
                     "Can't parse <proxy_info> element in cc_config.xml"
@@ -392,6 +419,7 @@ int CONFIG::parse_options_client(XML_PARSER& xp) {
         if (xp.parse_bool("use_all_gpus", use_all_gpus)) continue;
         if (xp.parse_bool("use_certs", use_certs)) continue;
         if (xp.parse_bool("use_certs_only", use_certs_only)) continue;
+        if (xp.parse_bool("vbox_window", vbox_window)) continue;
 
         msg_printf_notice(NULL, false,
             "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
@@ -469,6 +497,9 @@ int CONFIG::parse(FILE* f) {
     return parse(xp, log_flags);
 }
 
+// read config file, e.g. cc_config.xml
+// Called on startup and in response to GUI RPC requesting reread
+//
 int read_config_file(bool init, const char* fname) {
     if (!init) {
         msg_printf(NULL, MSG_INFO, "Re-reading %s", fname);
@@ -487,9 +518,10 @@ int read_config_file(bool init, const char* fname) {
         config.max_stdout_file_size, config.max_stderr_file_size
     );
 #endif
+    config_proxy_info = config.proxy_info;
+
     if (init) {
         coprocs = config.config_coprocs;
-        config_proxy_info = config.proxy_info;
         if (strlen(config.data_dir)) {
 #ifdef _WIN32
             _chdir(config.data_dir);
@@ -497,41 +529,138 @@ int read_config_file(bool init, const char* fname) {
             chdir(config.data_dir);
 #endif
         }
+    } else {
+        select_proxy_info();        // in case added or removed proxy info
     }
     return 0;
 }
 
 // Do stuff involving GPU exclusions.
-// - Count excluded GPUS per project.
-//   NOTE: this is currently done just on the project level.
-//   Could do it at the app level also.
-// - Flag app versions and results for which all GPUs are excluded
+// - check syntax
+// - set APP::non_excluded_instances[rsc_type]
+//   (used in RR sim)
+// - set PROJECT::rsc_pwf[rsc_type].non_excluded_instances
+//   (used in work fetch)
+// - set PROJECT::rsc_pwf[rsc_type].ncoprocs_excluded
+//   (used in RR sim and work fetch)
+// - set APP_VERSION::coproc_missing for app versions where
+//   all instances are excluded
+// - set RESULT::coproc_missing for results for which
+//   APP_VERSION::coproc_missing is set.
 //
 void process_gpu_exclusions() {
-    unsigned int i, j;
+    unsigned int i, j, a;
     PROJECT *p;
 
+    // check the syntactic validity of the exclusions
+    //
+    for (i=0; i<config.exclude_gpus.size(); i++) {
+        EXCLUDE_GPU& eg = config.exclude_gpus[i];
+        p = gstate.lookup_project(eg.url.c_str());
+        if (!p) {
+            msg_printf(0, MSG_USER_ALERT,
+                "Bad URL in GPU exclusion: %s", eg.url.c_str()
+            );
+            continue;
+        }
+        if (!eg.type.empty()) {
+            bool found = false;
+            string types;
+            for (int k=1; k<coprocs.n_rsc; k++) {
+                COPROC& cp = coprocs.coprocs[k];
+                if (eg.type == cp.type) {
+                    found = true;
+                    break;
+                }
+                types += " " + string(cp.type);
+            }
+            if (!found) {
+                msg_printf(p, MSG_USER_ALERT,
+                    "Bad type '%s' in GPU exclusion; valid types:%s",
+                    eg.type.c_str(), types.c_str()
+                );
+                continue;
+            }
+            if (!eg.appname.empty()) {
+                APP* app = gstate.lookup_app(p, eg.appname.c_str());
+                if (!app) {
+                    msg_printf(p, MSG_USER_ALERT,
+                        "Nonexistent app '%s' in GPU exclusion",
+                        eg.appname.c_str()
+                    );
+                    continue;
+                }
+            }
+        }
+    }
+
+    for (i=0; i<gstate.apps.size(); i++) {
+        APP* app = gstate.apps[i];
+        for (int k=1; k<coprocs.n_rsc; k++) {
+            COPROC& cp = coprocs.coprocs[k];
+            app->non_excluded_instances[k] = (1<<cp.count)-1;  // all 1's
+        }
+    }
+
     for (i=0; i<gstate.projects.size(); i++) {
         p = gstate.projects[i];
         for (int k=1; k<coprocs.n_rsc; k++) {
-            int n=0;
             COPROC& cp = coprocs.coprocs[k];
+            int all_instances = (1<<cp.count)-1;  // bitmap of 1 for all inst
+            p->rsc_pwf[k].non_excluded_instances = all_instances;
             for (j=0; j<config.exclude_gpus.size(); j++) {
                 EXCLUDE_GPU& eg = config.exclude_gpus[j];
-                if (strcmp(eg.url.c_str(), p->master_url)) continue;
-                if (!eg.appname.empty()) continue;
                 if (!eg.type.empty() && (eg.type != cp.type)) continue;
+                if (strcmp(eg.url.c_str(), p->master_url)) continue;
+                int mask;
                 if (eg.device_num >= 0) {
+                    int index = cp.device_num_index(eg.device_num);
                     // exclusion may refer to nonexistent GPU
                     //
-                    if (cp.device_num_exists(eg.device_num)) {
-                        n++;
+                    if (index < 0) continue;
+                    mask = 1<<index;
+                } else {
+                    mask = all_instances;
+                }
+                if (eg.appname.empty()) {
+                    // exclusion applies to all apps
+                    //
+                    for (a=0; a<gstate.apps.size(); a++) {
+                        APP* app = gstate.apps[a];
+                        if (app->project != p) continue;
+                        app->non_excluded_instances[k] &= ~mask;
                     }
                 } else {
-                    n = cp.count;
+                    // exclusion applies to a particular app
+                    //
+                    APP* app = gstate.lookup_app(p, eg.appname.c_str());
+                    if (!app) continue;
+                    app->non_excluded_instances[k] &= ~mask;
+                }
+            }
+
+            p->rsc_pwf[k].non_excluded_instances = 0;
+            for (a=0; a<gstate.apps.size(); a++) {
+                APP* app = gstate.apps[a];
+                if (app->project != p) continue;
+                p->rsc_pwf[k].non_excluded_instances |= app->non_excluded_instances[k];
+            }
+
+            // compute ncoprocs_excluded as the number of instances
+            // excluded for at least 1 app
+            //
+            p->rsc_pwf[k].ncoprocs_excluded = 0;
+            for (int b=0; b<cp.count; b++) {
+                int mask = 1<<b;
+                for (a=0; a<gstate.apps.size(); a++) {
+                    APP* app = gstate.apps[a];
+                    if (app->project != p) continue;
+                    if (!(app->non_excluded_instances[k] & mask)) {
+                        p->rsc_pwf[k].ncoprocs_excluded++;
+                        break;
+                    }
                 }
             }
-            p->ncoprocs_excluded[k] = n;
         }
     }
 
