@@ -15,7 +15,7 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-// command-line version of the BOINC core client
+// command-line version of the BOINC client
 
 // This file contains no GUI-related code.
 
@@ -40,6 +40,11 @@
 #include <cstdlib>
 #include <unistd.h>
 #include <csignal>
+
+#ifdef ANDROID
+#include "android/log.h"
+#endif
+
 #endif
 
 #if (defined (__APPLE__) && defined(SANDBOX) && defined(_DEBUG))
@@ -51,13 +56,11 @@
 #include "str_util.h"
 #include "str_replace.h"
 #include "util.h"
-#include "unix_util.h"
 #include "prefs.h"
 #include "filesys.h"
 #include "network.h"
 #include "idlemon.h"
 
-#include "cs_proxy.h"
 #include "client_state.h"
 #include "file_names.h"
 #include "log_flags.h"
@@ -67,6 +70,7 @@
 
 #include "main.h"
 
+
 // Log informational messages to system specific places
 //
 void log_message_startup(const char* msg) {
@@ -82,6 +86,8 @@ void log_message_startup(const char* msg) {
         LogEventInfoMessage(evt_msg);
 #elif defined(__EMX__)
 #elif defined (__APPLE__)
+#elif defined (ANDROID)
+        __android_log_print(ANDROID_LOG_INFO, "BOINC", evt_msg);
 #else
         syslog(LOG_DAEMON|LOG_INFO, evt_msg);
 #endif
@@ -96,7 +102,8 @@ void log_message_error(const char* msg) {
     snprintf(evt_msg, sizeof(evt_msg),
         "%s\n"
         "GLE: %s\n",
-        msg, windows_error_string(evt_msg, (sizeof(evt_msg)-((int)strlen(msg)+7)))
+        msg, 
+        windows_format_error_string(GetLastError(), evt_msg, (sizeof(evt_msg)-((int)strlen(msg)+7)))
     );
 #else
     snprintf(evt_msg, sizeof(evt_msg),
@@ -111,6 +118,8 @@ void log_message_error(const char* msg) {
         LogEventErrorMessage(evt_msg);
 #elif defined(__EMX__)
 #elif defined (__APPLE__)
+#elif defined (ANDROID)
+        __android_log_print(ANDROID_LOG_ERROR, "BOINC", evt_msg);
 #else
         syslog(LOG_DAEMON|LOG_ERR, evt_msg);
 #endif
@@ -131,6 +140,8 @@ void log_message_error(const char* msg, int error_code) {
         LogEventErrorMessage(evt_msg);
 #elif defined(__EMX__)
 #elif defined (__APPLE__)
+#elif defined (ANDROID)
+        __android_log_print(ANDROID_LOG_ERROR, "BOINC", evt_msg);
 #else
         syslog(LOG_DAEMON|LOG_ERR, evt_msg);
 #endif
@@ -312,24 +323,6 @@ int boinc_main_loop() {
         return retval;
     }
 
-    // must parse env vars after gstate.init();
-    // otherwise items will get overwritten with state file info
-    //
-    gstate.parse_env_vars();
-
-    // do this after parsing env vars
-    //
-    proxy_info_startup();
-
-    if (gstate.projects.size() == 0) {
-        msg_printf(NULL, MSG_INFO,
-            "This computer is not attached to any projects"
-        );
-        msg_printf(NULL, MSG_INFO,
-            "Visit http://boinc.berkeley.edu for instructions"
-        );
-    }
-
     log_message_startup("Initialization completed");
 
     while (1) {
@@ -398,7 +391,9 @@ int main(int argc, char** argv) {
         // FIXME FIXME.  Duplicate instances of -detach may cause this to be
         // executed unnecessarily.  At worst, I think it leads to a few extra
         // processes being created and destroyed.
-        if (strcmp(argv[index], "-detach") == 0 || strcmp(argv[index], "--detach") == 0) {
+        if (strcmp(argv[index], "-detach") == 0 || strcmp(argv[index], "--detach") == 0 ||
+            strcmp(argv[index], "-detach_console") == 0 || strcmp(argv[index], "--detach_console") == 0
+        ) {
             int i, len;
             char *commandLine;
             STARTUPINFO si;
@@ -430,12 +425,13 @@ int main(int argc, char** argv) {
 
             // If process creation succeeds, we exit, if it fails punt and continue
             // as usual.  We won't detach properly, but the program will run.
-            if (CreateProcess(NULL, commandLine, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {
+            if (CreateProcess(NULL, commandLine, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi)) {
                 exit(0);
             }
             break;
         }
 #endif
+
     }
 
     init_core_client(argc, argv);
