@@ -37,6 +37,7 @@ void PROJECT::init() {
     project_specific_prefs = "";
     gui_urls = "";
     resource_share = 100;
+    desired_disk_usage = 0;
     for (int i=0; i<MAX_RSC; i++) {
         no_rsc_pref[i] = false;
         no_rsc_config[i] = false;
@@ -108,6 +109,7 @@ void PROJECT::init() {
     completions_ratio_stdev = 0.1;  // for the first couple of completions - guess.
     completions_required_stdevs = 3.0;
     result_index = 0;
+    ignore = false;
 #endif
 }
 
@@ -246,6 +248,8 @@ int PROJECT::parse_state(XML_PARSER& xp) {
             if (btemp) handle_no_rsc_apps(this, "CPU");
             continue;
         }
+
+        // deprecated
         if (xp.parse_bool("no_cuda_apps", btemp)) {
             if (btemp) handle_no_rsc_apps(this, GPU_TYPE_NVIDIA);
             continue;
@@ -254,6 +258,7 @@ int PROJECT::parse_state(XML_PARSER& xp) {
             if (btemp) handle_no_rsc_apps(this, GPU_TYPE_ATI);
             continue;
         }
+
         if (xp.parse_str("no_rsc_apps", buf, sizeof(buf))) {
             handle_no_rsc_apps(this, buf);
             continue;
@@ -270,6 +275,10 @@ int PROJECT::parse_state(XML_PARSER& xp) {
             if (btemp) handle_no_rsc_ams(this, GPU_TYPE_ATI);
             continue;
         }
+        if (xp.parse_bool("no_intel_gpu_ams", btemp)) {
+            if (btemp) handle_no_rsc_ams(this, GPU_TYPE_INTEL);
+            continue;
+        }
         if (xp.parse_str("no_rsc_ams", buf, sizeof(buf))) {
             handle_no_rsc_ams(this, buf);
             continue;
@@ -292,6 +301,7 @@ int PROJECT::parse_state(XML_PARSER& xp) {
             trickle_up_ops.push_back(new TRICKLE_UP_OP(stemp));
             continue;
         }
+        if (xp.parse_double("desired_disk_usage", desired_disk_usage)) continue;
         if (log_flags.unparsed_xml) {
             msg_printf(0, MSG_INFO,
                 "[unparsed_xml] PROJECT::parse_state(): unrecognized: %s",
@@ -343,6 +353,7 @@ int PROJECT::write_state(MIOFILE& out, bool gui_rpc) {
         "    <rec_time>%f</rec_time>\n"
 
         "    <resource_share>%f</resource_share>\n"
+        "    <desired_disk_usage>%f</desired_disk_usage>\n"
         "    <duration_correction_factor>%f</duration_correction_factor>\n"
         "    <sched_rpc_pending>%d</sched_rpc_pending>\n"
         "    <send_time_stats_log>%d</send_time_stats_log>\n"
@@ -374,6 +385,7 @@ int PROJECT::write_state(MIOFILE& out, bool gui_rpc) {
         pwf.rec,
         pwf.rec_time,
         resource_share,
+        desired_disk_usage,
         duration_correction_factor,
         sched_rpc_pending,
         send_time_stats_log,
@@ -415,7 +427,7 @@ int PROJECT::write_state(MIOFILE& out, bool gui_rpc) {
         if (no_rsc_pref[j]) {
             out.printf("    <no_rsc_pref>%s</no_rsc_pref>\n", rsc_name(j));
         }
-        if (j>0 && gui_rpc && (ncoprocs_excluded[j] == rsc_work_fetch[j].ninstances)) {
+        if (j>0 && gui_rpc && (rsc_pwf[j].ncoprocs_excluded == rsc_work_fetch[j].ninstances)) {
             out.printf("    <no_rsc_config>%s</no_rsc_config>\n", rsc_name(j));
         }
     }
@@ -529,6 +541,7 @@ void PROJECT::copy_state_fields(PROJECT& p) {
     if (ams_resource_share >= 0) {
         resource_share = ams_resource_share;
     }
+    desired_disk_usage = p.desired_disk_usage;
     use_symlinks = p.use_symlinks;
 }
 
@@ -616,7 +629,7 @@ const char* PROJECT::get_scheduler_url(int index, double r) {
 //
 void PROJECT::delete_project_file_symlinks() {
     unsigned int i;
-    char project_dir[256], path[256];
+    char project_dir[256], path[MAXPATHLEN];
 
     get_project_dir(this, project_dir, sizeof(project_dir));
     for (i=0; i<project_files.size(); i++) {
@@ -675,7 +688,7 @@ void PROJECT::write_project_files(MIOFILE& f) {
 // has several logical names, so try them all
 //
 int PROJECT::write_symlink_for_project_file(FILE_INFO* fip) {
-    char project_dir[256], link_path[256], file_path[256];
+    char project_dir[256], link_path[MAXPATHLEN], file_path[MAXPATHLEN];
     unsigned int i;
 
     get_project_dir(this, project_dir, sizeof(project_dir));
