@@ -67,7 +67,7 @@ bool SCHEDULER_OP::check_master_fetch_start() {
             "Couldn't start download of scheduler list: %s", boincerror(retval)
         );
         p->master_fetch_failures++;
-        backoff(p, "scheduler list fetch failed\n");
+        project_rpc_backoff(p, "scheduler list fetch failed\n");
         return false;
     }
     msg_printf(p, MSG_INFO, "Fetching scheduler list");
@@ -78,7 +78,7 @@ bool SCHEDULER_OP::check_master_fetch_start() {
 
 // try to initiate an RPC to the given project.
 // If there are multiple schedulers, start with a random one.
-// User messages and backoff() is done at this level.
+// User messages and project RPC backoff is done at this level.
 //
 int SCHEDULER_OP::init_op_project(PROJECT* p, int r) {
     int retval;
@@ -100,13 +100,13 @@ int SCHEDULER_OP::init_op_project(PROJECT* p, int r) {
             sprintf(err_msg,
                 "Scheduler list fetch initialization failed: %d\n", retval
             );
-            backoff(p, err_msg);
+            project_rpc_backoff(p, err_msg);
         }
         return retval;
     }
 
     if (reason == RPC_REASON_INIT) {
-        work_fetch.set_initial_work_request();
+        work_fetch.set_initial_work_request(p);
         if (!gstate.cpu_benchmarks_done()) {
             gstate.cpu_benchmarks_set_defaults();
         }
@@ -122,7 +122,7 @@ int SCHEDULER_OP::init_op_project(PROJECT* p, int r) {
             "scheduler request to %s failed: %s\n",
             p->get_scheduler_url(url_index, url_random), boincerror(retval)
         );
-        backoff(p, err_msg);
+        project_rpc_backoff(p, err_msg);
     } else {
         // RPC started OK, so we must have network connectivity.
         // Now's a good time to check for new BOINC versions
@@ -148,7 +148,7 @@ int SCHEDULER_OP::init_op_project(PROJECT* p, int r) {
 // Back off contacting this project's schedulers,
 // and output an error msg if needed
 //
-void SCHEDULER_OP::backoff(PROJECT* p, const char *reason_msg) {
+void SCHEDULER_OP::project_rpc_backoff(PROJECT* p, const char *reason_msg) {
     char buf[1024];
 
     if (gstate.in_abort_sequence) {
@@ -199,7 +199,7 @@ void SCHEDULER_OP::backoff(PROJECT* p, const char *reason_msg) {
 // will be retriggered automatically
 //
 void SCHEDULER_OP::rpc_failed(const char* msg) {
-    backoff(cur_proj, msg);
+    project_rpc_backoff(cur_proj, msg);
     switch (cur_proj->sched_rpc_pending) {
     case RPC_REASON_INIT:
     case RPC_REASON_ACCT_MGR_REQ:
@@ -229,7 +229,6 @@ static void request_string(char* buf) {
 int SCHEDULER_OP::start_rpc(PROJECT* p) {
     int retval;
     char request_file[1024], reply_file[1024], buf[256];
-    const char *trickle_up_msg;
 
     safe_strcpy(scheduler_url, p->get_scheduler_url(url_index, url_random));
     if (log_flags.sched_ops) {
@@ -237,28 +236,24 @@ int SCHEDULER_OP::start_rpc(PROJECT* p) {
             "Sending scheduler request: %s.", rpc_reason_string(reason)
         );
         if (p->trickle_up_pending && reason != RPC_REASON_TRICKLE_UP) {
-            trickle_up_msg = ", sending trickle-up message";
-        } else {
-            trickle_up_msg = "";
+            msg_printf(p, MSG_INFO, "Sending trickle-up message");
+        }
+        if (p->nresults_returned) {
+            msg_printf(p, MSG_INFO,
+                "Reporting %d completed tasks", p->nresults_returned
+            );
         }
         request_string(buf);
         if (strlen(buf)) {
-            if (p->nresults_returned) {
-                msg_printf(p, MSG_INFO,
-                    "Reporting %d completed tasks, requesting new tasks for %s%s",
-                    p->nresults_returned, buf, trickle_up_msg
-                );
-            } else {
-                msg_printf(p, MSG_INFO, "Requesting new tasks for %s%s", buf, trickle_up_msg);
-            }
+            msg_printf(p, MSG_INFO, "Requesting new tasks for %s", buf);
         } else {
-            if (p->nresults_returned) {
+            if (p->pwf.cant_fetch_work_reason) {
                 msg_printf(p, MSG_INFO,
-                    "Reporting %d completed tasks, not requesting new tasks%s",
-                    p->nresults_returned, trickle_up_msg
+                    "Not requesting tasks: %s",
+                    cant_fetch_work_string(p->pwf.cant_fetch_work_reason)
                 );
             } else {
-                msg_printf(p, MSG_INFO, "Not reporting or requesting tasks%s", trickle_up_msg);
+                msg_printf(p, MSG_INFO, "Not requesting tasks");
             }
         }
     }
@@ -305,7 +300,7 @@ int SCHEDULER_OP::init_master_fetch(PROJECT* p) {
         msg_printf(p, MSG_INFO, "[sched_op] Fetching master file");
     }
     cur_proj = p;
-    retval = http_op.init_get(p, p->master_url, master_filename, true);
+    retval = http_op.init_get(p, p->master_url, master_filename, true, 0, 0);
     if (retval) {
         if (log_flags.sched_ops) {
             msg_printf(p, MSG_INFO,
@@ -505,10 +500,10 @@ bool SCHEDULER_OP::poll() {
                 case 0:
                     break;
                 case ERR_PROJECT_DOWN:
-                    backoff(cur_proj, "project is down");
+                    project_rpc_backoff(cur_proj, "project is down");
                     break;
                 default:
-                    backoff(cur_proj, "can't parse scheduler reply");
+                    project_rpc_backoff(cur_proj, "can't parse scheduler reply");
                     break;
                 }
                 cur_proj->sched_rpc_pending = 0;
@@ -859,6 +854,8 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
                 handle_no_rsc_apps("CPU", project, btemp);
             }
             continue;
+
+        // deprecated syntax
         } else if (xp.parse_bool("no_cuda_apps", btemp)) {
             if (!project->anonymous_platform) {
                 handle_no_rsc_apps(GPU_TYPE_NVIDIA, project, btemp);
@@ -869,9 +866,10 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
                 handle_no_rsc_apps(GPU_TYPE_ATI, project, btemp);
             }
             continue;
+
         } else if (xp.parse_str("no_rsc_apps", buf, sizeof(buf))) {
             if (!project->anonymous_platform) {
-                handle_no_rsc_apps(buf, project, btemp);
+                handle_no_rsc_apps(buf, project, true);
             }
             continue;
         } else if (xp.parse_bool("verify_files_on_app_start", project->verify_files_on_app_start)) {
@@ -904,6 +902,8 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
             continue;
         } else if (xp.parse_int("teamid", project->teamid)) {
             continue;
+        } else if (xp.parse_double("desired_disk_usage", project->desired_disk_usage)) {
+            continue;
         } else {
             if (log_flags.unparsed_xml) {
                 msg_printf(project, MSG_INFO,
