@@ -52,19 +52,24 @@
 //      simulate use of EDF sim by scheduler
 //  [--cpu_sched_rr_only]
 //      use only RR scheduling
-//  [--use_hyst_fetch]
-//      client work fetch uses hysteresis
 //  [--rec_half_life X]
 //      half-life of recent est credit
 
+#include <math.h>
+
 #include "error_numbers.h"
 #include "str_util.h"
 #include "util.h"
 #include "log_flags.h"
 #include "filesys.h"
+#include "../sched/edf_sim.h"
+
 #include "client_msgs.h"
 #include "client_state.h"
-#include "../sched/edf_sim.h"
+#include "project.h"
+#include "result.h"
+#include "scheduler_op.h"
+
 #include "sim.h"
 
 #define SCHED_RETRY_DELAY_MIN    60                // 1 minute
@@ -79,13 +84,13 @@ const char* outfile_prefix = "./";
 #define RESULTS_DAT_FNAME "results.dat"
 #define RESULTS_TXT_FNAME "results.txt"
 #define SUMMARY_FNAME "summary.txt"
-#define DEBT_FNAME "debt.dat"
+#define REC_FNAME "rec.dat"
 
 bool user_active;
 double duration = 86400, delta = 60;
 FILE* logfile;
 FILE* html_out;
-FILE* debt_file;
+FILE* rec_file;
 FILE* index_file;
 FILE* summary_file;
 char log_filename[256];
@@ -106,7 +111,7 @@ bool active;
 bool gpu_active;
 bool connected;
 
-extern double debt_adjust_period;
+extern double rec_adjust_period;
 
 SIM_RESULTS sim_results;
 int njobs;
@@ -120,7 +125,6 @@ void usage(char* prog) {
         "[--delta X]\n"
         "[--server_uses_workload]\n"
         "[--cpu_sched_rr_only]\n"
-        "[--use_hyst_fetch]\n"
         "[--rec_half_life X]\n",
         prog
     );
@@ -287,7 +291,7 @@ void CLIENT_STATE::handle_completed_results(PROJECT* p) {
 void CLIENT_STATE::get_workload(vector<IP_RESULT>& ip_results) {
     for (unsigned int i=0; i<results.size(); i++) {
         RESULT* rp = results[i];
-        double x = rp->estimated_time_remaining();
+        double x = rp->estimated_runtime_remaining();
         if (x == 0) continue;
         IP_RESULT ipr(rp->name, rp->report_deadline-now, x);
         ip_results.push_back(ipr);
@@ -340,6 +344,22 @@ bool CLIENT_STATE::simulate_rpc(PROJECT* p) {
     int infeasible_count = 0;
     vector<RESULT*> new_results;
 
+    bool avail;
+    if (p->last_rpc_time) {
+        double delta = now - p->last_rpc_time;
+        avail = p->available.sample(delta);
+    } else {
+        avail = p->available.sample(0);
+    }
+    p->last_rpc_time = now;
+    if (!avail) {
+        sprintf(buf, "RPC to %s skipped - project down<br>", p->project_name);
+        html_msg += buf;
+        msg_printf(p, MSG_INFO, "RPC skipped: project down");
+        gstate.scheduler_op->project_rpc_backoff(p, "project down");
+        return false;
+    }
+
     // save request params for WORK_FETCH::handle_reply
     //
     double save_cpu_req_secs = rsc_work_fetch[0].req_secs;
@@ -434,7 +454,7 @@ bool CLIENT_STATE::simulate_rpc(PROJECT* p) {
     if (new_results.size() == 0) {
         for (int i=0; i<coprocs.n_rsc; i++) {
             if (rsc_work_fetch[i].req_secs) {
-                p->rsc_pwf[i].backoff(p, rsc_name(i));
+                p->rsc_pwf[i].resource_backoff(p, rsc_name(i));
             }
         }
     } else {
@@ -456,6 +476,7 @@ bool CLIENT_STATE::simulate_rpc(PROJECT* p) {
     work_fetch.handle_reply(p, &sr, new_results);
     p->nrpc_failures = 0;
     p->sched_rpc_pending = false;
+    p->min_rpc_time = now + 900;
     if (sent_something) {
         request_schedule_cpus("simulate_rpc");
         request_work_fetch("simulate_rpc");
@@ -496,7 +517,7 @@ bool CLIENT_STATE::scheduler_rpc_poll() {
 #if 0
         p = next_project_sched_rpc_pending();
         if (p) {
-            work_fetch.compute_work_request(p);
+            work_fetch.piggyback_work_request(p);
             action = simulate_rpc(p);
             break;
         }
@@ -505,7 +526,7 @@ bool CLIENT_STATE::scheduler_rpc_poll() {
         p = find_project_with_overdue_results(false);
         if (p) {
             //printf("doing RPC to %s to report results\n", p->project_name);
-            work_fetch.compute_work_request(p);
+            work_fetch.piggyback_work_request(p);
             action = simulate_rpc(p);
             break;
         }
@@ -522,7 +543,7 @@ bool CLIENT_STATE::scheduler_rpc_poll() {
         must_check_work_fetch = false;
         last_work_fetch_time = now;
 
-        p = work_fetch.choose_project(true);
+        p = work_fetch.choose_project(true, NULL);
 
         if (p) {
             action = simulate_rpc(p);
@@ -986,12 +1007,12 @@ void set_initial_rec() {
 }
 
 void write_recs() {
-    fprintf(debt_file, "%f ", gstate.now);
+    fprintf(rec_file, "%f ", gstate.now);
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
         PROJECT* p = gstate.projects[i];
-        fprintf(debt_file, "%f ", p->pwf.rec);
+        fprintf(rec_file, "%f ", p->pwf.rec);
     }
-    fprintf(debt_file, "\n");
+    fprintf(rec_file, "\n");
 }
 
 void make_graph(const char* title, const char* fname, int field) {
@@ -1008,7 +1029,7 @@ void make_graph(const char* title, const char* fname, int field) {
     );
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
         PROJECT* p = gstate.projects[i];
-        fprintf(f, "\"%sdebt.dat\" using 1:%d title \"%s\" with lines%s",
+        fprintf(f, "\"%srec.dat\" using 1:%d title \"%s\" with lines%s",
             outfile_prefix, 2+i+field, p->project_name,
             (i==gstate.projects.size()-1)?"\n":", \\\n"
         );
@@ -1027,12 +1048,10 @@ static void write_inputs() {
     fprintf(f,
         "Existing jobs only: %s\n"
         "Round-robin only: %s\n"
-        "scheduler EDF sim: %s\n"
-        "hysteresis work fetch: %s\n",
+        "scheduler EDF sim: %s\n",
         existing_jobs_only?"yes":"no",
         cpu_sched_rr_only?"yes":"no",
-        server_uses_workload?"yes":"no",
-        use_hyst_fetch?"yes":"no"
+        server_uses_workload?"yes":"no"
     );
     fprintf(f,
         "REC half-life: %f\n", config.rec_half_life
@@ -1067,13 +1086,11 @@ void simulate() {
         "   Scheduling period %f\n"
         "Scheduling policies\n"
         "   Round-robin only: %s\n"
-        "   Scheduler EDF simulation: %s\n"
-        "   Hysteresis work fetch: %s\n",
+        "   Scheduler EDF simulation: %s\n",
         gstate.work_buf_min(), gstate.work_buf_total(),
         gstate.global_prefs.cpu_scheduling_period(),
         cpu_sched_rr_only?"yes":"no",
-        server_uses_workload?"yes":"no",
-        use_hyst_fetch?"yes":"no"
+        server_uses_workload?"yes":"no"
     );
     fprintf(summary_file,
         "   REC half-life: %f\n", config.rec_half_life
@@ -1226,6 +1243,15 @@ void get_app_params() {
                 continue;
             }
 
+            if (app->non_cpu_intensive) {
+                fprintf(summary_file,
+                    "   app %s: ignoring - non CPU intensive\n",
+                    app->name
+                );
+                app->ignore = true;
+                continue;
+            }
+
             // if missing app params, fill in defaults
             //
             if (!app->fpops_est) {
@@ -1266,7 +1292,7 @@ void get_app_params() {
     );
 }
 
-// zero backoffs and debts.
+// zero backoffs and REC
 //
 void clear_backoff() {
     unsigned int i;
@@ -1289,15 +1315,11 @@ void cull_projects() {
     for (i=0; i<gstate.projects.size(); i++) {
         p = gstate.projects[i];
         p->no_apps = true;
-        for (int j=0; j<coprocs.n_rsc; j++) {
-            p->no_rsc_apps[j] = true;
-        }
     }
     for (i=0; i<gstate.app_versions.size(); i++) {
         APP_VERSION* avp = gstate.app_versions[i];
         if (avp->app->ignore) continue;
         int rt = avp->gpu_usage.rsc_type;
-        avp->project->no_rsc_apps[rt] = false;
     }
     for (i=0; i<gstate.apps.size(); i++) {
         APP* app = gstate.apps[i];
@@ -1305,21 +1327,48 @@ void cull_projects() {
             app->project->no_apps = false;
         }
     }
-    vector<PROJECT*>::iterator iter;
-    iter = gstate.projects.begin();
-    while (iter != gstate.projects.end()) {
-        p = *iter;
+    for (i=0; i<gstate.projects.size(); i++) {
+        p = gstate.projects[i];
         if (p->no_apps) {
             fprintf(summary_file,
                 "%s: Removing from simulation - no apps\n",
                 p->project_name
             );
-            iter = gstate.projects.erase(iter);
+            p->ignore = true;
         } else if (p->non_cpu_intensive) {
             fprintf(summary_file,
                 "%s: Removing from simulation - non CPU intensive\n",
                 p->project_name
             );
+            p->ignore = true;
+        }
+    }
+
+    // remove results and active tasks of projects we're culling
+    //
+    vector<ACTIVE_TASK*>::iterator ati = gstate.active_tasks.active_tasks.begin();
+    while (ati != gstate.active_tasks.active_tasks.end()) {
+        ACTIVE_TASK* atp = *ati;
+        if (atp->wup->project->ignore) {
+            ati = gstate.active_tasks.active_tasks.erase(ati);
+        } else {
+            ati++;
+        }
+    }
+    vector<RESULT*>::iterator ri = gstate.results.begin();
+    while (ri != gstate.results.end()) {
+        RESULT* rp = *ri;
+        if (rp->project->ignore) {
+            ri = gstate.results.erase(ri);
+        } else {
+            ri++;
+        }
+    }
+
+    vector<PROJECT*>::iterator iter = gstate.projects.begin();
+    while (iter != gstate.projects.end()) {
+        p = *iter;
+        if (p->ignore) {
             iter = gstate.projects.erase(iter);
         } else {
             iter++;
@@ -1360,6 +1409,8 @@ void do_client_simulation() {
         exit(1);
     }
 
+    // if tasks have pending transfers, mark as completed
+    //
     for (unsigned int i=0; i<gstate.results.size(); i++) {
         RESULT* rp = gstate.results[i];
         if (rp->state() < RESULT_FILES_DOWNLOADED) {
@@ -1409,7 +1460,7 @@ void do_client_simulation() {
 
     //set_initial_rec();
 
-    debt_adjust_period = delta;
+    rec_adjust_period = delta;
 
     gstate.request_work_fetch("init");
     simulate();
@@ -1446,7 +1497,7 @@ void do_client_simulation() {
     );
     print_project_results(summary_file);
 
-    fclose(debt_file);
+    fclose(rec_file);
     make_graph("REC", "rec", 0);
 }
 
@@ -1479,8 +1530,6 @@ int main(int argc, char** argv) {
             server_uses_workload = true;
         } else if (!strcmp(opt, "--cpu_sched_rr_only")) {
             cpu_sched_rr_only = true;
-        } else if (!strcmp(opt, "--use_hyst_fetch")) {
-            use_hyst_fetch = true;
         } else if (!strcmp(opt, "--rec_half_life")) {
             config.rec_half_life = atof(argv[i++]);
         } else {
@@ -1508,8 +1557,8 @@ int main(int argc, char** argv) {
     }
     setbuf(logfile, 0);
 
-    sprintf(buf, "%s%s", outfile_prefix, DEBT_FNAME);
-    debt_file = fopen(buf, "w");
+    sprintf(buf, "%s%s", outfile_prefix, REC_FNAME);
+    rec_file = fopen(buf, "w");
 
     sprintf(buf, "%s%s", outfile_prefix, SUMMARY_FNAME);
     summary_file = fopen(buf, "w");
