@@ -34,7 +34,7 @@ struct SIM_RESULTS {
     double idle_frac;
     int nrpcs;
 
-    void compute();
+    void compute_figures_of_merit();
     void print(FILE* f, bool human_readable=false);
     void parse(FILE* f);
     void add(SIM_RESULTS& r);
@@ -90,6 +90,7 @@ extern SIM_RESULTS sim_results;
 extern double calculate_exponential_backoff(
     int n, double MIN, double MAX
 );
+extern char* sim_time_string(int t);
 
 extern bool dcf_dont_use;
 extern bool dcf_stats;
