@@ -35,37 +35,9 @@
 
 using std::vector;
 
-bool use_hyst_fetch = true;
-
 RSC_WORK_FETCH rsc_work_fetch[MAX_RSC];
 WORK_FETCH work_fetch;
 
-#define FETCH_IF_IDLE_INSTANCE          0
-    // If resource has an idle instance,
-    // get work for it from the project with greatest LTD,
-    // even if it's overworked.
-#define FETCH_IF_MAJOR_SHORTFALL        1
-    // If resource is saturated for less than work_buf_min(),
-    // get work for it from the project with greatest LTD,
-    // even if it's overworked.
-#define FETCH_IF_MINOR_SHORTFALL        2
-    // If resource is saturated for less than work_buf_total(),
-    // get work for it from the non-overworked project with greatest LTD.
-#define FETCH_IF_PROJECT_STARVED        3
-    // If any project is not overworked and has too few jobs
-    // to use its instance share,
-    // get work from the one with greatest LTD.
-
-static const char* criterion_name(int criterion) {
-    switch (criterion) {
-    case FETCH_IF_IDLE_INSTANCE: return "idle instance";
-    case FETCH_IF_MAJOR_SHORTFALL: return "major shortfall";
-    case FETCH_IF_MINOR_SHORTFALL: return "minor shortfall";
-    case FETCH_IF_PROJECT_STARVED: return "starved";
-    }
-    return "unknown";
-}
-
 inline bool dont_fetch(PROJECT* p, int rsc_type) {
     if (p->no_rsc_pref[rsc_type]) return true;
     if (p->no_rsc_config[rsc_type]) return true;
@@ -143,13 +115,13 @@ bool RSC_PROJECT_WORK_FETCH::compute_may_have_work(PROJECT* p, int rsc_type) {
 void RSC_PROJECT_WORK_FETCH::rr_init(PROJECT* p, int rsc_type) {
     may_have_work = compute_may_have_work(p, rsc_type);
     fetchable_share = 0;
-    has_runnable_jobs = false;
+    n_runnable_jobs = 0;
     sim_nused = 0;
     nused_total = 0;
     deadlines_missed = 0;
 }
 
-void RSC_PROJECT_WORK_FETCH::backoff(PROJECT* p, const char* name) {
+void RSC_PROJECT_WORK_FETCH::resource_backoff(PROJECT* p, const char* name) {
     if (backoff_interval) {
         backoff_interval *= 2;
         if (backoff_interval > WF_MAX_BACKOFF_INTERVAL) backoff_interval = WF_MAX_BACKOFF_INTERVAL;
@@ -171,12 +143,6 @@ RSC_PROJECT_WORK_FETCH& RSC_WORK_FETCH::project_state(PROJECT* p) {
     return p->rsc_pwf[rsc_type];
 }
 
-bool RSC_WORK_FETCH::may_have_work(PROJECT* p) {
-    if (dont_fetch(p, rsc_type)) return false;
-    RSC_PROJECT_WORK_FETCH& w = project_state(p);
-    return (w.backoff_time < gstate.now);
-}
-
 void RSC_WORK_FETCH::rr_init() {
     shortfall = 0;
     nidle_now = 0;
@@ -185,25 +151,22 @@ void RSC_WORK_FETCH::rr_init() {
     deadline_missed_instances = 0;
     saturated_time = 0;
     busy_time_estimator.reset();
+    sim_used_instances = 0;
 }
 
-void RSC_WORK_FETCH::accumulate_shortfall(double d_time) {
+void RSC_WORK_FETCH::update_stats(double sim_now, double dt, double buf_end) {
     double idle = ninstances - sim_nused;
-    if (idle > 1e-6) {
-        //msg_printf(0, MSG_INFO, "adding shortfall %d %f", rsc_type, idle*d_time);
-        shortfall += idle*d_time;
+    if (idle > 1e-6 && sim_now < buf_end) {
+        double dt2;
+        if (sim_now + dt > buf_end) {
+            dt2 = buf_end - sim_now;
+        } else {
+            dt2 = dt;
+        }
+        shortfall += idle*dt2;
     }
-#if 0
-    msg_printf(0, MSG_INFO, "accum shortf (%s): idle %f dt %f sf %f",
-        rsc_name(rsc_type), idle, d_time, shortfall
-    );
-#endif
-}
-
-void RSC_WORK_FETCH::update_saturated_time(double dt) {
-    double idle = ninstances - sim_nused;
     if (idle < 1e-6) {
-        saturated_time = dt;
+        saturated_time = sim_now + dt - gstate.now;
     }
 }
 
@@ -220,158 +183,141 @@ static bool wacky_dcf(PROJECT* p) {
 // If this resource is below min buffer level,
 // return the highest-priority project that may have jobs for it.
 //
-PROJECT* RSC_WORK_FETCH::choose_project_hyst(bool enforce_hyst) {
+// It the resource has instanced starved because of exclusions,
+// return the highest-priority project that may have jobs
+// and doesn't exclude those instances.
+//
+// Only choose a project if the buffer is below min level;
+// if strict_hyst is true, relax this to max level
+//
+// If backoff_exempt_project is non-NULL,
+// don't enforce resource backoffs for that project;
+// this is for when we're going to do a scheduler RPC anyway
+// and we're deciding whether to piggyback a work request
+//
+PROJECT* RSC_WORK_FETCH::choose_project_hyst(
+    bool strict_hyst,
+    PROJECT* backoff_exempt_project
+) {
     PROJECT* pbest = NULL;
-    if (enforce_hyst) {
-        if (saturated_time > gstate.work_buf_min()) return NULL;
+    bool buffer_low = true;
+    if (strict_hyst) {
+        if (saturated_time > gstate.work_buf_min()) buffer_low = false;
+    } else {
+        if (saturated_time > gstate.work_buf_total()) buffer_low = false;
     }
-    if (saturated_time > gstate.work_buf_total()) return NULL;
+
+    if (log_flags.work_fetch_debug) {
+        msg_printf(0, MSG_INFO,
+            "[work_fetch] choose_project() for %s: buffer_low: %s; sim_excluded_instances %d\n",
+            rsc_name(rsc_type), buffer_low?"yes":"no", sim_excluded_instances
+        );
+    }
+
+    if (!buffer_low && !sim_excluded_instances) return NULL;
 
     for (unsigned i=0; i<gstate.projects.size(); i++) {
         PROJECT* p = gstate.projects[i];
-        if (!p->pwf.can_fetch_work) continue;
-        if (!project_state(p).may_have_work) continue;
+
+        // check whether we can fetch work of any type from this project
+        //
+        if (p->pwf.cant_fetch_work_reason) {
+            //msg_printf(p, MSG_INFO, "skip: cfwr %d", p->pwf.cant_fetch_work_reason);
+            continue;
+        }
+
+        // see whether work fetch for this resource is banned
+        // by prefs, config, project, or acct mgr
+        //
+        if (dont_fetch(p, rsc_type)) {
+            //msg_printf(p, MSG_INFO, "skip: dont_fetch");
+            continue;
+        }
+
+        // check backoff
+        //
+        if (p != backoff_exempt_project) {
+            if (project_state(p).backoff_time > gstate.now) {
+                //msg_printf(p, MSG_INFO, "skip: backoff");
+                continue;
+            }
+        }
 
         // if project has zero resource share,
         // only fetch work if a device is idle
         //
         if (p->resource_share == 0 && nidle_now == 0) {
+            //msg_printf(p, MSG_INFO, "skip: zero share");
             continue;
         }
 
         // if project has excluded GPUs of this type,
-        // and it has runnable jobs for this type,
-        // don't fetch work for it.
-        // TODO: THIS IS CRUDE. Making it smarter would require
+        // we need to avoid fetching work just because there's an idle instance
+        // or a shortfall;
+        // fetching work might not alleviate either of these,
+        // and we'd end up fetching unbounded work.
+        // At the same time, we want to respect work buf params if possible.
+        //
+        // Current policy:
+        // don't fetch work if remaining time of this project's jobs
+        // exceeds work_buf_min * (#usable instances / #instances)
+        //
+        // TODO: THIS IS FAIRLY CRUDE. Making it smarter would require
         // computing shortfall etc. on a per-project basis
         //
-        if (rsc_type) {
-            if (p->ncoprocs_excluded[rsc_type] == ninstances) {
-                continue;
-            }
-            if (p->ncoprocs_excluded[rsc_type]
-                && p->rsc_pwf[rsc_type].has_runnable_jobs
-            ){
+        int nexcl = p->rsc_pwf[rsc_type].ncoprocs_excluded;
+        if (rsc_type && nexcl) {
+            int n_not_excluded = ninstances - nexcl;
+            if (p->rsc_pwf[rsc_type].queue_est > (gstate.work_buf_min() * n_not_excluded)/ninstances) {
+                //msg_printf(p, MSG_INFO, "skip: too much work");
                 continue;
             }
         }
 
         RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
-        if (rpwf.anon_skip) continue;
-        if (pbest) {
-            if (pbest->sched_priority > p->sched_priority) {
-                continue;
-            }
-        }
-        pbest = p;
-    }
-    if (!pbest) return NULL;
-    work_fetch.clear_request();
-    work_fetch.set_all_requests_hyst(pbest, rsc_type);
-    return pbest;
-}
-
-// Choose the best project to ask for work for this resource,
-// given the specific criterion
-//
-PROJECT* RSC_WORK_FETCH::choose_project(int criterion) {
-    PROJECT* pbest = NULL;
-
-    switch (criterion) {
-    case FETCH_IF_IDLE_INSTANCE:
-        if (nidle_now == 0) return NULL;
-        break;
-    case FETCH_IF_MAJOR_SHORTFALL:
-        if (saturated_time > gstate.work_buf_min()) return NULL;
-        break;
-    case FETCH_IF_MINOR_SHORTFALL:
-        if (saturated_time > gstate.work_buf_total()) return NULL;
-        break;
-    case FETCH_IF_PROJECT_STARVED:
-        if (deadline_missed_instances >= ninstances) return NULL;
-        break;
-    }
-
-    for (unsigned i=0; i<gstate.projects.size(); i++) {
-        PROJECT* p = gstate.projects[i];
-        if (!p->pwf.can_fetch_work) continue;
-        if (!project_state(p).may_have_work) continue;
-        RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
-        if (rpwf.anon_skip) continue;
-        switch (criterion) {
-        case FETCH_IF_MINOR_SHORTFALL:
-            if (wacky_dcf(p)) continue;
-            if (!p->resource_share) continue;
-            break;
-        case FETCH_IF_MAJOR_SHORTFALL:
-            if (wacky_dcf(p)) continue;
-            if (!p->resource_share) continue;
-            break;
-        case FETCH_IF_PROJECT_STARVED:
-            if (p->sched_priority < 0) continue;
-            if (rpwf.nused_total >= ninstances) continue;
-            if (!p->resource_share) continue;
-            break;
+        if (rpwf.anon_skip) {
+            //msg_printf(p, MSG_INFO, "skip: anon");
+            continue;
         }
 
-        if (pbest) {
-            if (!p->resource_share) {
-                continue;
-            }
-            if (pbest->sched_priority > p->sched_priority) {
-                continue;
-            }
-        }
-        pbest = p;
-    }
-    if (!pbest) return NULL;
-
-    // decide how much work to request from each resource
-    //
-    work_fetch.clear_request();
-    switch (criterion) {
-    case FETCH_IF_IDLE_INSTANCE:
-    case FETCH_IF_MAJOR_SHORTFALL:
-        set_request(pbest);
-        break;
-    case FETCH_IF_PROJECT_STARVED:
-        set_request(pbest);
-        break;
-    case FETCH_IF_MINOR_SHORTFALL:
-        // in this case, potentially request work for all resources
+        // if we're sending work only because of exclusion starvation,
+        // make sure this project can use the starved instances
         //
-        if (pbest->sched_priority < 0) {
-            set_request(pbest);
-        } else {
-            work_fetch.set_all_requests(pbest);
+        if (!buffer_low) {
+            if ((sim_excluded_instances & rpwf.non_excluded_instances) == 0) {
+                //msg_printf(p, MSG_INFO, "skip: excl");
+                continue;
+            }
         }
-        break;
+
+        if (pbest) {
+            if (pbest->sched_priority > p->sched_priority) {
+                //msg_printf(p, MSG_INFO, "skip: prio");
+                continue;
+            }
+        }
+        pbest = p;
     }
-    // in principle there should be a nonzero request.
-    // check, just in case
-    //
-    if (!req_secs && !req_instances) {
+    if (!pbest) {
         if (log_flags.work_fetch_debug) {
-            msg_printf(pbest, MSG_INFO,
-                "[work_fetch] error: project chosen but zero request"
+            msg_printf(0, MSG_INFO,
+                "[work_fetch] no eligible project for %s",
+                rsc_name(rsc_type)
             );
         }
-        return 0;
+        return NULL;
     }
-
-    if (log_flags.work_fetch_debug) {
-        msg_printf(pbest, MSG_INFO,
-            "[work_fetch] chosen: %s %s: %.2f inst, %.2f sec",
-            criterion_name(criterion), rsc_name(rsc_type),
-            req_instances, req_secs
-        );
+    work_fetch.clear_request();
+    if (buffer_low) {
+        work_fetch.set_all_requests_hyst(pbest, rsc_type);
+    } else {
+        set_request_excluded(pbest);
     }
-
     return pbest;
 }
 
 // request this project's share of shortfall and instances.
-// don't request anything if project is overworked or backed off.
+// don't request anything if project is backed off.
 //
 void RSC_WORK_FETCH::set_request(PROJECT* p) {
     if (dont_fetch(p, rsc_type)) return;
@@ -391,6 +337,7 @@ void RSC_WORK_FETCH::set_request(PROJECT* p) {
     RSC_PROJECT_WORK_FETCH& w = project_state(p);
     if (!w.may_have_work) return;
     if (w.anon_skip) return;
+    double non_excl_inst = ninstances - w.ncoprocs_excluded;
     if (shortfall) {
         if (wacky_dcf(p)) {
             // if project's DCF is too big or small,
@@ -399,17 +346,24 @@ void RSC_WORK_FETCH::set_request(PROJECT* p) {
             req_secs = 1;
         } else {
             req_secs = shortfall;
+            if (w.ncoprocs_excluded) {
+                req_secs *= non_excl_inst/ninstances;
+            }
         }
     }
 
-    // our share of the idle instances
-    //
-    req_instances = nidle_now * w.fetchable_share;
+    double instance_share = ninstances*w.fetchable_share;
+    if (instance_share > non_excl_inst) {
+        instance_share = non_excl_inst;
+    }
+    instance_share -= w.nused_total;
+    req_instances = std::max(nidle_now, instance_share);
 
     if (log_flags.work_fetch_debug) {
-        msg_printf(0, MSG_INFO,
-            "[work_fetch] set_request(): ninst %d nused_total %f nidle_now %f fetch share %f req_inst %f",
-            ninstances, w.nused_total, nidle_now, w.fetchable_share, req_instances
+        msg_printf(p, MSG_INFO,
+            "[work_fetch] set_request() for %s: ninst %d nused_total %f nidle_now %f fetch share %f req_inst %f req_secs %f",
+            rsc_name(rsc_type), ninstances, w.nused_total, nidle_now,
+            w.fetchable_share, req_instances, req_secs
         );
     }
     if (req_instances && !req_secs) {
@@ -417,14 +371,38 @@ void RSC_WORK_FETCH::set_request(PROJECT* p) {
     }
 }
 
+// We're fetching work because some instances are starved because
+// of exclusions.
+// See how many N of these instances are not excluded for this project.
+// Ask for N instances and for N*work_buf_min seconds.
+//
+void RSC_WORK_FETCH::set_request_excluded(PROJECT* p) {
+    RSC_PROJECT_WORK_FETCH& pwf = project_state(p);
+
+    int inst_mask = sim_excluded_instances & pwf.non_excluded_instances;
+    int n = 0;
+    for (int i=0; i<ninstances; i++) {
+        if ((i<<i) & inst_mask) {
+            n++;
+        }
+    }
+    req_instances = n;
+    if (p->resource_share == 0 || config.fetch_minimal_work) {
+        req_secs = 1;
+    } else {
+        req_secs = n*gstate.work_buf_total();
+    }
+}
+
 void RSC_WORK_FETCH::print_state(const char* name) {
+    msg_printf(0, MSG_INFO, "[work_fetch] --- state for %s ---", name);
     msg_printf(0, MSG_INFO,
-        "[work_fetch] %s: shortfall %.2f nidle %.2f saturated %.2f busy %.2f",
-        name,
+        "[work_fetch] shortfall %.2f nidle %.2f saturated %.2f busy %.2f",
         shortfall, nidle_now, saturated_time,
         busy_time_estimator.get_busy_time()
     );
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
+        char buf[256];
         PROJECT* p = gstate.projects[i];
         if (p->non_cpu_intensive) continue;
         RSC_PROJECT_WORK_FETCH& pwf = project_state(p);
@@ -433,10 +411,17 @@ void RSC_WORK_FETCH::print_state(const char* name) {
         bool no_rsc_apps = p->no_rsc_apps[rsc_type];
         bool no_rsc_ams = p->no_rsc_ams[rsc_type];
         double bt = pwf.backoff_time>gstate.now?pwf.backoff_time-gstate.now:0;
+        if (bt) {
+            sprintf(buf, " (resource backoff: %.2f, inc %.2f)",
+                bt, pwf.backoff_interval
+            );
+        } else {
+            strcpy(buf, "");
+        }
         msg_printf(p, MSG_INFO,
-            "[work_fetch] %s: fetch share %.3f rsc backoff (dt %.2f, inc %.2f)%s%s%s%s",
-            name,
-            pwf.fetchable_share, bt, pwf.backoff_interval,
+            "[work_fetch] fetch share %.3f%s%s%s%s%s",
+            pwf.fetchable_share,
+            buf,
             no_rsc_pref?" (blocked by prefs)":"",
             no_rsc_apps?" (no apps)":"",
             no_rsc_ams?" (blocked by account manager)":"",
@@ -452,16 +437,19 @@ void RSC_WORK_FETCH::clear_request() {
 
 ///////////////  PROJECT_WORK_FETCH  ///////////////
 
-bool PROJECT_WORK_FETCH::compute_can_fetch_work(PROJECT* p) {
-    if (p->non_cpu_intensive) return false;
-    if (p->suspended_via_gui) return false;
-    if (p->master_url_fetch_pending) return false;
-    if (p->min_rpc_time > gstate.now) return false;
-    if (p->dont_request_more_work) return false;
-    if (p->some_download_stalled()) return false;
-    if (p->some_result_suspended()) return false;
-    if (p->too_many_uploading_results) return false;
-    return true;
+int PROJECT_WORK_FETCH::compute_cant_fetch_work_reason(PROJECT* p) {
+    if (p->non_cpu_intensive) return CANT_FETCH_WORK_NON_CPU_INTENSIVE;
+    if (p->suspended_via_gui) return CANT_FETCH_WORK_SUSPENDED_VIA_GUI;
+    if (p->master_url_fetch_pending) return CANT_FETCH_WORK_MASTER_URL_FETCH_PENDING;
+    if (p->dont_request_more_work) return CANT_FETCH_WORK_DONT_REQUEST_MORE_WORK;
+    if (p->some_download_stalled()) return CANT_FETCH_WORK_DOWNLOAD_STALLED;
+    if (p->some_result_suspended()) return CANT_FETCH_WORK_RESULT_SUSPENDED;
+    if (p->too_many_uploading_results) return CANT_FETCH_WORK_TOO_MANY_UPLOADS;
+
+    // this goes last
+    //
+    if (p->min_rpc_time > gstate.now) return CANT_FETCH_WORK_MIN_RPC_TIME;
+    return 0;
 }
 
 void PROJECT_WORK_FETCH::reset(PROJECT* p) {
@@ -472,14 +460,23 @@ void PROJECT_WORK_FETCH::reset(PROJECT* p) {
 
 ///////////////  WORK_FETCH  ///////////////
 
+// mark the projects from which we can fetch work
+//
+void WORK_FETCH::compute_cant_fetch_work_reason() {
+    for (unsigned int i=0; i<gstate.projects.size(); i++) {
+        PROJECT* p = gstate.projects[i];
+        p->pwf.cant_fetch_work_reason = p->pwf.compute_cant_fetch_work_reason(p);
+    }
+}
+
 void WORK_FETCH::rr_init() {
     for (int i=0; i<coprocs.n_rsc; i++) {
         rsc_work_fetch[i].rr_init();
     }
+    compute_cant_fetch_work_reason();
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
         PROJECT* p = gstate.projects[i];
-        p->pwf.can_fetch_work = p->pwf.compute_can_fetch_work(p);
-        p->pwf.has_runnable_jobs = false;
+        p->pwf.n_runnable_jobs = 0;
         for (int j=0; j<coprocs.n_rsc; j++) {
             p->rsc_pwf[j].rr_init(p, j);
         }
@@ -494,11 +491,17 @@ void RSC_WORK_FETCH::supplement(PROJECT* pp) {
     for (unsigned i=0; i<gstate.projects.size(); i++) {
         PROJECT* p = gstate.projects[i];
         if (p == pp) continue;
-        if (!p->pwf.can_fetch_work) continue;
+        if (p->pwf.cant_fetch_work_reason) continue;
         if (!project_state(p).may_have_work) continue;
         RSC_PROJECT_WORK_FETCH& rpwf = project_state(p);
         if (rpwf.anon_skip) continue;
         if (p->sched_priority > x) {
+            if (log_flags.work_fetch_debug) {
+                msg_printf(pp, MSG_INFO,
+                    "[work_fetch]: not requesting work for %s: %s has higher priority",
+                    rsc_name(rsc_type), p->get_project_name()
+                );
+            }
             return;
         }
     }
@@ -521,6 +524,12 @@ void WORK_FETCH::set_all_requests_hyst(PROJECT* p, int rsc_type) {
             if (rsc_work_fetch[i].saturated_time > gstate.work_buf_total()) {
                 continue;
             }
+            
+            // don't fetch work if backup project and no idle instances
+            //
+            if (p->resource_share==0 && rsc_work_fetch[i].nidle_now==0) {
+                continue;
+            }
 
             if (i>0 && !gpus_usable) {
                 continue;
@@ -543,23 +552,26 @@ void WORK_FETCH::print_state() {
     msg_printf(0, MSG_INFO, "[work_fetch] target work buffer: %.2f + %.2f sec",
         gstate.work_buf_min(), gstate.work_buf_additional()
     );
+    msg_printf(0, MSG_INFO, "[work_fetch] --- project states ---");
     for (unsigned int i=0; i<gstate.projects.size(); i++) {
-        char buf[256];
         PROJECT* p = gstate.projects[i];
-        if (p->non_cpu_intensive) continue;
-        if (p->min_rpc_time > gstate.now) {
-            sprintf(buf, " (project backoff %.2f)", p->min_rpc_time - gstate.now);
+        char buf[256];
+        if (p->pwf.cant_fetch_work_reason) {
+            sprintf(buf, "can't req work: %s",
+                cant_fetch_work_string(p->pwf.cant_fetch_work_reason)
+            );
         } else {
-            strcpy(buf, "");
+            strcpy(buf, "can req work");
         }
-        msg_printf(p, MSG_INFO, "[work_fetch] REC %.3f priority %.6f%s%s%s%s%s",
+        if (p->min_rpc_time > gstate.now) {
+            char buf2[256];
+            sprintf(buf2, " (backoff: %.2f sec)", p->min_rpc_time - gstate.now);
+            strcat(buf, buf2);
+        }
+        msg_printf(p, MSG_INFO, "[work_fetch] REC %.3f prio %.6f %s",
             p->pwf.rec,
             p->sched_priority,
-            buf,
-            p->suspended_via_gui?" (susp via GUI)":"",
-            p->master_url_fetch_pending?" (master fetch pending)":"",
-            p->dont_request_more_work?" (no new tasks)":"",
-            p->too_many_uploading_results?" (too many uploads)":""
+            buf
         );
     }
     for (int i=0; i<coprocs.n_rsc; i++) {
@@ -577,7 +589,7 @@ void WORK_FETCH::clear_request() {
 // we're going to contact this project for reasons other than work fetch;
 // decide if we should piggy-back a work fetch request.
 //
-void WORK_FETCH::compute_work_request(PROJECT* p) {
+void WORK_FETCH::piggyback_work_request(PROJECT* p) {
     clear_request();
     if (config.fetch_minimal_work && gstate.had_or_requested_work) return;
     if (p->dont_request_more_work) return;
@@ -588,8 +600,29 @@ void WORK_FETCH::compute_work_request(PROJECT* p) {
         return;
     }
 
-    PROJECT* bestp = choose_project(false);
+    // if project was updated from manager and config says so,
+    // always fetch work if needed
+    //
+    if (p->sched_rpc_pending && config.fetch_on_update) {
+        set_all_requests_hyst(p, -1);
+        return;
+    }
+    compute_cant_fetch_work_reason();
+    PROJECT* bestp = choose_project(false, p);
     if (p != bestp) {
+        if (p->pwf.cant_fetch_work_reason == 0) {
+            if (bestp) {
+                p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_NOT_HIGHEST_PRIORITY;
+                if (log_flags.work_fetch_debug) {
+                    msg_printf(0, MSG_INFO,
+                        "[work_fetch] not piggybacking work req: %s has higher priority",
+                        bestp->get_project_name()
+                    );
+                }
+            } else {
+                p->pwf.cant_fetch_work_reason = CANT_FETCH_WORK_DONT_NEED;
+            }
+        }
         clear_request();
     }
 }
@@ -611,9 +644,12 @@ PROJECT* WORK_FETCH::non_cpu_intensive_project_needing_work() {
 }
 
 // choose a project to fetch work from,
-// and set the request fields of resource objects
+// and set the request fields of resource objects.
 //
-PROJECT* WORK_FETCH::choose_project(bool enforce_hyst) {
+PROJECT* WORK_FETCH::choose_project(
+    bool strict_hyst,
+    PROJECT* backoff_exempt_project
+) {
     PROJECT* p;
 
     if (log_flags.work_fetch_debug) {
@@ -643,57 +679,15 @@ PROJECT* WORK_FETCH::choose_project(bool enforce_hyst) {
     }
 
     p = 0;
-if (use_hyst_fetch) {
     if (gpus_usable) {
         for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project_hyst(enforce_hyst);
+            p = rsc_work_fetch[i].choose_project_hyst(strict_hyst, backoff_exempt_project);
             if (p) break;
         }
     }
     if (!p) {
-        p = rsc_work_fetch[0].choose_project_hyst(enforce_hyst);
+        p = rsc_work_fetch[0].choose_project_hyst(strict_hyst, backoff_exempt_project);
     }
-} else {
-    if (gpus_usable) {
-        for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project(FETCH_IF_IDLE_INSTANCE);
-            if (p) break;
-        }
-    }
-    if (!p) {
-        p = rsc_work_fetch[0].choose_project(FETCH_IF_IDLE_INSTANCE);
-    }
-
-    if (!p && gpus_usable) {
-        for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project(FETCH_IF_MAJOR_SHORTFALL);
-            if (p) break;
-        }
-    }
-    if (!p) {
-        p = rsc_work_fetch[0].choose_project(FETCH_IF_MAJOR_SHORTFALL);
-    }
-    
-    if (!p && gpus_usable) {
-        for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project(FETCH_IF_MINOR_SHORTFALL);
-            if (p) break;
-        }
-    }
-    if (!p) {
-        p = rsc_work_fetch[0].choose_project(FETCH_IF_MINOR_SHORTFALL);
-    }
-
-    if (!p && gpus_usable) {
-        for (int i=1; i<coprocs.n_rsc; i++) {
-            p = rsc_work_fetch[i].choose_project(FETCH_IF_PROJECT_STARVED);
-            if (p) break;
-        }
-    }
-    if (!p) {
-        p = rsc_work_fetch[0].choose_project(FETCH_IF_PROJECT_STARVED);
-    }
-}
 
     if (log_flags.work_fetch_debug) {
         print_state();
@@ -709,13 +703,13 @@ void WORK_FETCH::accumulate_inst_sec(ACTIVE_TASK* atp, double dt) {
     APP_VERSION* avp = atp->result->avp;
     PROJECT* p = atp->result->project;
     double x = dt*avp->avg_ncpus;
-    p->rsc_pwf[0].secs_this_debt_interval += x;
-    rsc_work_fetch[0].secs_this_debt_interval += x;
+    p->rsc_pwf[0].secs_this_rec_interval += x;
+    rsc_work_fetch[0].secs_this_rec_interval += x;
     int rt = avp->gpu_usage.rsc_type;
     if (rt) {
         x = dt*avp->gpu_usage.usage;
-        p->rsc_pwf[rt].secs_this_debt_interval += x;
-        rsc_work_fetch[rt].secs_this_debt_interval += x;
+        p->rsc_pwf[rt].secs_this_rec_interval += x;
+        rsc_work_fetch[rt].secs_this_rec_interval += x;
     }
 }
 
@@ -727,7 +721,7 @@ void WORK_FETCH::compute_shares() {
     for (i=0; i<gstate.projects.size(); i++) {
         p = gstate.projects[i];
         if (p->non_cpu_intensive) continue;
-        if (!p->pwf.can_fetch_work) continue;
+        if (p->pwf.cant_fetch_work_reason) continue;
         for (int j=0; j<coprocs.n_rsc; j++) {
             if (p->rsc_pwf[j].may_have_work) {
                 rsc_work_fetch[j].total_fetchable_share += p->resource_share;
@@ -737,7 +731,7 @@ void WORK_FETCH::compute_shares() {
     for (i=0; i<gstate.projects.size(); i++) {
         p = gstate.projects[i];
         if (p->non_cpu_intensive) continue;
-        if (!p->pwf.can_fetch_work) continue;
+        if (p->pwf.cant_fetch_work_reason) continue;
         for (int j=0; j<coprocs.n_rsc; j++) {
             if (p->rsc_pwf[j].may_have_work) {
                 p->rsc_pwf[j].fetchable_share = rsc_work_fetch[j].total_fetchable_share?p->resource_share/rsc_work_fetch[j].total_fetchable_share:1;
@@ -812,7 +806,7 @@ void WORK_FETCH::handle_reply(
         if (p->sched_rpc_pending != RPC_REASON_PROJECT_REQ) {
             for (int i=0; i<coprocs.n_rsc; i++) {
                 if (rsc_work_fetch[i].req_secs) {
-                    p->rsc_pwf[i].backoff(p, rsc_name(i));
+                    p->rsc_pwf[i].resource_backoff(p, rsc_name(i));
                 }
             }
         }
@@ -834,9 +828,15 @@ void WORK_FETCH::handle_reply(
 // arrange to always get one job, even if we don't need it or can't handle it.
 // (this is probably what user wants)
 //
-void WORK_FETCH::set_initial_work_request() {
+void WORK_FETCH::set_initial_work_request(PROJECT* p) {
     for (int i=0; i<coprocs.n_rsc; i++) {
         rsc_work_fetch[i].req_secs = 1;
+        if (i) {
+            RSC_WORK_FETCH& rwf = rsc_work_fetch[i];
+            if (rwf.ninstances ==  p->rsc_pwf[i].ncoprocs_excluded) {
+                rsc_work_fetch[i].req_secs = 0;
+            }
+        }
         rsc_work_fetch[i].req_instances = 0;
         rsc_work_fetch[i].busy_time_estimator.reset();
     }
@@ -920,12 +920,11 @@ double ACTIVE_TASK::est_dur() {
     if (fraction_done <= 0) return wu_est;
     if (wu_est < elapsed_time) wu_est = elapsed_time;
     double frac_est = fraction_done_elapsed_time / fraction_done;
-    double fraction_left = 1-fraction_done;
-    double wu_weight = fraction_left * fraction_left * fraction_left;
-    double fd_weight = 1 - wu_weight;
+    double fd_weight = fraction_done * fraction_done;
+    double wu_weight = 1 - fd_weight;
     double x = fd_weight*frac_est + wu_weight*wu_est;
 #if 0
-    if (log_flags.rr_simulation) {
+    //if (log_flags.rr_simulation) {
         msg_printf(result->project, MSG_INFO,
             "[rr_sim] %s frac_est %f = %f/%f",
             result->name, frac_est, fraction_done_elapsed_time, fraction_done
@@ -934,7 +933,7 @@ double ACTIVE_TASK::est_dur() {
             "[rr_sim] %s dur: %.2f = %.3f*%.2f + %.3f*%.2f",
             result->name, x, fd_weight, frac_est, wu_weight, wu_est
         );
-    }
+    //}
 #endif
     return x;
 }
@@ -953,6 +952,12 @@ double CLIENT_STATE::overall_gpu_frac() {
     if (x > 1) x = 1;
     return x;
 }
+double CLIENT_STATE::overall_cpu_and_network_frac() {
+    double x = time_stats.on_frac * time_stats.cpu_and_network_available_frac;
+    if (x < 0.01) x = 0.01;
+    if (x > 1) x = 1;
+    return x;
+}
 
 // called when benchmarks change
 //
@@ -984,4 +989,3 @@ void CLIENT_STATE::generate_new_host_cpid() {
         }
     }
 }
-
