@@ -31,8 +31,8 @@
 #include "BOINCGUIApp.h"
 #include "SkinManager.h"
 #include "MainDocument.h"
-#include "BOINCWizards.h"
 #include "BOINCBaseWizard.h"
+#include "WizardAttach.h"
 #include "CompletionErrorPage.h"
 
 
@@ -196,7 +196,7 @@ void CCompletionErrorPage::OnPageChanged( wxWizardExEvent& event ) {
 
     if (IS_ATTACHTOPROJECTWIZARD()) {
         m_pTitleStaticCtrl->SetLabel(
-            _("Failed to attach to project")
+            _("Failed to add project")
         );
     } else if (IS_ACCOUNTMANAGERWIZARD()) {
         if (IS_ACCOUNTMANAGERUPDATEWIZARD()) {
@@ -209,7 +209,7 @@ void CCompletionErrorPage::OnPageChanged( wxWizardExEvent& event ) {
             );
         } else {
             m_pTitleStaticCtrl->SetLabel(
-                _("Failed to attach to account manager")
+                _("Failed to add account manager")
             );
         }
     } else {
@@ -218,7 +218,7 @@ void CCompletionErrorPage::OnPageChanged( wxWizardExEvent& event ) {
 
     if (m_pServerMessagesCtrl->GetLabel().IsEmpty()) {
         m_pDirectionsStaticCtrl->SetLabel(
-            _("An error has occurred;\ncheck Messages for details.\n\nClick Finish to close.")
+            _("An error has occurred;\ncheck the Event Log for details.\n\nClick Finish to close.")
         );
     } else {
         m_pDirectionsStaticCtrl->SetLabel(
