@@ -24,6 +24,7 @@
 #include "BOINCGUIApp.h"
 #include "MainDocument.h"
 #include "NoticeListCtrl.h"
+#include "BOINCInternetFSHandler.h"
 
 ////@begin XPM images
 ////@end XPM images
@@ -376,6 +377,11 @@ CNoticeListCtrl::CNoticeListCtrl( )
 CNoticeListCtrl::CNoticeListCtrl( wxWindow* parent )
 {
     Create( parent );
+    
+    wxFileSystemHandler *internetFSHandler = wxGetApp().GetInternetFSHandler();
+    if (internetFSHandler) {
+        ((CBOINCInternetFSHandler*)internetFSHandler)->SetAbortInternetIO(false);
+    }
 }
  
  
@@ -386,6 +392,11 @@ CNoticeListCtrl::~CNoticeListCtrl( )
         delete m_accessible;
     }
 #endif
+
+    wxFileSystemHandler *internetFSHandler = wxGetApp().GetInternetFSHandler();
+    if (internetFSHandler) {
+        ((CBOINCInternetFSHandler*)internetFSHandler)->SetAbortInternetIO(false);
+    }
 }
 
 
@@ -412,9 +423,11 @@ bool CNoticeListCtrl::Create( wxWindow* parent )
 #endif
 ////@end CNoticeListCtrl creation
 
-    // Display the empty notice notification until we have some
-    // notices to display.
+    // Display the fetching notices message until we have notices
+    // to display or have determined that there are no notices.
+    m_bDisplayFetchingNotices = false;
     m_bDisplayEmptyNotice = true;
+    m_bNeedsReloading = false;
 
     return TRUE;
 }
@@ -475,12 +488,7 @@ wxString CNoticeListCtrl::OnGetItem(size_t i) const {
     wxASSERT(pDoc);
     wxASSERT(wxDynamicCast(pDoc, CMainDocument));
 
-
-    if (m_bDisplayEmptyNotice) {
-        strBuffer = wxT("<table border=0 cellpadding=5><tr><td>");
-        strBuffer += _("There are no notices at this time.");
-        strBuffer += wxT("</font></td></tr></table><hr>");
-    } else {
+    if (pDoc->IsConnected()) {
         NOTICE* np = pDoc->notice((unsigned int)i);
 
         strURL = wxString(np->link, wxConvUTF8);
@@ -541,12 +549,18 @@ wxString CNoticeListCtrl::OnGetItem(size_t i) const {
 }
 
 
+void CNoticeListCtrl::Clear() {
+    m_bNeedsReloading = true;
+    UpdateUI();
+}
+
+
 /*!
  * Update the UI.
  */
- 
 bool CNoticeListCtrl::UpdateUI()
 {
+    static bool bAlreadyRunning = false;
     CMainDocument*  pDoc   = wxGetApp().GetDocument();
 
     wxASSERT(pDoc);
@@ -554,33 +568,57 @@ bool CNoticeListCtrl::UpdateUI()
 
     // Call Freeze() / Thaw() only when actually needed; 
     // otherwise it causes unnecessary redraws
-    if (pDoc->GetNoticeCount() <= 0) {
-        m_bDisplayEmptyNotice = true;
-        Freeze();
-        SetItemCount(1);
-        Thaw();
+    int noticeCount = pDoc->GetNoticeCount();
+    if ((noticeCount < 0) || (!pDoc->IsConnected()) || m_bNeedsReloading) {
+        if (GetItemCount()) {
+            SetItemCount(0);
+            Refresh();
+        }
+        // Display "Fetching Notices" text only when connected
+        m_bDisplayFetchingNotices = pDoc->IsConnected();
+        m_bDisplayEmptyNotice = false;
+        m_bNeedsReloading = false;
         return true;
     }
     
-    if (
-        pDoc->notices.complete ||
-        ((int)GetItemCount() != pDoc->GetNoticeCount()) ||
-        ((pDoc->GetNoticeCount() > 0) && (m_bDisplayEmptyNotice == true))
-    ) {
-        pDoc->notices.complete = false;
-        m_bDisplayEmptyNotice = false;
-        Freeze();
-        SetItemCount(pDoc->GetNoticeCount());
-        Thaw();
+    if (noticeCount == 0) {
+        if (GetItemCount()) {
+            SetItemCount(0);
+            Refresh();
+        }
+        m_bDisplayFetchingNotices = false;
+        m_bDisplayEmptyNotice = true;
+        m_bNeedsReloading = false;
+        return true;
     }
-
-#ifdef __WXMAC__
-    // Enable accessibility only after drawing the page 
-    // to avoid a mysterious crash bug
-    if (m_accessible == NULL) {
-        m_accessible = new CNoticeListCtrlAccessible(this);
-    }
-#endif
     
+    // We must prevent re-entry because our asynchronous 
+    // Internet access on Windows calls Yield() which can 
+    // allow this to be called again.
+    if (!bAlreadyRunning) {
+        bAlreadyRunning = true;
+        if (
+            pDoc->IsConnected() &&
+            (pDoc->notices.complete ||
+            ((int)GetItemCount() != noticeCount))
+        ) {
+            pDoc->notices.complete = false;
+            Freeze();
+            SetItemCount(noticeCount);
+            m_bDisplayFetchingNotices = false;
+            m_bDisplayEmptyNotice = false;
+            Thaw();
+        }
+#ifdef __WXMAC__
+        // Enable accessibility only after drawing the page 
+        // to avoid a mysterious crash bug
+        if (m_accessible == NULL) {
+            m_accessible = new CNoticeListCtrlAccessible(this);
+        }
+#endif
+
+        bAlreadyRunning = false;
+    }
+        
     return true;
 }
