@@ -61,7 +61,7 @@ public:
     virtual wxString&       GetViewName();
     virtual wxString&       GetViewDisplayName();
     virtual const char**    GetViewIcon();
-    virtual const int       GetViewCurrentViewPage();
+    virtual int             GetViewCurrentViewPage();
 
     virtual wxString        GetKeyValue1(int iRowIndex);
     virtual wxString        GetKeyValue2(int iRowIndex);
