@@ -28,7 +28,9 @@
 
 
 #define SORTTASKLIST 1  /* TRUE to sort task selection control alphabetically */
-#define SLIDESHOWBORDER 3
+#define SLIDESHOWWIDTH 290
+#define SLIDESHOWHEIGHT 126
+#define SLIDESHOWBORDER 1
 #define DESCRIPTIONSPACER 4
 #define HIDEDEFAULTSLIDE 1
 #define TESTALLDESCRIPTIONS 0
@@ -40,7 +42,7 @@ enum { suspendedIcon, waitingIcon, runningIcon };
 IMPLEMENT_DYNAMIC_CLASS(CScrolledTextBox, wxScrolledWindow)
 
 BEGIN_EVENT_TABLE(CScrolledTextBox, wxScrolledWindow)
-	EVT_ERASE_BACKGROUND(CScrolledTextBox::OnEraseBackground)
+    EVT_ERASE_BACKGROUND(CScrolledTextBox::OnEraseBackground)
 END_EVENT_TABLE()
 
 CScrolledTextBox::CScrolledTextBox() {
@@ -50,13 +52,13 @@ CScrolledTextBox::CScrolledTextBox() {
 CScrolledTextBox::CScrolledTextBox( wxWindow* parent) :
     wxScrolledWindow( parent, ID_SGPROJECTDESCRIPTION, wxDefaultPosition, wxDefaultSize, wxVSCROLL)
 {
-	SetForegroundColour(*wxBLACK);
+    SetForegroundColour(*wxBLACK);
 
-	m_TextSizer = new wxBoxSizer( wxVERTICAL );
+    m_TextSizer = new wxBoxSizer( wxVERTICAL );
 
-	this->SetSizerAndFit( m_TextSizer );
-	this->Layout();
-	this->FitInside();
+    this->SetSizerAndFit( m_TextSizer );
+    this->Layout();
+    this->FitInside();
 }
 
 
@@ -85,7 +87,7 @@ void CScrolledTextBox::SetValue(const wxString& s) {
     SetScrollRate(1, lineHeight);
 }
 
-		
+        
 void CScrolledTextBox::OnEraseBackground(wxEraseEvent& event) {
     wxDC *dc = event.GetDC();
     wxPoint p = GetParent()->GetPosition();
@@ -173,8 +175,8 @@ int CScrolledTextBox::Wrap(const wxString& text, int widthMax, int *lineHeight)
 IMPLEMENT_DYNAMIC_CLASS(CSlideShowPanel, wxPanel)
 
 BEGIN_EVENT_TABLE(CSlideShowPanel, wxPanel)
-	EVT_ERASE_BACKGROUND(CSlideShowPanel::OnEraseBackground)
-	EVT_TIMER(ID_CHANGE_SLIDE_TIMER, CSlideShowPanel::OnSlideShowTimer)
+    EVT_ERASE_BACKGROUND(CSlideShowPanel::OnEraseBackground)
+    EVT_TIMER(ID_CHANGE_SLIDE_TIMER, CSlideShowPanel::OnSlideShowTimer)
     EVT_PAINT(CSlideShowPanel::OnPaint)
 END_EVENT_TABLE()
 
@@ -183,27 +185,29 @@ CSlideShowPanel::CSlideShowPanel() {
 
 
 CSlideShowPanel::CSlideShowPanel( wxWindow* parent ) :
-    wxPanel( parent, wxID_ANY, wxDefaultPosition, wxSize(290+(2*SLIDESHOWBORDER), 126+(2*SLIDESHOWBORDER)), wxBORDER_NONE )
+    wxPanel( parent, wxID_ANY, wxDefaultPosition,
+            wxSize(SLIDESHOWWIDTH+(2*SLIDESHOWBORDER),
+            SLIDESHOWHEIGHT+(2*SLIDESHOWBORDER)), wxBORDER_NONE )
 {
     int w, h;
-	wxBoxSizer* bSizer1;
-	bSizer1 = new wxBoxSizer( wxVERTICAL );
+    wxBoxSizer* bSizer1;
+    bSizer1 = new wxBoxSizer( wxVERTICAL );
 
     m_institution = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
-	bSizer1->Add( m_institution, 0, 0, 0 );
+    bSizer1->Add( m_institution, 0, 0, 0 );
 
     m_scienceArea = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
-	bSizer1->Add( m_scienceArea, 0, 0, 0 );
+    bSizer1->Add( m_scienceArea, 0, 0, 0 );
     
     bSizer1->AddSpacer(DESCRIPTIONSPACER);
 
     m_description = new CScrolledTextBox( this );
     GetSize(&w, &h);
     m_description->SetMinSize(wxSize(w, h - (2 * GetCharHeight()) - DESCRIPTIONSPACER));
-	bSizer1->Add( m_description, 1, wxEXPAND, 0 );
+    bSizer1->Add( m_description, 1, wxEXPAND, 0 );
 
-	this->SetSizer( bSizer1 );
-	this->Layout();
+    this->SetSizer( bSizer1 );
+    this->Layout();
 
     m_SlideBitmap = wxNullBitmap;
     m_bCurrentSlideIsDefault = false;
@@ -215,15 +219,15 @@ CSlideShowPanel::CSlideShowPanel( wxWindow* parent ) :
 #endif    
 
     m_ChangeSlideTimer = new wxTimer(this, ID_CHANGE_SLIDE_TIMER);
-	m_ChangeSlideTimer->Start(10000);
+    m_ChangeSlideTimer->Start(10000);
 }
 
 CSlideShowPanel::~CSlideShowPanel()
 {
-	if ( m_ChangeSlideTimer->IsRunning() ) {
-		m_ChangeSlideTimer->Stop();
-	}
-	delete m_ChangeSlideTimer;
+    if ( m_ChangeSlideTimer->IsRunning() ) {
+        m_ChangeSlideTimer->Stop();
+    }
+    delete m_ChangeSlideTimer;
 }
 
 
@@ -233,7 +237,7 @@ void CSlideShowPanel::OnSlideShowTimer(wxTimerEvent& WXUNUSED(event)) {
 
 
 void CSlideShowPanel::AdvanceSlideShow(bool changeSlide, bool reload) {
-	double xRatio, yRatio, ratio;
+    double xRatio, yRatio, ratio;
     unsigned int i;
     wxString s;
     TaskSelectionData* selData = ((CSimpleTaskPanel*)GetParent())->GetTaskSelectionData();
@@ -341,23 +345,23 @@ numSlides = 0;
         }
     }
     if (m_SlideBitmap.Ok()) {
-		// Check to see if they need to be rescaled to fit in the window
-		ratio = 1.0;
-		xRatio = ((double) GetSize().GetWidth() - (2*SLIDESHOWBORDER))/((double) m_SlideBitmap.GetWidth());
-		yRatio = ((double) GetSize().GetHeight() - (2*SLIDESHOWBORDER))/((double) m_SlideBitmap.GetHeight());
-		if ( xRatio < ratio ) {
-			ratio = xRatio;
-		}
-		if ( yRatio < ratio ) {
-			ratio = yRatio;
-		}
-		if ( ratio < 1.0 ) {
-			wxImage img = m_SlideBitmap.ConvertToImage();
-			img.Rescale((int) (m_SlideBitmap.GetWidth()*ratio), (int) (m_SlideBitmap.GetHeight()*ratio));
+        // Check to see if they need to be rescaled to fit in the window
+        ratio = 1.0;
+        xRatio = (double)SLIDESHOWWIDTH / (double)m_SlideBitmap.GetWidth();
+        yRatio = (double)SLIDESHOWHEIGHT / (double)m_SlideBitmap.GetHeight();
+        if ( xRatio < ratio ) {
+            ratio = xRatio;
+        }
+        if ( yRatio < ratio ) {
+            ratio = yRatio;
+        }
+        if ( ratio < 1.0 ) {
+            wxImage img = m_SlideBitmap.ConvertToImage();
+            img.Rescale((int) (m_SlideBitmap.GetWidth()*ratio), (int) (m_SlideBitmap.GetHeight()*ratio));
             wxBitmap *bm = new wxBitmap(img);
-			m_SlideBitmap = *bm;
+            m_SlideBitmap = *bm;
             delete bm;
-		}
+        }
         Refresh();
     }
 }
@@ -379,26 +383,27 @@ numSlides = 0;
     if (numSlides > 0)
 #endif  // HIDEDEFAULTSLIDE
     {
-        int w, h;
+        int w, h, x;
         wxPen oldPen = dc.GetPen();
         wxBrush oldBrush = dc.GetBrush();
         int oldMode = dc.GetBackgroundMode();
-        wxPen bgPen(*wxLIGHT_GREY, 3);
+        wxPen bgPen(*wxBLACK, 2*SLIDESHOWBORDER+1);
         dc.SetBackgroundMode(wxSOLID);
         dc.SetPen(bgPen);
         dc.SetBrush(*wxBLACK_BRUSH);
         
         GetSize(&w, &h);
-        dc.DrawRectangle(0, 0, w, h);
+        x = (w - SLIDESHOWWIDTH) / 2;
+        dc.DrawRectangle(x, SLIDESHOWBORDER, SLIDESHOWWIDTH, SLIDESHOWHEIGHT);
         // Restore Mode, Pen and Brush 
         dc.SetBackgroundMode(oldMode);
         dc.SetPen(oldPen);
         dc.SetBrush(oldBrush);
         
         if(m_SlideBitmap.Ok()) 
-        { 
-            dc.DrawBitmap(m_SlideBitmap, 
-                            (w - m_SlideBitmap.GetWidth())/2, 
+        {
+            dc.DrawBitmap(m_SlideBitmap,
+                            (w - m_SlideBitmap.GetWidth())/2,
                             (h - m_SlideBitmap.GetHeight())/2
                             ); 
         }
@@ -444,111 +449,111 @@ CSimpleTaskPanel::CSimpleTaskPanel( wxWindow* parent ) :
     m_sNotAvailableString = _("Not available");
     m_progressBarRect = NULL;
 
-	SetForegroundColour(*wxBLACK);
+    SetForegroundColour(*wxBLACK);
     
-	wxBoxSizer* bSizer1;
-	bSizer1 = new wxBoxSizer( wxVERTICAL );
+    wxBoxSizer* bSizer1;
+    bSizer1 = new wxBoxSizer( wxVERTICAL );
 
-	wxBoxSizer* bSizer2;
-	bSizer2 = new wxBoxSizer( wxHORIZONTAL );
-	
-	m_myTasksLabel = new CTransparentStaticText( this, wxID_ANY, _("Tasks:"), wxDefaultPosition, wxDefaultSize, 0 );
-	m_myTasksLabel->Wrap( -1 );
-	bSizer2->Add( m_myTasksLabel, 0, wxRIGHT, 5 );
-	
-	m_TaskSelectionCtrl = new CBOINCBitmapComboBox( this, ID_SGTASKSELECTOR, wxT(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY ); 
+    wxBoxSizer* bSizer2;
+    bSizer2 = new wxBoxSizer( wxHORIZONTAL );
+    
+    m_myTasksLabel = new CTransparentStaticText( this, wxID_ANY, _("Tasks:"), wxDefaultPosition, wxDefaultSize, 0 );
+    m_myTasksLabel->Wrap( -1 );
+    bSizer2->Add( m_myTasksLabel, 0, wxRIGHT, 5 );
+    
+    m_TaskSelectionCtrl = new CBOINCBitmapComboBox( this, ID_SGTASKSELECTOR, wxT(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY ); 
     // TODO: Might want better wording for Task Selection Combo Box tooltip
     str = _("Select a task to access");
     m_TaskSelectionCtrl->SetToolTip(str);
-	bSizer2->Add( m_TaskSelectionCtrl, 1, wxRIGHT | wxEXPAND, SIDEMARGINS );
-	
-	bSizer1->Add( bSizer2, 0, wxEXPAND | wxTOP | wxLEFT, 10 );
-	
+    bSizer2->Add( m_TaskSelectionCtrl, 1, wxRIGHT | wxEXPAND, SIDEMARGINS );
+    
+    bSizer1->Add( bSizer2, 0, wxEXPAND | wxTOP | wxLEFT, 10 );
+    
     bSizer1->AddSpacer(5);
-	
-	wxBoxSizer* bSizer3;
-	bSizer3 = new wxBoxSizer( wxHORIZONTAL );
-	
-	m_TaskProjectLabel = new CTransparentStaticText( this, wxID_ANY, _("From:"), wxDefaultPosition, wxDefaultSize, 0 );
-	m_TaskProjectLabel->Wrap( -1 );
-	bSizer3->Add( m_TaskProjectLabel, 0, wxRIGHT, 5 );
-	
-	m_TaskProjectName = new CTransparentStaticText( this, wxID_ANY, wxT("SETI@home"), wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
-	m_TaskProjectName->Wrap( -1 );
+    
+    wxBoxSizer* bSizer3;
+    bSizer3 = new wxBoxSizer( wxHORIZONTAL );
+    
+    m_TaskProjectLabel = new CTransparentStaticText( this, wxID_ANY, _("From:"), wxDefaultPosition, wxDefaultSize, 0 );
+    m_TaskProjectLabel->Wrap( -1 );
+    bSizer3->Add( m_TaskProjectLabel, 0, wxRIGHT, 5 );
+    
+    m_TaskProjectName = new CTransparentStaticText( this, wxID_ANY, wxT("SETI@home"), wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
+    m_TaskProjectName->Wrap( -1 );
     wxFont theFont = m_TaskProjectName->GetFont();
     theFont.SetWeight(wxFONTWEIGHT_BOLD);
     m_TaskProjectName->SetFont(theFont); 
-	bSizer3->Add( m_TaskProjectName, 1, 0, 0 );
-	
-	bSizer1->Add( bSizer3, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    bSizer3->Add( m_TaskProjectName, 1, 0, 0 );
+    
+    bSizer1->Add( bSizer3, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
 
 #if SELECTBYRESULTNAME
-	m_TaskApplicationName = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
-	m_TaskApplicationName->Wrap( -1 );
+    m_TaskApplicationName = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
+    m_TaskApplicationName->Wrap( -1 );
 
-	bSizer1->Add( m_TaskApplicationName, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    bSizer1->Add( m_TaskApplicationName, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
 #endif  // SELECTBYRESULTNAME
 
     bSizer1->AddSpacer(10);
-	
-    m_SlideShowArea = new CSlideShowPanel(this);
-    m_SlideShowArea->SetMinSize(wxSize(290+(2*SLIDESHOWBORDER), 126+(2*SLIDESHOWBORDER)));
-	m_SlideShowArea->Enable( false );
     
-	bSizer1->Add( m_SlideShowArea, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    m_SlideShowArea = new CSlideShowPanel(this);
+    m_SlideShowArea->SetMinSize(wxSize(SLIDESHOWWIDTH+(2*SLIDESHOWBORDER), SLIDESHOWHEIGHT+(2*SLIDESHOWBORDER)));
+    m_SlideShowArea->Enable( false );
+    
+    bSizer1->Add( m_SlideShowArea, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
 
     bSizer1->AddSpacer(10);
-	
-	m_ElapsedTimeValue = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
-	m_ElapsedTimeValue->Wrap( -1 );
-	bSizer1->Add( m_ElapsedTimeValue, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
-	
+    
+    m_ElapsedTimeValue = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
+    m_ElapsedTimeValue->Wrap( -1 );
+    bSizer1->Add( m_ElapsedTimeValue, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    
     bSizer1->AddSpacer(7);
-	
-	m_TimeRemainingValue = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
-	m_TimeRemainingValue->Wrap( -1 );
-	bSizer1->Add( m_TimeRemainingValue, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
-	
+    
+    m_TimeRemainingValue = new CTransparentStaticText( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
+    m_TimeRemainingValue->Wrap( -1 );
+    bSizer1->Add( m_TimeRemainingValue, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    
     bSizer1->AddSpacer(7);
-	
-	wxBoxSizer* bSizer4;
-	bSizer4 = new wxBoxSizer( wxHORIZONTAL );
-	
+    
+    wxBoxSizer* bSizer4;
+    bSizer4 = new wxBoxSizer( wxHORIZONTAL );
+    
     // TODO: Standard Mac progress indicator's animation uses lots of CPU 
     // time, and also triggers unnecessary Erase events.  Should we use a 
     // non-standard progress indicator on Mac?  See also optimizations in 
     // CSimpleGUIPanel::OnEraseBackground and CSimpleTaskPanel::OnEraseBackground.
-	m_ProgressBar = new wxGauge( this, wxID_ANY, 100, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL );
+    m_ProgressBar = new wxGauge( this, wxID_ANY, 100, wxDefaultPosition, wxDefaultSize, wxGA_HORIZONTAL );
     m_iPctDoneX10 = 1000;
-	m_ProgressBar->SetValue( 100 );
+    m_ProgressBar->SetValue( 100 );
     GetTextExtent(wxT("0"), &w, &h);
     m_ProgressBar->SetMinSize(wxSize(245, h));
     m_ProgressBar->SetToolTip(_("This task's progress"));
-	bSizer4->Add( m_ProgressBar, 0, wxRIGHT, 5 );
-	
-	m_ProgressValueText = new CTransparentStaticText( this, wxID_ANY, wxT("100.000%"), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT | wxST_NO_AUTORESIZE );
-	m_ProgressValueText->Wrap( -1 );
-	bSizer4->Add( m_ProgressValueText, 0, wxALIGN_RIGHT, 0 );
-	
-	bSizer1->Add( bSizer4, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
-	
+    bSizer4->Add( m_ProgressBar, 0, wxRIGHT, 5 );
+    
+    m_ProgressValueText = new CTransparentStaticText( this, wxID_ANY, wxT("100.000%"), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT | wxST_NO_AUTORESIZE );
+    m_ProgressValueText->Wrap( -1 );
+    bSizer4->Add( m_ProgressValueText, 0, wxALIGN_RIGHT, 0 );
+    
+    bSizer1->Add( bSizer4, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    
     bSizer1->AddSpacer(7);
-	
+    
     // TODO: Can we determine the longest status string and initialize with it?
-	m_StatusValueText = new CTransparentStaticText( this, wxID_ANY, m_sNoProjectsString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
-	m_StatusValueText->Wrap( -1 );
-	bSizer1->Add( m_StatusValueText, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
+    m_StatusValueText = new CTransparentStaticText( this, wxID_ANY, m_sNoProjectsString, wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE );
+    m_StatusValueText->Wrap( -1 );
+    bSizer1->Add( m_StatusValueText, 0, wxLEFT | wxRIGHT | wxEXPAND, SIDEMARGINS );
 
     bSizer1->AddSpacer(7);
-	
-	m_TaskCommandsButton = new CSimpleTaskPopupButton( this, ID_TASKSCOMMANDBUTTON, _("Task Commands"), wxDefaultPosition, wxDefaultSize, 0 );
+    
+    m_TaskCommandsButton = new CSimpleTaskPopupButton( this, ID_TASKSCOMMANDBUTTON, _("Task Commands"), wxDefaultPosition, wxDefaultSize, 0 );
     m_TaskCommandsButton->SetToolTip(_("Pop up a menu of commands to apply to this task"));
-	bSizer1->Add( m_TaskCommandsButton, 0, wxLEFT | wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, SIDEMARGINS );
+    bSizer1->Add( m_TaskCommandsButton, 0, wxLEFT | wxRIGHT | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, SIDEMARGINS );
     
     bSizer1->AddSpacer(10);
-	
-	this->SetSizer( bSizer1 );
-	this->Layout();
+    
+    this->SetSizer( bSizer1 );
+    this->Layout();
 
     m_ProgressRect = m_ProgressBar->GetRect();
 #ifdef __WXMAC__
@@ -562,13 +567,13 @@ CSimpleTaskPanel::~CSimpleTaskPanel()
 {
     TaskSelectionData *selData;
     int count = m_TaskSelectionCtrl->GetCount();
-	for(int j = count-1; j >=0; --j) {
+    for(int j = count-1; j >=0; --j) {
         selData = (TaskSelectionData*)m_TaskSelectionCtrl->GetClientData(j);
         selData->slideShowFileNames.Clear();
         delete selData;
         // Indicate to Clear() we have cleaned up the Selection Data
         m_TaskSelectionCtrl->SetClientData(j, NULL);
-	}
+    }
     m_TaskSelectionCtrl->Clear();
         
     if (m_progressBarRect) {
@@ -597,7 +602,7 @@ void CSimpleTaskPanel::UpdatePanel(bool delayShow) {
     int                 workCount = pDoc->GetSimpleGUIWorkCount();
 
     // Workaround for Linux refresh problem
-    static bool		wasDelayed = false;
+    static bool        wasDelayed = false;
 
 #ifndef __WXMAC__
     Freeze();
@@ -789,11 +794,10 @@ void CSimpleTaskPanel::GetApplicationAndProjectNames(RESULT* result, wxString* a
             strAppBuffer = wxString(state_result->avp->app_name, wxConvUTF8);
         }
         
-        if (avp->ncudas) {
-            strGPUBuffer = wxString(" (NVIDIA GPU)", wxConvUTF8);
-        }
-        if (avp->natis) {
-            strGPUBuffer = wxString(" (ATI GPU)", wxConvUTF8);
+        char buf[256];
+        if (avp->gpu_type) {
+            sprintf(buf, " (%s)", proc_type_name(avp->gpu_type));
+            strGPUBuffer = wxString(buf, wxConvUTF8);
         }
 
         appName->Printf(
@@ -861,7 +865,7 @@ wxString CSimpleTaskPanel::FormatTime(float fBuffer) {
     wxInt32        iMin = 0;
     wxInt32        iSec = 0;
     wxTimeSpan     ts;
-	wxString strBuffer= wxEmptyString;
+    wxString strBuffer= wxEmptyString;
 
     if (0 >= fBuffer) {
         strBuffer = wxT("---");
@@ -879,10 +883,10 @@ wxString CSimpleTaskPanel::FormatTime(float fBuffer) {
 
 
 void CSimpleTaskPanel::FindSlideShowFiles(TaskSelectionData *selData) {
-	RESULT* state_result;
+    RESULT* state_result;
     char urlDirectory[1024];
-	char fileName[1024];
-	char resolvedFileName[1024];
+    char fileName[1024];
+    char resolvedFileName[1024];
     int j;
     CMainDocument*      pDoc = wxGetApp().GetDocument();
     wxASSERT(pDoc);
@@ -928,8 +932,8 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
     wxLogTrace(wxT("Function Start/End"), wxT("CSimpleTaskPanel::UpdateTaskSelectionList - Function Begin"));
     int i, j, count, newIcon;
     TaskSelectionData *selData;
-	RESULT* result;
-	RESULT* ctrlResult;
+    RESULT* result;
+    RESULT* ctrlResult;
     PROJECT* project;
     std::vector<bool>is_alive;
     bool needRefresh = false;
@@ -937,25 +941,32 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
     CMainDocument*      pDoc = wxGetApp().GetDocument();
     CSkinSimple* pSkinSimple = wxGetApp().GetSkinManager()->GetSimple();
 
+    static bool bAlreadyRunning = false;
+
     wxASSERT(pDoc);
     wxASSERT(pSkinSimple);
     wxASSERT(wxDynamicCast(pSkinSimple, CSkinSimple));
     
+    if (bAlreadyRunning) {
+        return;
+    }
+    bAlreadyRunning = true;
+    
     count = m_TaskSelectionCtrl->GetCount();
-	// Mark all inactive (this lets us loop only once)
+    // Mark all inactive (this lets us loop only once)
     for (i=0; i<count; ++i) {
         is_alive.push_back(false);
     }
     
-	// First update existing entries and add new ones
-	for(int i = 0; i < (int) pDoc->results.results.size(); i++) {
-		bool found = false;
+    // First update existing entries and add new ones
+    for(i = 0; i < (int) pDoc->results.results.size(); i++) {
+        bool found = false;
         
-		result = pDoc->result(i);
-		// only check tasks that are active
-		if ( result == NULL || !result->active_task ) {
-			continue;
-		}
+        result = pDoc->result(i);
+        // only check tasks that are active
+        if ( result == NULL || !result->active_task ) {
+            continue;
+        }
 
         resname = wxEmptyString;
 #if SELECTBYRESULTNAME
@@ -964,24 +975,24 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
         GetApplicationAndProjectNames(result, &resname, NULL);
 #endif  // SELECTBYRESULTNAME
         
-		// loop through the items already in Task Selection Control to find this result
+        // loop through the items already in Task Selection Control to find this result
         count = m_TaskSelectionCtrl->GetCount();
-		for(j = 0; j < count; ++j) {
+        for(j = 0; j < count; ++j) {
             selData = (TaskSelectionData*)m_TaskSelectionCtrl->GetClientData(j);
-			if (!strcmp(result->name, selData->result_name) && 
+            if (!strcmp(result->name, selData->result_name) && 
                 !strcmp(result->project_url, selData->project_url)
             ) {
                 selData->result = result;
-				found = true;
-				is_alive.at(j) = true;
-				break; // skip out of this loop
-			}
-		}
+                found = true;
+                is_alive.at(j) = true;
+                break; // skip out of this loop
+            }
+        }
         
         // if it isn't currently in the list then we have a new one!  lets add it
         if (!found) {
             int alphaOrder;
-		    for(j = 0; j < count; ++j) {
+            for(j = 0; j < count; ++j) {
                 alphaOrder = (m_TaskSelectionCtrl->GetString(j)).CmpNoCase(resname);
 #if SORTTASKLIST
                 if (alphaOrder > 0) {
@@ -1032,8 +1043,8 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
     }       // End for (i) loop
 
     // Check items in descending order so deletion won't change indexes of items yet to be checked
-	for(j = count-1; j >=0; --j) {
-		if(!is_alive.at(j)) {
+    for(j = count-1; j >=0; --j) {
+        if(!is_alive.at(j)) {
             wxLogTrace(wxT("Function Status"), wxT("CSimpleTaskPanel::UpdateTaskSelectionList - Task '%d' no longer alive"), j);
             selData = (TaskSelectionData*)m_TaskSelectionCtrl->GetClientData(j);
             wxLogTrace(wxT("Function Status"), wxT("CSimpleTaskPanel::UpdateTaskSelectionList - selData '%p' "), selData);
@@ -1044,7 +1055,7 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
             wxLogTrace(wxT("Function Status"), wxT("CSimpleTaskPanel::UpdateTaskSelectionList - Deleting control data"));
             // Indicate to Delete() we have cleaned up the Selection Data
             m_TaskSelectionCtrl->SetClientData(j, NULL);
-			m_TaskSelectionCtrl->Delete(j);
+            m_TaskSelectionCtrl->Delete(j);
             if (j == m_CurrentTaskSelection) {
                 int newCount = m_TaskSelectionCtrl->GetCount();
                 if (m_CurrentTaskSelection < newCount) {
@@ -1064,8 +1075,8 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
                 --m_CurrentTaskSelection;
                 m_TaskSelectionCtrl->SetSelection(m_CurrentTaskSelection);
             }
-		}
-	}
+        }
+    }
 
     if ((m_CurrentTaskSelection >= 0) && !m_bStableTaskInfoChanged) {
         selData = (TaskSelectionData*)m_TaskSelectionCtrl->GetClientData(m_CurrentTaskSelection);
@@ -1109,77 +1120,80 @@ void CSimpleTaskPanel::UpdateTaskSelectionList(bool reskin) {
     if (needRefresh) {
         m_TaskSelectionCtrl->Refresh();
     }
+
+    bAlreadyRunning = false;
+
     wxLogTrace(wxT("Function Start/End"), wxT("CSimpleTaskPanel::UpdateTaskSelectionList - Function End"));
 }
 
 
 bool CSimpleTaskPanel::isRunning(RESULT* result) {
-	bool outcome = false;
+    bool outcome = false;
 
-	// It must be scheduled to be running
-	if ( result->scheduler_state == CPU_SCHED_SCHEDULED ) {
-		// If either the project or task have been suspended, then it cannot be running
-		if ( !result->suspended_via_gui && !result->project_suspended_via_gui ) {
-			CC_STATUS status;
+    // It must be scheduled to be running
+    if ( result->scheduler_state == CPU_SCHED_SCHEDULED ) {
+        // If either the project or task have been suspended, then it cannot be running
+        if ( !result->suspended_via_gui && !result->project_suspended_via_gui ) {
+            CC_STATUS status;
             CMainDocument*      pDoc = wxGetApp().GetDocument();
             wxASSERT(pDoc);
             
-			pDoc->GetCoreClientStatus(status);
-			// Make sure that the core client isn't global suspended for some reason
-			if ( status.task_suspend_reason == 0 || status.task_suspend_reason == SUSPEND_REASON_CPU_THROTTLE ) {
-				outcome = true;
-			}
-		}
-	}
+            pDoc->GetCoreClientStatus(status);
+            // Make sure that the core client isn't global suspended for some reason
+            if ( status.task_suspend_reason == 0 || status.task_suspend_reason == SUSPEND_REASON_CPU_THROTTLE ) {
+                outcome = true;
+            }
+        }
+    }
 
-	return outcome;
+    return outcome;
 }
 
 
 bool CSimpleTaskPanel::DownloadingResults() {
-	bool return_value = false;
+    bool return_value = false;
     CMainDocument*      pDoc = wxGetApp().GetDocument();
     wxASSERT(pDoc);
             
-	if ( pDoc->results.results.size() > 0 ) {
-		RESULT* result;
-		for(unsigned int i=0; !return_value && i < pDoc->results.results.size(); i++ ) {
-			result = pDoc->result(i);
-			if ( result != NULL && result->state == RESULT_FILES_DOWNLOADING ) {
-				return_value = true;
-			}
-		}
-	}
-	return return_value;
+    if ( pDoc->results.results.size() > 0 ) {
+        RESULT* result;
+        for(unsigned int i=0; !return_value && i < pDoc->results.results.size(); i++ ) {
+            result = pDoc->result(i);
+            if ( result != NULL && result->state == RESULT_FILES_DOWNLOADING ) {
+                return_value = true;
+            }
+        }
+    }
+    return return_value;
 }
 
 bool CSimpleTaskPanel::Suspended() {
-	CC_STATUS status;
+    CC_STATUS status;
     CMainDocument*      pDoc = wxGetApp().GetDocument();
     wxASSERT(pDoc);
     
     bool result = false;
-	pDoc->GetCoreClientStatus(status);
-	if ( pDoc->IsConnected() && status.task_suspend_reason > 0 && status.task_suspend_reason != SUSPEND_REASON_DISK_SIZE &&  status.task_suspend_reason != SUSPEND_REASON_CPU_THROTTLE ) {
-		result = true;
-	}
-	return result;
+    pDoc->GetCoreClientStatus(status);
+    if ( pDoc->IsConnected() && status.task_suspend_reason > 0 && status.task_suspend_reason != SUSPEND_REASON_DISK_SIZE &&  status.task_suspend_reason != SUSPEND_REASON_CPU_THROTTLE ) {
+        result = true;
+    }
+    return result;
 }
 
 // Check to see if a project update is scheduled or in progress
 bool CSimpleTaskPanel::ProjectUpdateScheduled() {
-	PROJECT* project;
+    PROJECT* project;
     CMainDocument*      pDoc = wxGetApp().GetDocument();
     wxASSERT(pDoc);
     
     int prjCount = pDoc->GetSimpleProjectCount();
-	for(int i=0; i<prjCount; i++) {
-		project = pDoc->state.projects[i];
-		if ( project->sched_rpc_pending || project->master_url_fetch_pending || project->scheduler_rpc_in_progress ) {
-			return true;
-		}
-	}
-	return false;
+    for(int i=0; i<prjCount; i++) {
+        project = pDoc->state.projects[i];
+        if ( project->sched_rpc_pending || project->master_url_fetch_pending || project->scheduler_rpc_in_progress ) {
+            return true;
+        }
+    }
+    return false;
 }
 
 void CSimpleTaskPanel::DisplayIdleState() {
@@ -1187,48 +1201,48 @@ void CSimpleTaskPanel::DisplayIdleState() {
     wxASSERT(pDoc);
             
     if ( pDoc->IsReconnecting() ) {
-		error_time = 0;
+        error_time = 0;
         UpdateStaticText(&m_StatusValueText, _("Retrieving current status."));
-	} else if ( pDoc->IsConnected() && pDoc->state.projects.size() == 0) {
-		error_time = 0;
-		UpdateStaticText(&m_StatusValueText, m_sNoProjectsString);
-	} else if ( DownloadingResults() ) {
-		error_time = 0;
-		UpdateStaticText(&m_StatusValueText, _("Downloading work from the server."));
-	} else if ( Suspended() ) {
-		CC_STATUS status;
-		pDoc->GetCoreClientStatus(status);
-		if ( status.task_suspend_reason & SUSPEND_REASON_BATTERIES ) {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Running On Batteries."));
-		} else if ( status.task_suspend_reason & SUSPEND_REASON_USER_ACTIVE ) {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  User Active."));
-		} else if ( status.task_suspend_reason & SUSPEND_REASON_USER_REQ ) {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  User paused processing."));
-		} else if ( status.task_suspend_reason & SUSPEND_REASON_TIME_OF_DAY ) {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Time of Day."));
-		} else if ( status.task_suspend_reason & SUSPEND_REASON_BENCHMARKS ) {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Benchmarks Running."));
-		} else {
-			UpdateStaticText(&m_StatusValueText, _("Processing Suspended."));
-		}
-	} else if ( ProjectUpdateScheduled() ) {
-		error_time = 0;
-		UpdateStaticText(&m_StatusValueText, _("Waiting to contact project servers."));
-	} else {
-		if ( error_time == 0 ) {
-			error_time = time(NULL) + 10;
-			UpdateStaticText(&m_StatusValueText, _("Retrieving current status"));
-		} else if ( error_time < time(NULL) ) {
+    } else if ( pDoc->IsConnected() && pDoc->state.projects.size() == 0) {
+        error_time = 0;
+        UpdateStaticText(&m_StatusValueText, m_sNoProjectsString);
+    } else if ( DownloadingResults() ) {
+        error_time = 0;
+        UpdateStaticText(&m_StatusValueText, _("Downloading work from the server."));
+    } else if ( Suspended() ) {
+        CC_STATUS status;
+        pDoc->GetCoreClientStatus(status);
+        if ( status.task_suspend_reason & SUSPEND_REASON_BATTERIES ) {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Running On Batteries."));
+        } else if ( status.task_suspend_reason & SUSPEND_REASON_USER_ACTIVE ) {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  User Active."));
+        } else if ( status.task_suspend_reason & SUSPEND_REASON_USER_REQ ) {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  User paused processing."));
+        } else if ( status.task_suspend_reason & SUSPEND_REASON_TIME_OF_DAY ) {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Time of Day."));
+        } else if ( status.task_suspend_reason & SUSPEND_REASON_BENCHMARKS ) {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended:  Benchmarks Running."));
+        } else {
+            UpdateStaticText(&m_StatusValueText, _("Processing Suspended."));
+        }
+    } else if ( ProjectUpdateScheduled() ) {
+        error_time = 0;
+        UpdateStaticText(&m_StatusValueText, _("Waiting to contact project servers."));
+    } else {
+        if ( error_time == 0 ) {
+            error_time = time(NULL) + 10;
+            UpdateStaticText(&m_StatusValueText, _("Retrieving current status"));
+        } else if ( error_time < time(NULL) ) {
             // TODO: should we display "ERROR" like old Simple GUI?
-			if ( pDoc->IsConnected() ) {
-				UpdateStaticText(&m_StatusValueText, _("No work available to process"));
-			} else {
-				UpdateStaticText(&m_StatusValueText, _("Unable to connect to the core client"));
-			}
-		} else {
-			UpdateStaticText(&m_StatusValueText, _("Retrieving current status"));
-		}
-	}
+            if ( pDoc->IsConnected() ) {
+                UpdateStaticText(&m_StatusValueText, _("No work available to process"));
+            } else {
+                UpdateStaticText(&m_StatusValueText, _("Unable to connect to the core client"));
+            }
+        } else {
+            UpdateStaticText(&m_StatusValueText, _("Retrieving current status"));
+        }
+    }
 }
 
 
