@@ -126,11 +126,11 @@ function search($params) {
     if (strlen($params->keywords)) {
         $kw = BoincDb::escape_string($params->keywords);
         $name_lc = strtolower($kw);
-        $name_lc = escape_pattern($name_lc);
 
         $list2 = get_teams("name='$name_lc'", $params->active);
         merge_lists($list2, $list, 20);
 
+        $name_lc = escape_pattern($name_lc);
         $list2 = get_teams("name like '".$name_lc."%'", $params->active);
         merge_lists($list2, $list, 5);
 
@@ -142,7 +142,8 @@ function search($params) {
         $tried = true;
     }
     if (strlen($params->country) && $params->country!='None') {
-        $list2 = get_teams("country = '$params->country'", $params->active);
+        $country = BoincDb::escape_string($params->country);
+        $list2 = get_teams("country = '$country'", $params->active);
         //echo "<br>country matches: ",sizeof($list2);
         merge_lists($list2, $list, 1);
         $tried = true;
@@ -167,7 +168,7 @@ if ($submit || $xml) {
     $params = null;
     $params->keywords = get_str('keywords', true);
     $params->country = get_str("country", true);
-    $params->type = get_str("type", true);
+    $params->type = get_int("type", true);
     $params->active = get_str('active', true);
     $list = search($params);
     if ($xml) {
