@@ -25,10 +25,6 @@
 #include <string>
 #endif
 
-#ifdef _MSC_VER
-#define strdup _strdup
-#endif
-
 #include "error_numbers.h"
 #include "filesys.h"
 #include "miofile.h"
@@ -39,6 +35,10 @@
 
 #include "app_ipc.h"
 
+#ifdef _MSC_VER
+#define strdup _strdup
+#endif
+
 using std::string;
 
 const char* xml_graphics_modes[NGRAPHICS_MSGS] = {
@@ -100,6 +100,7 @@ void APP_INIT_DATA::copy(const APP_INIT_DATA& a) {
     teamid                        = a.teamid;
     hostid                        = a.hostid;
     slot                          = a.slot;
+    client_pid                    = a.client_pid;
     user_total_credit             = a.user_total_credit;
     user_expavg_credit            = a.user_expavg_credit;
     host_total_credit             = a.host_total_credit;
@@ -127,6 +128,7 @@ void APP_INIT_DATA::copy(const APP_INIT_DATA& a) {
     } else {
         project_preferences = NULL;
     }
+    vbox_window                   = a.vbox_window;
 }
 
 int write_init_data_file(FILE* f, APP_INIT_DATA& ai) {
@@ -192,6 +194,7 @@ int write_init_data_file(FILE* f, APP_INIT_DATA& ai) {
 #endif
     fprintf(f,
         "<slot>%d</slot>\n"
+        "<client_pid>%d</client_pid>\n"
         "<wu_cpu_time>%f</wu_cpu_time>\n"
         "<starting_elapsed_time>%f</starting_elapsed_time>\n"
         "<using_sandbox>%d</using_sandbox>\n"
@@ -211,8 +214,10 @@ int write_init_data_file(FILE* f, APP_INIT_DATA& ai) {
         "<rsc_fpops_bound>%f</rsc_fpops_bound>\n"
         "<rsc_memory_bound>%f</rsc_memory_bound>\n"
         "<rsc_disk_bound>%f</rsc_disk_bound>\n"
-        "<computation_deadline>%f</computation_deadline>\n",
+        "<computation_deadline>%f</computation_deadline>\n"
+        "<vbox_window>%d</vbox_window>\n",
         ai.slot,
+        ai.client_pid,
         ai.wu_cpu_time,
         ai.starting_elapsed_time,
         ai.using_sandbox?1:0,
@@ -232,7 +237,8 @@ int write_init_data_file(FILE* f, APP_INIT_DATA& ai) {
         ai.rsc_fpops_bound,
         ai.rsc_memory_bound,
         ai.rsc_disk_bound,
-        ai.computation_deadline
+        ai.computation_deadline,
+        ai.vbox_window
     );
     MIOFILE mf;
     mf.init_file(f);
@@ -263,6 +269,7 @@ void APP_INIT_DATA::clear() {
     strcpy(result_name, "");
     strcpy(authenticator, "");
     slot = 0;
+    client_pid = 0;
     user_total_credit = 0;
     user_expavg_credit = 0;
     host_total_credit = 0;
@@ -282,12 +289,13 @@ void APP_INIT_DATA::clear() {
     fraction_done_end = 0;
     checkpoint_period = 0;
     strcpy(gpu_type, "");
-    gpu_device_num = 0;
+    gpu_device_num = -1;
     // -1 means an older version without gpu_opencl_dev_index field
     gpu_opencl_dev_index = -1;
     ncpus = 0;
     memset(&shmem_seg_name, 0, sizeof(shmem_seg_name));
     wu_cpu_time = 0;
+    vbox_window = false;
 }
 
 int parse_init_data_file(FILE* f, APP_INIT_DATA& ai) {
@@ -367,6 +375,7 @@ int parse_init_data_file(FILE* f, APP_INIT_DATA& ai) {
         if (xp.parse_int("shm_key", ai.shmem_seg_name)) continue;
 #endif
         if (xp.parse_int("slot", ai.slot)) continue;
+        if (xp.parse_int("client_pid", ai.client_pid)) continue;
         if (xp.parse_double("user_total_credit", ai.user_total_credit)) continue;
         if (xp.parse_double("user_expavg_credit", ai.user_expavg_credit)) continue;
         if (xp.parse_double("host_total_credit", ai.host_total_credit)) continue;
@@ -387,6 +396,7 @@ int parse_init_data_file(FILE* f, APP_INIT_DATA& ai) {
         if (xp.parse_double("ncpus", ai.ncpus)) continue;
         if (xp.parse_double("fraction_done_start", ai.fraction_done_start)) continue;
         if (xp.parse_double("fraction_done_end", ai.fraction_done_end)) continue;
+        if (xp.parse_bool("vbox_window", ai.vbox_window)) continue;
         xp.skip_unexpected(false, "parse_init_data_file");
     }
     fprintf(stderr, "parse_init_data_file: no end tag\n");
