@@ -15,6 +15,8 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+#include "cc_config.h"
+
 #ifdef _WIN32
 #include "boinc_win.h"
 #else
@@ -72,7 +74,6 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
         if (xp.parse_bool("cpu_sched_status", cpu_sched_status)) continue;
         if (xp.parse_bool("dcf_debug", dcf_debug)) continue;
         if (xp.parse_bool("disk_usage_debug", disk_usage_debug)) continue;
-        if (xp.parse_bool("priority_debug", priority_debug)) continue;
         if (xp.parse_bool("file_xfer_debug", file_xfer_debug)) continue;
         if (xp.parse_bool("gui_rpc_debug", gui_rpc_debug)) continue;
         if (xp.parse_bool("heartbeat_debug", heartbeat_debug)) continue;
@@ -80,7 +81,9 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
         if (xp.parse_bool("http_xfer_debug", http_xfer_debug)) continue;
         if (xp.parse_bool("mem_usage_debug", mem_usage_debug)) continue;
         if (xp.parse_bool("network_status_debug", network_status_debug)) continue;
+        if (xp.parse_bool("notice_debug", notice_debug)) continue;
         if (xp.parse_bool("poll_debug", poll_debug)) continue;
+        if (xp.parse_bool("priority_debug", priority_debug)) continue;
         if (xp.parse_bool("proxy_debug", proxy_debug)) continue;
         if (xp.parse_bool("rr_simulation", rr_simulation)) continue;
         if (xp.parse_bool("rrsim_detail", rrsim_detail)) continue;
@@ -95,7 +98,6 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
         if (xp.parse_bool("trickle_debug", trickle_debug)) continue;
         if (xp.parse_bool("unparsed_xml", unparsed_xml)) continue;
         if (xp.parse_bool("work_fetch_debug", work_fetch_debug)) continue;
-        if (xp.parse_bool("notice_debug", notice_debug)) continue;
         xp.skip_unexpected(true, "LOG_FLAGS::parse");
     }
     return ERR_XML_PARSE;
@@ -118,7 +120,6 @@ int LOG_FLAGS::write(MIOFILE& out) {
         "        <cpu_sched_status>%d</cpu_sched_status>\n"
         "        <dcf_debug>%d</dcf_debug>\n"
         "        <disk_usage_debug>%d</disk_usage_debug>\n"
-        "        <priority_debug>%d</priority_debug>\n"
         "        <file_xfer_debug>%d</file_xfer_debug>\n"
         "        <gui_rpc_debug>%d</gui_rpc_debug>\n"
         "        <heartbeat_debug>%d</heartbeat_debug>\n"
@@ -126,7 +127,9 @@ int LOG_FLAGS::write(MIOFILE& out) {
         "        <http_xfer_debug>%d</http_xfer_debug>\n"
         "        <mem_usage_debug>%d</mem_usage_debug>\n"
         "        <network_status_debug>%d</network_status_debug>\n"
+        "        <notice_debug>%d</notice_debug>\n"
         "        <poll_debug>%d</poll_debug>\n"
+        "        <priority_debug>%d</priority_debug>\n"
         "        <proxy_debug>%d</proxy_debug>\n"
         "        <rr_simulation>%d</rr_simulation>\n"
         "        <rrsim_detail>%d</rrsim_detail>\n"
@@ -141,7 +144,6 @@ int LOG_FLAGS::write(MIOFILE& out) {
         "        <trickle_debug>%d</trickle_debug>\n"
         "        <unparsed_xml>%d</unparsed_xml>\n"
         "        <work_fetch_debug>%d</work_fetch_debug>\n"
-        "        <notice_debug>%d</notice_debug>\n"
         "    </log_flags>\n",
         file_xfer ? 1 : 0,
         sched_ops ? 1 : 0,
@@ -157,7 +159,6 @@ int LOG_FLAGS::write(MIOFILE& out) {
         cpu_sched_status ? 1 : 0,
         dcf_debug ? 1 : 0,
         disk_usage_debug ? 1 : 0,
-        priority_debug ? 1 : 0,
         file_xfer_debug ? 1 : 0,
         gui_rpc_debug ? 1 : 0,
         heartbeat_debug ? 1 : 0,
@@ -165,7 +166,9 @@ int LOG_FLAGS::write(MIOFILE& out) {
         http_xfer_debug ? 1 : 0,
         mem_usage_debug ? 1 : 0,
         network_status_debug ? 1 : 0,
+        notice_debug ? 1 : 0,
         poll_debug ? 1 : 0,
+        priority_debug ? 1 : 0,
         proxy_debug ? 1 : 0,
         rr_simulation ? 1 : 0,
         rrsim_detail ? 1 : 0,
@@ -179,8 +182,7 @@ int LOG_FLAGS::write(MIOFILE& out) {
         time_debug ? 1 : 0,
         trickle_debug ? 1 : 0,
         unparsed_xml ? 1 : 0,
-        work_fetch_debug ? 1 : 0,
-        notice_debug ? 1 : 0
+        work_fetch_debug ? 1 : 0
     );
     
     return 0;
@@ -210,12 +212,14 @@ void CONFIG::defaults() {
     exit_before_start = false;
     exit_when_idle = false;
     fetch_minimal_work = false;
+    fetch_on_update = false;
     force_auth = "default";
     http_1_0 = false;
     http_transfer_timeout = 300;
     http_transfer_timeout_bps = 10;
-    ignore_nvidia_dev.clear();
-    ignore_ati_dev.clear();
+    for (int i=1; i<NPROC_TYPES; i++) {
+        ignore_gpu_instance[i].clear();
+    }
     max_file_xfers = 8;
     max_file_xfers_per_project = 2;
     max_stderr_file_size = 0;
@@ -242,6 +246,7 @@ void CONFIG::defaults() {
     use_all_gpus = false;
     use_certs = false;
     use_certs_only = false;
+    vbox_window = false;
 }
 
 int EXCLUDE_GPU::parse(XML_PARSER& xp) {
@@ -282,8 +287,9 @@ int CONFIG::parse_options(XML_PARSER& xp) {
     alt_platforms.clear();
     exclusive_apps.clear();
     exclusive_gpu_apps.clear();
-    ignore_nvidia_dev.clear();
-    ignore_ati_dev.clear();
+    for (int i=1; i<NPROC_TYPES; i++) {
+        ignore_gpu_instance[i].clear();
+    }
     exclude_gpus.clear();
 
     while (!xp.get_tag()) {
@@ -351,6 +357,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
             continue;
         }
         if (xp.parse_bool("fetch_minimal_work", fetch_minimal_work)) continue;
+        if (xp.parse_bool("fetch_on_update", fetch_on_update)) continue;
         if (xp.parse_string("force_auth", force_auth)) {
             downcase_string(force_auth);
             continue;
@@ -359,11 +366,15 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_int("http_transfer_timeout", http_transfer_timeout)) continue;
         if (xp.parse_int("http_transfer_timeout_bps", http_transfer_timeout_bps)) continue;
         if (xp.parse_int("ignore_cuda_dev", n) || xp.parse_int("ignore_nvidia_dev", n)) {
-            ignore_nvidia_dev.push_back(n);
+            ignore_gpu_instance[PROC_TYPE_NVIDIA_GPU].push_back(n);
             continue;
         }
         if (xp.parse_int("ignore_ati_dev", n)) {
-            ignore_ati_dev.push_back(n);
+            ignore_gpu_instance[PROC_TYPE_AMD_GPU].push_back(n);
+            continue;
+        }
+        if (xp.parse_int("ignore_intel_gpu_dev", n)) {
+            ignore_gpu_instance[PROC_TYPE_INTEL_GPU].push_back(n);
             continue;
         }
         if (xp.parse_int("max_file_xfers", max_file_xfers)) continue;
@@ -404,6 +415,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_bool("use_all_gpus", use_all_gpus)) continue;
         if (xp.parse_bool("use_certs", use_certs)) continue;
         if (xp.parse_bool("use_certs_only", use_certs_only)) continue;
+        if (xp.parse_bool("vbox_window", vbox_window)) continue;
 
         xp.skip_unexpected(true, "CONFIG::parse_options");
     }
@@ -520,6 +532,7 @@ int CONFIG::write(MIOFILE& out, LOG_FLAGS& log_flags) {
         "        <exit_before_start>%d</exit_before_start>\n"
         "        <exit_when_idle>%d</exit_when_idle>\n"
         "        <fetch_minimal_work>%d</fetch_minimal_work>\n"
+        "        <fetch_on_update>%d</fetch_on_update>\n"
         "        <force_auth>%s</force_auth>\n"
         "        <http_1_0>%d</http_1_0>\n"
         "        <http_transfer_timeout>%d</http_transfer_timeout>\n"
@@ -528,23 +541,31 @@ int CONFIG::write(MIOFILE& out, LOG_FLAGS& log_flags) {
         exit_before_start,
         exit_when_idle,
         fetch_minimal_work,
+        fetch_on_update,
         force_auth.c_str(),
         http_1_0,
         http_transfer_timeout,
         http_transfer_timeout_bps
     );
         
-    for (i=0; i<ignore_nvidia_dev.size(); ++i) {
+    for (i=0; i<ignore_gpu_instance[PROC_TYPE_NVIDIA_GPU].size(); ++i) {
         out.printf(
             "        <ignore_nvidia_dev>%d</ignore_nvidia_dev>\n",
-            ignore_nvidia_dev[i]
+            ignore_gpu_instance[PROC_TYPE_NVIDIA_GPU][i]
         );
     }
 
-    for (i=0; i<ignore_ati_dev.size(); ++i) {
+    for (i=0; i<ignore_gpu_instance[PROC_TYPE_AMD_GPU].size(); ++i) {
         out.printf(
             "        <ignore_ati_dev>%d</ignore_ati_dev>\n",
-            ignore_ati_dev[i]
+            ignore_gpu_instance[PROC_TYPE_AMD_GPU][i]
+        );
+    }
+
+    for (i=0; i<ignore_gpu_instance[PROC_TYPE_INTEL_GPU].size(); ++i) {
+        out.printf(
+            "        <ignore_intel_gpu_dev>%d</ignore_intel_gpu_dev>\n",
+            ignore_gpu_instance[PROC_TYPE_INTEL_GPU][i]
         );
     }
         
@@ -590,7 +611,8 @@ int CONFIG::write(MIOFILE& out, LOG_FLAGS& log_flags) {
         "        <unsigned_apps_ok>%d</unsigned_apps_ok>\n"
         "        <use_all_gpus>%d</use_all_gpus>\n"
         "        <use_certs>%d</use_certs>\n"
-        "        <use_certs_only>%d</use_certs_only>\n",
+        "        <use_certs_only>%d</use_certs_only>\n"
+        "        <vbox_window>%d</vbox_window>\n",
         rec_half_life/86400,
         report_results_immediately,
         run_apps_manually,
@@ -603,7 +625,8 @@ int CONFIG::write(MIOFILE& out, LOG_FLAGS& log_flags) {
         unsigned_apps_ok,
         use_all_gpus,
         use_certs,
-        use_certs_only
+        use_certs_only,
+        vbox_window
     );
 
     out.printf("    </options>\n</cc_config>\n");
