@@ -30,7 +30,7 @@
 #include "proxy_info.h"
 #include "coproc.h"
 
-class XML_PARSER;
+struct XML_PARSER;
 
 #define MAX_FILE_XFERS_PER_PROJECT      2
 #define MAX_FILE_XFERS                  8
@@ -46,7 +46,7 @@ struct LOG_FLAGS {
     bool sched_ops;
         // interactions with schedulers
     bool task;
-        // task start and finish
+        // task start and finish, and suspend/resume
 
     // off by default; intended for developers and testers
     //
@@ -151,12 +151,12 @@ struct CONFIG {
     bool exit_before_start;
     bool exit_when_idle;
     bool fetch_minimal_work;
+    bool fetch_on_update;
     std::string force_auth;
     bool http_1_0;
     int http_transfer_timeout_bps;
     int http_transfer_timeout;
-    std::vector<int> ignore_ati_dev;
-    std::vector<int> ignore_nvidia_dev;
+    std::vector<int> ignore_gpu_instance[NPROC_TYPES];
     int max_file_xfers;
     int max_file_xfers_per_project;
     int max_stderr_file_size;
@@ -184,6 +184,7 @@ struct CONFIG {
     bool use_certs;
     bool use_certs_only;
         // overrides use_certs
+    bool vbox_window;
 
     CONFIG();
     void defaults();
