@@ -44,6 +44,10 @@
 #include <openssl/engine.h>
 #include <openssl/err.h>
 
+#ifdef _USING_FCGI_
+#include "boinc_fcgi.h"
+#endif
+
 #include "md5_file.h"
 #include "cert_sig.h"
 #include "filesys.h"
@@ -51,10 +55,6 @@
 
 #include "crypt.h"
 
-#ifdef _USING_FCGI_
-#include "boinc_fcgi.h"
-#endif
-
 // NOTE: the fast CGI I/O library doesn't have fscanf(),
 // so some of the following have been modified to use
 // fgets() and sscanf() instead
@@ -345,7 +345,9 @@ int check_file_signature(
     clear_signature.len = MD5_LEN;
     retval = decrypt_public(key, signature, clear_signature);
     if (retval) {
-        fprintf(stderr, "check_file_signature: decrypt_public error %d\n", retval);
+        fprintf(stderr,
+            "check_file_signature: decrypt_public error %d\n", retval
+        );
         return retval;
     }
     answer = !strncmp(md5_buf, clear_buf, n);
@@ -602,9 +604,9 @@ char *check_validity(
 
     DIRREF dir = dir_open(certPath);
 
-    char file[256];
-    while (dir_scan(file, dir, sizeof(file))) {
-        char fpath[512];
+    char file[MAXPATHLEN];
+    while (!dir_scan(file, dir, sizeof(file))) {
+        char fpath[MAXPATHLEN];
 	    snprintf(fpath, sizeof(fpath), "%s/%s", certPath, file);
         // TODO : replace '128'  
 	    if (check_validity_of_cert(fpath, md5_md, signature, 128, caPath)) {
@@ -624,7 +626,7 @@ int cert_verify_file(
     int rbytes;
     unsigned char md5_md[MD5_DIGEST_LENGTH],  rbuf[2048];
     char buf[256];
-    char fbuf[512];
+    char fbuf[MAXPATHLEN];
     int verified = false;
     int file_counter = 0;
     DATA_BLOCK sig_db;
@@ -657,7 +659,7 @@ int cert_verify_file(
         sscan_hex_data(signatures->signatures.at(i).signature, sig_db);
         file_counter = 0;
         while (1) {
-            snprintf(fbuf, 512, "%s/%s.%d", trustLocation, signatures->signatures.at(i).hash,
+            snprintf(fbuf, MAXPATHLEN, "%s/%s.%d", trustLocation, signatures->signatures.at(i).hash,
                 file_counter);
 #ifndef _USING_FCGI_
             FILE *f = fopen(fbuf, "r");
