@@ -18,6 +18,7 @@
 // This file is the underlying mechanism of GUI RPC client
 // (not the actual RPCs)
 
+
 #if defined(_WIN32) && !defined(__STDWX_H__) && !defined(_BOINC_WIN_) && !defined(_AFX_STDAFX_H_) 
 #include "boinc_win.h"
 #endif
@@ -50,6 +51,7 @@
 #include "md5_file.h"
 #include "network.h"
 #include "common_defs.h"
+
 #include "gui_rpc_client.h"
 
 using std::string;
@@ -127,12 +129,19 @@ int RPC_CLIENT::init(const char* host, int port) {
 
     // set up receive timeout; avoid hang if client doesn't respond
     //
+#ifdef _WIN32
+    DWORD dwTime = 30000;
+    if (setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, (char *)&dwTime,  sizeof dwTime)) {
+        // not fatal
+    } 
+#else
     struct timeval tv;
     tv.tv_sec = 30;
     tv.tv_usec = 0;
     if (setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, (char *)&tv,  sizeof tv)) {
         // not fatal
     } 
+#endif
     retval = connect(sock, (const sockaddr*)(&addr), addr_len(addr));
     if (retval) {
 #ifdef _WIN32
