@@ -25,6 +25,7 @@
 #include <string>
 #include <vector>
 #include <sys/socket.h>
+#include <sys/param.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
@@ -32,13 +33,14 @@
 #include <locale.h>
 #endif
 
-#include "miofile.h"
-#include "prefs.h"
-#include "hostinfo.h"
-#include "common_defs.h"
-#include "notice.h"
-#include "network.h"
 #include "cc_config.h"
+#include "common_defs.h"
+#include "filesys.h"
+#include "hostinfo.h"
+#include "miofile.h"
+#include "network.h"
+#include "notice.h"
+#include "prefs.h"
 
 struct GUI_URL {
     std::string name;
@@ -101,7 +103,7 @@ struct ALL_PROJECTS_LIST {
     ~ALL_PROJECTS_LIST();
 
     void clear();
-    void shuffle();
+    void alpha_sort();
 };
 
 struct RSC_DESC {
@@ -138,6 +140,7 @@ struct PROJECT {
     RSC_DESC rsc_desc_cpu;
     RSC_DESC rsc_desc_nvidia;
     RSC_DESC rsc_desc_ati;
+    RSC_DESC rsc_desc_intel_gpu;
 
     double sched_priority;
 
@@ -198,7 +201,9 @@ struct APP_VERSION {
     char platform[64];
     char plan_class[64];
     double avg_ncpus;
-    double ncudas;
+    int gpu_type;
+        // PROC_TYPE_xx
+    double gpu_usage;
     double natis;
     double gpu_ram;
     double flops;
@@ -274,10 +279,10 @@ struct RESULT {
     bool too_large;
     bool needs_shmem;
     bool edf_scheduled;
-    char graphics_exec_path[512];
+    char graphics_exec_path[MAXPATHLEN];
     char web_graphics_url[256];
     char remote_desktop_addr[256];
-    char slot_path[512];
+    char slot_path[MAXPATHLEN];
         // only present if graphics_exec_path is
     char resources[256];
 
@@ -366,6 +371,9 @@ struct GR_PROXY_INFO {
     void clear();
 };
 
+// Represents the entire client state.
+// Call get_state() infrequently.
+//
 struct CC_STATE {
     std::vector<PROJECT*> projects;
     std::vector<APP*> apps;
@@ -375,11 +383,12 @@ struct CC_STATE {
     std::vector<std::string> platforms;
         // platforms supported by client
     GLOBAL_PREFS global_prefs;  // working prefs, i.e. network + override
-    VERSION_INFO version_info;  // populated only if talking to pre-5.6 CC
+    VERSION_INFO version_info;  // populated only if talking to pre-5.6 client
     bool executing_as_daemon;   // true if client is running as a service / daemon
     HOST_INFO host_info;
-    bool have_nvidia;           // redundant; include for compat (set by <have_cuda/>)
-    bool have_ati;              // redundant; include for compat
+    TIME_STATS time_stats;
+    bool have_nvidia;           // deprecated; include for compat (set by <have_cuda/>)
+    bool have_ati;              // deprecated; include for compat
 
     CC_STATE();
     ~CC_STATE();
@@ -395,6 +404,11 @@ struct CC_STATE {
     void print();
     void clear();
     int parse(XML_PARSER&);
+    inline bool have_gpu() {
+        return !host_info.coprocs.none()
+            || have_nvidia || have_ati      // for old clients
+        ;
+    }
 };
 
 struct PROJECTS {
@@ -453,6 +467,7 @@ struct MESSAGES {
 
 struct NOTICES {
     bool complete;
+    bool received;
         // whether vector contains all notices, or just new ones
     std::vector<NOTICE*> notices;
 
