@@ -31,10 +31,10 @@
 #endif
 #endif
 
-#include "util.h"
-#include "parse.h"
-#include "md5_file.h"
 #include "error_numbers.h"
+#include "md5_file.h"
+#include "parse.h"
+#include "util.h"
 
 #include "hostinfo.h"
 
@@ -103,7 +103,10 @@ int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
         if (xp.parse_int("p_ncpus", p_ncpus)) continue;
         if (xp.parse_str("p_vendor", p_vendor, sizeof(p_vendor))) continue;
         if (xp.parse_str("p_model", p_model, sizeof(p_model))) continue;
-        if (xp.parse_str("p_features", p_features, sizeof(p_features))) continue;
+        if (xp.parse_str("p_features", p_features, sizeof(p_features))) {
+            downcase_string(p_features);
+            continue;
+        }
         if (xp.parse_double("m_nbytes", m_nbytes)) continue;
         if (xp.parse_double("m_cache", m_cache)) continue;
         if (xp.parse_double("m_swap", m_swap)) continue;
@@ -113,7 +116,7 @@ int HOST_INFO::parse(XML_PARSER& xp, bool benchmarks_only) {
         if (xp.parse_str("os_version", os_version, sizeof(os_version))) continue;
         if (xp.parse_str("virtualbox_version", virtualbox_version, sizeof(virtualbox_version))) continue;
         if (xp.match_tag("coprocs")) {
-            _coprocs.parse(xp);
+            this->coprocs.parse(xp);
         }
     }
     return ERR_XML_PARSE;
@@ -195,7 +198,7 @@ int HOST_INFO::write(
         );
     }
     if (include_coprocs) {
-        _coprocs.write_xml(out, false);
+        this->coprocs.write_xml(out, false);
     }
     out.printf(
         "</host_info>\n"
