@@ -61,12 +61,18 @@ public:
     char os_name[256];
     char os_version[256];
 
-    // the following are non-empty if that VM system is installed
+    // the following is non-empty if VBox is installed
     //
     char virtualbox_version[256];
-    // ... add entries for VMWare, others
 
-    COPROCS _coprocs;
+    COPROCS coprocs;
+
+#ifdef ANDROID
+    int battery_charge_pct;
+    int battery_state;
+    double battery_temperature_celsius;
+    void get_battery_status();
+#endif
 
     HOST_INFO();
     int parse(XML_PARSER&, bool benchmarks_only = false);
@@ -80,8 +86,13 @@ public:
     bool users_idle(bool check_all_logins, double idle_time_to_run, double *actual_idle_time=NULL);
 #else
     bool users_idle(bool check_all_logins, double idle_time_to_run);
+#endif
+#ifdef ANDROID
+    bool host_wifi_online();
 #endif
     int get_host_info();
+    int get_host_battery_charge();
+    int get_host_battery_state();
     int get_local_network_info();
     int get_virtualbox_version();
     void clear_host_info();
@@ -98,6 +109,8 @@ extern "C" {
 #include <IOKit/hidsystem/IOHIDParameter.h>
 #include <IOKit/hidsystem/event_status_driver.h>
 
+bool isDualGPUMacBook();
+
 // Apple has removed NxIdleTime() beginning with OS 10.6, so we must try
 // loading it at run time to avoid a link error.  For details, please see
 // the comments in the __APPLE__ version of HOST_INFO::users_idle() in
