@@ -15,16 +15,9 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#if   defined(_WIN32) && !defined(__STDWX_H__)
-#include "boinc_win.h"
-#elif defined(_WIN32) && defined(__STDWX_H__)
-#include "stdwx.h"
-#else
-#include "config.h"
-#endif
-
-#include "parse.h"
 #include "error_numbers.h"
+#include "parse.h"
+
 #include "proxy_info.h"
 
 int PROXY_INFO::parse(XML_PARSER& xp) {
