@@ -15,22 +15,28 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#if   defined(_WIN32) && !defined(__STDWX_H__)
+#ifdef  _WIN32
+#ifndef __STDWX_H__
 #include "boinc_win.h"
-#elif defined(_WIN32) && defined(__STDWX_H__)
+#else
 #include "stdwx.h"
 #endif
-#ifdef _WIN32
 #include "win_util.h"
-#ifdef _MSC_VER
-#define finite _finite
 #endif
+
+#if defined(_MSC_VER) || defined(__MINGW32__)
+#define finite _finite
 #endif
 
 #ifndef M_LN2
 #define M_LN2      0.693147180559945309417
 #endif
 
+#ifdef _USING_FCGI_
+#include "boinc_fcgi.h"
+#define perror FCGI::perror
+#endif
+
 #ifndef _WIN32
 #include "config.h"
 #if HAVE_UNISTD_H
@@ -44,6 +50,7 @@
 #include <errno.h>
 #include <string>
 #include <cstring>
+#include <math.h>
 #if HAVE_IEEEFP_H
 #include <ieeefp.h>
 extern "C" {
@@ -55,17 +62,12 @@ extern "C" {
 #include "error_numbers.h"
 #include "common_defs.h"
 #include "filesys.h"
-#include "util.h"
 #include "base64.h"
 #include "mfile.h"
 #include "miofile.h"
 #include "parse.h"
 
-
-#ifdef _USING_FCGI_
-#include "boinc_fcgi.h"
-#define perror FCGI::perror
-#endif
+#include "util.h"
 
 using std::min;
 using std::string;
@@ -310,7 +312,7 @@ void boinc_crash() {
 #ifdef _WIN32
     DebugBreak();
 #else
-    *(int*)0 = 0;
+    abort();
 #endif
 }
 
@@ -410,7 +412,7 @@ int run_program(
         &process_info
     );
     if (!retval) {
-        windows_error_string(error_msg, sizeof(error_msg));
+        windows_format_error_string(GetLastError(), error_msg, sizeof(error_msg));
         fprintf(stderr, "CreateProcess failed: '%s'\n", error_msg);
         return -1; // CreateProcess returns 1 if successful, false if it failed.
     }
@@ -438,7 +440,11 @@ int run_program(
             if (retval) return retval;
         }
         execv(file, argv);
+#ifdef _USING_FCGI_
+        FCGI::perror("execv");
+#else
         perror("execv");
+#endif
         exit(errno);
     }
 
@@ -516,7 +522,7 @@ static int get_client_mutex(const char*) {
     }
 #else
 static int get_client_mutex(const char* dir) {
-    char path[1024];
+    char path[MAXPATHLEN];
     static FILE_LOCK file_lock;
 
     sprintf(path, "%s/%s", dir, LOCK_FILE_NAME);
