@@ -21,10 +21,13 @@
 #include "stdwx.h"
 #endif
 
+#if defined(_MSC_VER) || defined(__MINGW32__)
+#define snprintf    _snprintf
+#endif
+
 #include "diagnostics.h"
 #include "win_util.h"
 
-
 /**
  * Find out if we are on a Windows 2000 compatible system
  **/
@@ -802,7 +805,7 @@ bool is_remote_desktop() {
     static tWTSFM pWTSFM = NULL;
     LPTSTR pBuf = NULL;
     DWORD dwLength;
-    USHORT usProtocol=-1, usConnectionState=-1;
+    USHORT usProtocol=0, usConnectionState=0;
 
     if (!wtsapi32lib) {
         wtsapi32lib = LoadLibrary(_T("wtsapi32.dll"));
@@ -862,3 +865,56 @@ bool is_remote_desktop() {
     return false;
 }
 
+std::wstring A2W(const std::string& str) {
+  int length_wide = MultiByteToWideChar(CP_ACP, 0, str.data(), -1, NULL, 0);
+  wchar_t *string_wide = static_cast<wchar_t*>(_alloca((length_wide * sizeof(wchar_t)) + sizeof(wchar_t)));
+  MultiByteToWideChar(CP_ACP, 0, str.data(), -1, string_wide, length_wide);
+  std::wstring result(string_wide, length_wide);
+  return result;
+}
+
+std::string W2A(const std::wstring& str) {
+  int length_ansi = WideCharToMultiByte(CP_UTF8, 0, str.data(), -1, NULL, 0, NULL, NULL);
+  char* string_ansi = static_cast<char*>(_alloca(length_ansi + sizeof(char)));
+  WideCharToMultiByte(CP_UTF8, 0, str.data(), -1, string_ansi, length_ansi, NULL, NULL);
+  std::string result(string_ansi, length_ansi);
+  return result;
+}
+
+// get message for given error
+//
+char* windows_format_error_string(
+    unsigned long dwError, char* pszBuf, int iSize
+) {
+    DWORD dwRet;
+    LPWSTR lpszTemp = NULL;
+
+    dwRet = FormatMessageW(
+        FORMAT_MESSAGE_ALLOCATE_BUFFER |
+        FORMAT_MESSAGE_FROM_SYSTEM |
+        FORMAT_MESSAGE_ARGUMENT_ARRAY,
+        NULL,
+        dwError,
+        LANG_NEUTRAL,
+        (LPWSTR)&lpszTemp,
+        0,
+        NULL
+    );
+
+    if (dwRet != 0) {
+        // convert from current character encoding into UTF8
+        std::string encoded_message = W2A(std::wstring(lpszTemp));
+
+        // include the hex error code as well
+        snprintf(pszBuf, iSize, "%s (0x%x)", encoded_message.c_str(), dwError);
+
+        if (lpszTemp) {
+            LocalFree((HLOCAL) lpszTemp);
+        }
+    } else {
+        strcpy(pszBuf, "(unknown error)");
+    }
+
+    return pszBuf;
+}
+
