@@ -1,4 +1,4 @@
-## $Id: setup_project.py 25322 2012-02-24 03:26:30Z davea $
+## $Id$
 
 # module for setting up a new project (either a real project or a test project
 # see tools/makeproject, test/testbase.py).
@@ -344,10 +344,21 @@ def install_boinc_files(dest_dir, install_web_files, install_server_files):
     force_symlink(dir('bin', 'start'), dir('bin', 'stop'))
     force_symlink(dir('bin', 'start'), dir('bin', 'status'))
     map(lambda (s): install(srcdir('py/Boinc',s), dir('py/Boinc',s)),
-        [ '__init__.py', 'add_util.py', 'boinc_db.py', 'boinc_project_path.py',
-          'boincxml.py', 'configxml.py', 'database.py',
-          'db_base.py', 'db_mid.py', 'projectxml.py',
-          'sched_messages.py', 'tools.py', 'util.py' ])
+        [
+            '__init__.py',
+            'add_util.py',
+            'boinc_db.py',
+            'boinc_project_path.py',
+            'boincxml.py',
+            'configxml.py',
+            'database.py',
+            'db_base.py',
+            'db_mid.py',
+            'projectxml.py',
+            'sched_messages.py',
+            'tools.py',
+            'util.py'
+        ])
     print >>open(dir('bin', 'boinc_path_config.py'), 'w'), '''
 # Generated by make_project
 import sys, os
@@ -372,23 +383,55 @@ sys.path.insert(0, os.path.join('%s', 'py'))
     map(lambda (s): install(builddir('sched',s), dir('cgi-bin',s)),
         [ 'cgi', 'file_upload_handler'])
     map(lambda (s): install(builddir('sched',s), dir('bin',s)),
-        [ 'make_work', 'feeder', 'transitioner', 'transitioner_catchup.php',
-          'sample_bitwise_validator', 'sample_trivial_validator',
-          'file_deleter', 'antique_file_deleter',
-          'sample_dummy_assimilator',
-          'sample_assimilator', 'sample_work_generator',
-          'single_job_assimilator', 
-          'assimilator.py', 'pymw_assimilator.py',
-          'update_stats', 'db_dump', 'db_purge', 'show_shmem', 'census',
-          'delete_file', 'get_file', 'put_file' ])
+        [
+            'adjust_user_priority',
+            'antique_file_deleter',
+            'assimilator.py',
+            'census',
+            'db_dump',
+            'db_purge',
+            'delete_file',
+            'feeder',
+            'file_deleter',
+            'get_file',
+            'make_work',
+            'put_file',
+            'pymw_assimilator.py',
+            'sample_assimilator',
+            'sample_bitwise_validator',
+            'sample_dummy_assimilator',
+            'sample_trivial_validator',
+            'sample_work_generator',
+            'show_shmem',
+            'single_job_assimilator', 
+            'transitioner',
+            'transitioner_catchup.php',
+            'update_stats'
+        ])
     map(lambda (s): install(builddir('vda',s), dir('bin',s)),
         [ 'vda', 'vdad' ])
     map(lambda (s): install(srcdir('tools',s), dir('bin',s)),
-        [ 'appmgr', 'create_work', 'xadd', 'dbcheck_files_exist', 'run_in_ops',
-          'update_versions', 'parse_config', 'grep_logs', 'db_query',
-          'watch_tcp', 'sign_executable', 'dir_hier_move',
-          'manage_privileges', 'cancel_jobs',
-          'dir_hier_path', 'boinc_submit', 'demo_submit', 'demo_query' ])
+        [
+            'appmgr',
+            'boinc_submit',
+            'cancel_jobs',
+            'create_work',
+            'dbcheck_files_exist',
+            'db_query',
+            'demo_query',
+            'demo_submit',
+            'dir_hier_move',
+            'dir_hier_path',
+            'grep_logs',
+            'manage_privileges',
+            'parse_config',
+            'run_in_ops',
+            'sign_executable',
+            'stage_file',
+            'update_versions',
+            'watch_tcp',
+            'xadd'
+        ])
     map(lambda (s): install(srcdir('lib',s), dir('bin',s)),
         [ 'crypt_prog' ])
     map(lambda (s): install(srcdir('sched',s), dir('',s)),
@@ -512,6 +555,7 @@ class Project:
         install(srcdir('html/project.sample/cache_parameters.inc'),
             self.dir('html/project/cache_parameters.inc'))
         install(srcdir('tools/project.xml'), self.dir('project.xml'))
+        install(srcdir('tools/gui_urls.xml'), self.dir('gui_urls.xml'))
         if not self.production:
             install(srcdir('test/uc_result'), self.dir('templates/uc_result'))
             install(srcdir('test/uc_wu_nodelete'), self.dir('templates/uc_wu'))
