@@ -58,15 +58,19 @@ using std::string;
 #include "vbox.h"
 
 VBOX_VM::VBOX_VM() {
+    virtualbox_version.clear();
     pFloppy = NULL;
     vm_master_name.clear();
     vm_name.clear();
     vm_cpu_count.clear();
+    vm_disk_controller_type.clear();
+    vm_disk_controller_model.clear();
     os_name.clear();
     memory_size_mb.clear();
     image_filename.clear();
     floppy_image_filename.clear();
     job_duration = 0.0;
+    fraction_done_filename.clear();
     suspended = false;
     network_suspended = false;
     online = false;
@@ -79,11 +83,16 @@ VBOX_VM::VBOX_VM() {
     enable_network = false;
     pf_guest_port = 0;
     pf_host_port = 0;
+    headless = true;
 #ifndef _WIN32
     vm_pid = 0;
 #else
     vm_pid_handle = 0;
 #endif
+
+    // Initialize default values
+    vm_disk_controller_type = "ide";
+    vm_disk_controller_model = "PIIX4";
 }
 
 VBOX_VM::~VBOX_VM() {
@@ -100,11 +109,15 @@ VBOX_VM::~VBOX_VM() {
 }
 
 int VBOX_VM::initialize() {
+    int rc = 0;
     string virtualbox_install_directory;
     string old_path;
     string new_path;
     string virtualbox_user_home;
+    string command;
+    string output;
     APP_INIT_DATA aid;
+    bool force_sandbox = false;
     char buf[256];
 
     boinc_get_init_data_p(&aid);
@@ -130,10 +143,21 @@ int VBOX_VM::initialize() {
     }
 #endif
 
+    // On *nix style systems, VirtualBox expects that there is a home directory specified
+    // by environment variable.  When it doesn't exist it attempts to store logging information
+    // in root's home directory.  Bad things happen if the process isn't owned by root.
+    //
+    // if the HOME environment variable is missing force VirtualBox to use a directory it
+    // has a reasonable chance of writing log files too.
+#ifndef _WIN32
+    if (NULL == getenv("HOME")) {
+        force_sandbox = true;
+    }
+#endif
+
     // Set the location in which the VirtualBox Configuration files can be
     // stored for this instance.
-    //
-    if (aid.using_sandbox) {
+    if (aid.using_sandbox || force_sandbox) {
         virtualbox_user_home = aid.project_dir;
         virtualbox_user_home += "/../virtualbox";
 
@@ -188,15 +212,28 @@ int VBOX_VM::initialize() {
 #endif
     }
 
-    return 0;
+    // Record the VirtualBox version information for later use.
+    command = "--version ";
+    rc = vbm_popen(command, output, "version check");
+
+    // Remove \r or \n from the output spew
+    string::iterator iter = output.begin();
+    while (iter != output.end()) {
+        if (*iter == '\r' || *iter == '\n') {
+            iter = output.erase(iter);
+        } else {
+            ++iter;
+        }
+    }
+
+    virtualbox_version = "VirtualBox " + output;
+
+    return rc;
 }
 
-int VBOX_VM::run() {
+int VBOX_VM::run(double elapsed_time) {
     int retval;
 
-    retval = initialize();
-    if (retval) return retval;
-
     if (!is_registered()) {
         if (is_hdd_registered()) {
             // Handle the case where a previous instance of the same projects VM
@@ -218,6 +255,19 @@ int VBOX_VM::run() {
     // various other functions will work.
     vm_name = vm_master_name;
 
+    // Check to see if the VM is already in a running state, if so, poweroff.
+    poll(false);
+    if (online) {
+        poweroff();
+    }
+
+    // If our last checkpoint time is greater than 0, restore from the previously
+    // saved snapshot
+    if (elapsed_time) {
+        restoresnapshot();
+    }
+
+    // Start the VM
     retval = start();
     if (retval) return retval;
 
@@ -233,16 +283,19 @@ int VBOX_VM::start() {
 
     fprintf(
         stderr,
-        "%s Starting virtual machine.\n",
+        "%s Starting VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
-    command = "startvm \"" + vm_name + "\" --type headless";
+    command = "startvm \"" + vm_name + "\"";
+    if (headless) {
+        command += " --type headless";
+    }
     retval = vbm_popen(command, output, "start VM");
     if (retval) return retval;
 
     // Wait for up to 5 minutes for the VM to switch states.  A system
     // under load can take a while.  Since the poll function can wait for up
-    // to a minute to execute a command we need to make this time based instead
+    // to 45 seconds to execute a command we need to make this time based instead
     // of interation based.
     timeout = dtime() + 300;
     do {
@@ -251,57 +304,97 @@ int VBOX_VM::start() {
         boinc_sleep(1.0);
     } while (timeout >= dtime());
 
-    if (!online) {
+    if (online) {
         fprintf(
             stderr,
-            "%s VM did not start in a timely fashion, aborting job.\n",
+            "%s Successfully started VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
-        return ERR_EXEC;
+        retval = BOINC_SUCCESS;
+    } else {
+        fprintf(
+            stderr,
+            "%s VM did not start within 5 minutes, aborting job.\n",
+            vboxwrapper_msg_prefix(buf, sizeof(buf))
+        );
+        retval = ERR_EXEC;
     }
 
-    return 0;
+    return retval;
 }
 
 int VBOX_VM::stop() {
     string command;
     string output;
-    double timeout;
     char buf[256];
-    int retval;
+    int retval = 0;
 
     fprintf(
         stderr,
-        "%s Stopping virtual machine.\n",
+        "%s Stopping VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     if (online) {
         command = "controlvm \"" + vm_name + "\" savestate";
-        retval = vbm_popen(command, output, "stop VM");
-        if (retval) return retval;
+        retval = vbm_popen(command, output, "stop VM", true, false);
 
-        // Wait for up to 5 minutes for the VM to switch states.  A system
-        // under load can take a while.  Since the poll function can wait for up
-        // to a minute to execute a command we need to make this time based instead
-        // of interation based.
-        timeout = dtime() + 300;
-        do {
-            poll(false);
-            if (!online) break;
-            boinc_sleep(1.0);
-        } while (timeout >= dtime());
+        poll(false);
 
-        if (online) {
+        if (!online) {
             fprintf(
                 stderr,
-                "%s VM did not stop in a timely fashion.\n",
+                "%s Successfully stopped VM.\n",
                 vboxwrapper_msg_prefix(buf, sizeof(buf))
             );
-            return ERR_EXEC;
+            retval = BOINC_SUCCESS;
+        } else {
+            fprintf(
+                stderr,
+                "%s VM did not stop when requested.\n",
+                vboxwrapper_msg_prefix(buf, sizeof(buf))
+            );
+            retval = ERR_EXEC;
         }
     }
 
-    return 0;
+    return retval;
+}
+
+int VBOX_VM::poweroff() {
+    string command;
+    string output;
+    char buf[256];
+    int retval = 0;
+
+    fprintf(
+        stderr,
+        "%s Powering off VM.\n",
+        vboxwrapper_msg_prefix(buf, sizeof(buf))
+    );
+    if (online) {
+        command = "controlvm \"" + vm_name + "\" poweroff";
+        retval = vbm_popen(command, output, "poweroff VM", true, false);
+
+        poll(false);
+
+        if (!online) {
+            fprintf(
+                stderr,
+                "%s Successfully powered off VM.\n",
+                vboxwrapper_msg_prefix(buf, sizeof(buf))
+            );
+            retval = BOINC_SUCCESS;
+        } else {
+            fprintf(
+                stderr,
+                "%s VM did not power off when requested.\n",
+                vboxwrapper_msg_prefix(buf, sizeof(buf))
+            );
+            retval = ERR_EXEC;
+        }
+    }
+
+    return retval;
 }
 
 int VBOX_VM::pause() {
@@ -328,9 +421,142 @@ int VBOX_VM::resume() {
     return 0;
 }
 
+int VBOX_VM::createsnapshot(double elapsed_time) {
+    string command;
+    string output;
+    char buf[256];
+    int retval;
+
+    fprintf(
+        stderr,
+        "%s Creating new snapshot for VM.\n",
+        vboxwrapper_msg_prefix(buf, sizeof(buf))
+    );
+
+    // Pause VM - Try and avoid the live snapshot and trigger an online
+    // snapshot instead.
+    pause();
+
+    // Create new snapshot
+    sprintf(buf, "%d", (int)elapsed_time);
+    command = "snapshot \"" + vm_name + "\" ";
+    command += "take boinc_";
+    command += buf;
+    retval = vbm_popen(command, output, "create new snapshot", true, true, 0);
+    if (retval) return retval;
+
+    // Resume VM
+    resume();
+
+    // Set the suspended flag back to false before deleting the stale
+    // snapshot
+    poll(false);
+
+    // Delete stale snapshot(s), if one exists
+    cleanupsnapshots(false);
+
+    fprintf(
+        stderr,
+        "%s Checkpoint completed.\n",
+        vboxwrapper_msg_prefix(buf, sizeof(buf))
+    );
+
+    return 0;
+}
+
+int VBOX_VM::cleanupsnapshots(bool delete_active) {
+    string command;
+    string output;
+    string line;
+    string uuid;
+    size_t eol_pos;
+    size_t eol_prev_pos;
+    size_t uuid_start;
+    size_t uuid_end;
+    char buf[256];
+    int retval;
+
+
+    // Enumerate snapshot(s)
+    command = "snapshot \"" + vm_name + "\" ";
+    command += "list ";
+    retval = vbm_popen(command, output, "enumerate snapshot(s)");
+    if (retval) return retval;
+
+    // Output should look a little like this:
+    //   Name: Snapshot 2 (UUID: 1751e9a6-49e7-4dcc-ab23-08428b665ddf)
+    //      Name: Snapshot 3 (UUID: 92fa8b35-873a-4197-9d54-7b6b746b2c58)
+    //         Name: Snapshot 4 (UUID: c049023a-5132-45d5-987d-a9cfadb09664) *
+    //
+    eol_prev_pos = 0;
+    eol_pos = output.find("\n");
+    while (eol_pos != string::npos) {
+        line = output.substr(eol_prev_pos, eol_pos - eol_prev_pos);
+
+        // This VM does not yet have any snapshots
+        if (line.find("does not have any snapshots") != string::npos) break;
+
+        // The * signifies that it is the active snapshot and one we do not want to delete
+        if (!delete_active && (line.find("*") != string::npos)) break;
+
+        uuid_start = line.find("(UUID: ");
+        if (uuid_start != string::npos) {
+            // We can parse the virtual machine ID from the line
+            uuid_start += 7;
+            uuid_end = line.find(")", uuid_start);
+            uuid = line.substr(uuid_start, uuid_end - uuid_start);
+
+            fprintf(
+                stderr,
+                "%s Deleting stale snapshot.\n",
+                vboxwrapper_msg_prefix(buf, sizeof(buf))
+            );
+
+            // Delete stale snapshot, if one exists
+            command = "snapshot \"" + vm_name + "\" ";
+            command += "delete \"";
+            command += uuid;
+            command += "\" ";
+            
+            vbm_popen(command, output, "delete stale snapshot", true, false, 0);
+        }
+
+        eol_prev_pos = eol_pos + 1;
+        eol_pos = output.find("\n", eol_prev_pos);
+    }
+
+    return 0;
+}
+
+int VBOX_VM::restoresnapshot() {
+    string command;
+    string output;
+    char buf[256];
+    int retval;
+
+    fprintf(
+        stderr,
+        "%s Restore from previously saved snapshot.\n",
+        vboxwrapper_msg_prefix(buf, sizeof(buf))
+    );
+
+    command = "snapshot \"" + vm_name + "\" ";
+    command += "restorecurrent ";
+    retval = vbm_popen(command, output, "restore current snapshot");
+    if (retval) return retval;
+
+    fprintf(
+        stderr,
+        "%s Restore completed.\n",
+        vboxwrapper_msg_prefix(buf, sizeof(buf))
+    );
+
+    return 0;
+}
+
 void VBOX_VM::cleanup() {
-    stop();
-    deregister_vm();
+    poweroff();
+    deregister_vm(true);
 
     // Give time enough for external processes to finish the cleanup process
     boinc_sleep(5.0);
@@ -377,6 +603,15 @@ void VBOX_VM::poll(bool log_state) {
                 online = true;
             } else if (vmstate == "restoring") {
                 online = true;
+            } else if (vmstate == "livesnapshotting") {
+                online = true;
+            } else if (vmstate == "deletingsnapshotlive") {
+                online = true;
+            } else if (vmstate == "deletingsnapshotlivepaused") {
+                online = true;
+            } else if (vmstate == "aborted") {
+                online = false;
+                crashed = true;
             } else if (vmstate == "gurumeditation") {
                 online = false;
                 crashed = true;
@@ -385,7 +620,7 @@ void VBOX_VM::poll(bool log_state) {
                 if (log_state) {
                     fprintf(
                         stderr,
-                        "%s Virtual machine is no longer is a running state. It is in '%s'.\n",
+                        "%s VM is no longer is a running state. It is in '%s'.\n",
                         vboxwrapper_msg_prefix(buf, sizeof(buf)),
                         vmstate.c_str()
                     );
@@ -395,6 +630,51 @@ void VBOX_VM::poll(bool log_state) {
     }
 }
 
+// Attempt to detect any condition that would prevent VirtualBox from running a VM properly, like:
+// 1. The DCOM service not being started on Windows
+// 2. Vboxmanage not being able to communicate with vboxsvc for some reason
+// 3. VirtualBox driver not loaded for the current Linux kernel.
+//
+// Luckly both of the above conditions can be detected by attempting to detect the host information
+// via vboxmanage and it is cross platform.
+//
+bool VBOX_VM::is_system_ready() {
+    string command;
+    string output;
+    bool rc = true;
+
+    command  = "list hostinfo ";
+    if (vbm_popen(command, output, "host info", false, false) == 0) {
+
+        if (output.find("Processor count:") == string::npos) {
+            rc = false;
+        }
+
+        if (output.find("WARNING: The vboxdrv kernel module is not loaded.") != string::npos) {
+            rc = false;
+        }
+
+    }
+
+    return rc;
+}
+
+bool VBOX_VM::is_registered() {
+    string command;
+    string output;
+
+    command  = "showvminfo \"" + vm_master_name + "\" ";
+    command += "--machinereadable ";
+
+    if (vbm_popen(command, output, "registration", false, false) == 0) {
+        if (output.find("VBOX_E_OBJECT_NOT_FOUND") == string::npos) {
+            // Error message not found in text
+            return true;
+        }
+    }
+    return false;
+}
+
 bool VBOX_VM::is_hdd_registered() {
     string command;
     string output;
@@ -413,22 +693,6 @@ bool VBOX_VM::is_hdd_registered() {
     return false;
 }
 
-bool VBOX_VM::is_registered() {
-    string command;
-    string output;
-
-    command  = "showvminfo \"" + vm_master_name + "\" ";
-    command += "--machinereadable ";
-
-    if (vbm_popen(command, output, "registration", false, false) == 0) {
-        if (output.find("VBOX_E_OBJECT_NOT_FOUND") == string::npos) {
-            // Error message not found in text
-            return true;
-        }
-    }
-    return false;
-}
-
 bool VBOX_VM::is_extpack_installed() {
     string command;
     string output;
@@ -462,7 +726,7 @@ int VBOX_VM::register_vm() {
 
     fprintf(
         stderr,
-        "%s Registering virtual machine. (%s) \n",
+        "%s Registering VM. (%s) \n",
         vboxwrapper_msg_prefix(buf, sizeof(buf)),
         vm_name.c_str()
     );
@@ -483,7 +747,7 @@ int VBOX_VM::register_vm() {
     //
     fprintf(
         stderr,
-        "%s Modifying virtual machine.\n",
+        "%s Modifying VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "modifyvm \"" + vm_name + "\" ";
@@ -502,35 +766,39 @@ int VBOX_VM::register_vm() {
     retval = vbm_popen(command, output, "modify");
     if (retval) return retval;
 
-    if ((vm_cpu_count == "1") ||
-        (!strstr(aid.host_info.p_features, "vmx") && !strstr(aid.host_info.p_features, "svm"))) {
+    // Only perform hardware acceleration check on 32-bit VM types, 64-bit VM types require it.
+    //
+    if (os_name.find("_64") == std::string::npos) {
         // Check to see if the processor supports hardware acceleration for virtualization
         // If it doesn't, disable the use of it in VirtualBox. Multi-core jobs require hardware
         // acceleration and actually override this setting.
         //
-        fprintf(
-            stderr,
-            "%s Disabling hardware acceleration support for virtualization.\n",
-            vboxwrapper_msg_prefix(buf, sizeof(buf))
-        );
-        command  = "modifyvm \"" + vm_name + "\" ";
-        command += "--hwvirtex off ";
+        if ((vm_cpu_count == "1") ||
+            (!strstr(aid.host_info.p_features, "vmx") && !strstr(aid.host_info.p_features, "svm"))) {
+            fprintf(
+                stderr,
+                "%s Disabling hardware acceleration support for virtualization.\n",
+                vboxwrapper_msg_prefix(buf, sizeof(buf))
+            );
+            command  = "modifyvm \"" + vm_name + "\" ";
+            command += "--hwvirtex off ";
 
-        retval = vbm_popen(command, output, "VT-x/AMD-V support");
-        if (retval) return retval;
+            retval = vbm_popen(command, output, "VT-x/AMD-V support");
+            if (retval) return retval;
+        }
     }
 
     // Add storage controller to VM
     //
     fprintf(
         stderr,
-        "%s Adding storage controller to virtual machine.\n",
+        "%s Adding storage controller to VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "storagectl \"" + vm_name + "\" ";
-    command += "--name \"IDE Controller\" ";
-    command += "--add ide ";
-    command += "--controller PIIX4 ";
+    command += "--name \"Hard Disk Controller\" ";
+    command += "--add \"" + vm_disk_controller_type + "\" ";
+    command += "--controller \"" + vm_disk_controller_model + "\" ";
 
     retval = vbm_popen(command, output, "add storage controller (fixed disk)");
     if (retval) return retval;
@@ -550,11 +818,11 @@ int VBOX_VM::register_vm() {
     //
     fprintf(
         stderr,
-        "%s Adding virtual disk drive to virtual machine.\n",
+        "%s Adding virtual disk drive to VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "storageattach \"" + vm_name + "\" ";
-    command += "--storagectl \"IDE Controller\" ";
+    command += "--storagectl \"Hard Disk Controller\" ";
     command += "--port 0 ";
     command += "--device 0 ";
     command += "--type hdd ";
@@ -588,7 +856,7 @@ int VBOX_VM::register_vm() {
 
         fprintf(
             stderr,
-            "%s Adding virtual floppy disk drive to virtual machine.\n",
+            "%s Adding virtual floppy disk drive to VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
         command  = "storageattach \"" + vm_name + "\" ";
@@ -618,7 +886,7 @@ int VBOX_VM::register_vm() {
 
             fprintf(
                 stderr,
-                "%s Enabling virtual machine firewall rules.\n",
+                "%s Enabling VM firewall rules.\n",
                 vboxwrapper_msg_prefix(buf, sizeof(buf))
             );
 
@@ -638,7 +906,7 @@ int VBOX_VM::register_vm() {
     if (enable_remotedesktop) {
         fprintf(
             stderr,
-            "%s Enabling remote desktop for virtual machine.\n",
+            "%s Enabling remote desktop for VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
         if (!is_extpack_installed()) {
@@ -669,7 +937,7 @@ int VBOX_VM::register_vm() {
     if (enable_shared_directory) {
         fprintf(
             stderr,
-            "%s Enabling shared directory for virtual machine.\n",
+            "%s Enabling shared directory for VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
         command  = "sharedfolder add \"" + vm_name + "\" ";
@@ -683,7 +951,7 @@ int VBOX_VM::register_vm() {
     return 0;
 }
 
-int VBOX_VM::deregister_vm() {
+int VBOX_VM::deregister_vm(bool delete_media) {
     string command;
     string output;
     string virtual_machine_slot_directory;
@@ -693,56 +961,49 @@ int VBOX_VM::deregister_vm() {
 
     fprintf(
         stderr,
-        "%s Deregistering virtual machine.\n",
+        "%s Deregistering VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
 
 
-    // Discard any saved state information
+    // Cleanup any left-over snapshots
     //
-    fprintf(
-        stderr,
-        "%s Discarding saved state of virtual machine.\n",
-        vboxwrapper_msg_prefix(buf, sizeof(buf))
-    );
-    command  = "discardstate \"" + vm_name + "\" ";
-
-    vbm_popen(command, output, "discard state", false);
+    cleanupsnapshots(true);
 
     // Delete its storage controller(s)
     //
     fprintf(
         stderr,
-        "%s Removing storage controller(s) from virtual machine.\n",
+        "%s Removing storage controller(s) from VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "storagectl \"" + vm_name + "\" ";
     command += "--name \"IDE Controller\" ";
     command += "--remove ";
 
-    vbm_popen(command, output, "deregister storage controller (fixed disk)");
+    vbm_popen(command, output, "deregister storage controller (fixed disk)", false, false);
 
     if (enable_floppyio) {
         command  = "storagectl \"" + vm_name + "\" ";
         command += "--name \"Floppy Controller\" ";
         command += "--remove ";
 
-        vbm_popen(command, output, "deregister storage controller (floppy disk)");
+        vbm_popen(command, output, "deregister storage controller (floppy disk)", false, false);
     }
 
-    // Next delete VM
+    // Next, delete VM
     //
     fprintf(
         stderr,
-        "%s Removing virtual machine from VirtualBox.\n",
+        "%s Removing VM from VirtualBox.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "unregistervm \"" + vm_name + "\" ";
     command += "--delete ";
 
-    vbm_popen(command, output, "delete VM");
+    vbm_popen(command, output, "delete VM", false, false);
 
-    // Lastly delete medium from Virtual Box Media Registry
+    // Lastly delete medium(s) from Virtual Box Media Registry
     //
     fprintf(
         stderr,
@@ -750,8 +1011,26 @@ int VBOX_VM::deregister_vm() {
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     command  = "closemedium disk \"" + virtual_machine_slot_directory + "/" + image_filename + "\" ";
+    if (delete_media) {
+        command += "--delete ";
+    }
+
+    vbm_popen(command, output, "remove virtual disk", false, false);
+
+    if (enable_floppyio) {
+        fprintf(
+            stderr,
+            "%s Removing virtual floppy disk from VirtualBox.\n",
+            vboxwrapper_msg_prefix(buf, sizeof(buf))
+        );
+        command  = "closemedium floppy \"" + virtual_machine_slot_directory + "/" + floppy_image_filename + "\" ";
+        if (delete_media) {
+            command += "--delete ";
+        }
+
+        vbm_popen(command, output, "remove virtual floppy disk", false, false);
+    }
 
-    vbm_popen(command, output, "remove virtual disk");
     return 0;
 }
 
@@ -792,12 +1071,16 @@ int VBOX_VM::deregister_stale_vm() {
         vm_name = output.substr(uuid_start, uuid_end - uuid_start);
 
         // Deregister stale VM by UUID
-        return deregister_vm();
+        return deregister_vm(false);
     } else {
         // Did the user delete the VM in VirtualBox and not the medium?  If so,
         // just remove the medium.
         command  = "closemedium disk \"" + virtual_machine_slot_directory + "/" + image_filename + "\" ";
-        vbm_popen(command, output, "remove virtual disk ", false);
+        vbm_popen(command, output, "remove virtual disk", false);
+        if (enable_floppyio) {
+            command  = "closemedium floppy \"" + virtual_machine_slot_directory + "/" + floppy_image_filename + "\" ";
+            vbm_popen(command, output, "remove virtual floppy disk", false);
+        }
     }
     return 0;
 }
@@ -848,15 +1131,19 @@ int VBOX_VM::get_install_directory(string& virtualbox_install_directory ) {
 
     if (hkSetupHive) RegCloseKey(hkSetupHive);
     if (lpszRegistryValue) free(lpszRegistryValue);
+    if (virtualbox_install_directory.empty()) {
+        return 1;
+    }
+    return 0;
 #else
     virtualbox_install_directory = "";
-#endif
     return 0;
+#endif
 }
 
 // Returns the current directory in which the executable resides.
 //
-int VBOX_VM::get_slot_directory( string& dir ) {
+int VBOX_VM::get_slot_directory(string& dir) {
     char slot_dir[256];
 
     getcwd(slot_dir, sizeof(slot_dir));
@@ -1124,14 +1411,14 @@ int VBOX_VM::get_vm_process_id(int& process_id) {
 }
 
 int VBOX_VM::get_port_forwarding_port() {
-    struct sockaddr_in addr;
+    sockaddr_in addr;
     BOINC_SOCKLEN_T addrsize;
     int sock;
     int retval;
 
-    addrsize = sizeof(struct sockaddr_in);
+    addrsize = sizeof(sockaddr_in);
 
-    memset(&addr, 0, sizeof(addr));
+    memset(&addr, 0, sizeof(sockaddr_in));
     addr.sin_family = AF_INET;
     addr.sin_port = htons(pf_host_port);
     addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -1139,12 +1426,12 @@ int VBOX_VM::get_port_forwarding_port() {
     retval = boinc_socket(sock);
     if (retval) return retval;
  
-    retval = bind(sock, (struct sockaddr *)&addr, addrsize);
+    retval = bind(sock, (const sockaddr*)&addr, addrsize);
     if (retval < 0) {
         boinc_close_socket(sock);
 
         // Lets see if we can get anything useable at this point
-        memset(&addr, 0, sizeof(addr));
+        memset(&addr, 0, sizeof(sockaddr_in));
         addr.sin_family = AF_INET;
         addr.sin_port = htons(0);
         addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -1152,29 +1439,29 @@ int VBOX_VM::get_port_forwarding_port() {
         retval = boinc_socket(sock);
         if (retval) return retval;
      
-        retval = bind(sock, (struct sockaddr *)&addr, addrsize);
+        retval = bind(sock, (const sockaddr*)&addr, addrsize);
         if (retval < 0) {
             boinc_close_socket(sock);
             return ERR_BIND;
         }
     }
 
-    getsockname(sock, (struct sockaddr *)&addr, &addrsize);
-    pf_host_port = addr.sin_port;
+    getsockname(sock, (sockaddr*)&addr, &addrsize);
+    pf_host_port = ntohs(addr.sin_port);
 
     boinc_close_socket(sock);
     return 0;
 }
 
 int VBOX_VM::get_remote_desktop_port() {
-    struct sockaddr_in addr;
+    sockaddr_in addr;
     BOINC_SOCKLEN_T addrsize;
     int sock;
     int retval;
 
-    addrsize = sizeof(struct sockaddr_in);
+    addrsize = sizeof(sockaddr_in);
 
-    memset(&addr, 0, sizeof(addr));
+    memset(&addr, 0, sizeof(sockaddr_in));
     addr.sin_family = AF_INET;
     addr.sin_port = htons(0);
     addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
@@ -1182,14 +1469,14 @@ int VBOX_VM::get_remote_desktop_port() {
     retval = boinc_socket(sock);
     if (retval) return retval;
  
-    retval = bind(sock, (struct sockaddr *)&addr, addrsize);
+    retval = bind(sock, (const sockaddr*)&addr, addrsize);
     if (retval < 0) {
         boinc_close_socket(sock);
         return ERR_BIND;
     }
 
-    getsockname(sock, (struct sockaddr *)&addr, &addrsize);
-    rd_host_port = addr.sin_port;
+    getsockname(sock, (sockaddr*)&addr, &addrsize);
+    rd_host_port = ntohs(addr.sin_port);
 
     boinc_close_socket(sock);
     return 0;
@@ -1218,7 +1505,7 @@ int VBOX_VM::set_network_access(bool enabled) {
     if (enabled) {
         fprintf(
             stderr,
-            "%s Enabling network access for virtual machine.\n",
+            "%s Enabling network access for VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
         command  = "modifyvm \"" + vm_name + "\" ";
@@ -1229,7 +1516,7 @@ int VBOX_VM::set_network_access(bool enabled) {
     } else {
         fprintf(
             stderr,
-            "%s Disabling network access for virtual machine.\n",
+            "%s Disabling network access for VM.\n",
             vboxwrapper_msg_prefix(buf, sizeof(buf))
         );
         command  = "modifyvm \"" + vm_name + "\" ";
@@ -1251,7 +1538,7 @@ int VBOX_VM::set_cpu_usage_fraction(double x) {
     //
     fprintf(
         stderr,
-        "%s Setting cpu throttle for virtual machine. (%d%%)\n",
+        "%s Setting cpu throttle for VM. (%d%%)\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf)),
         (int)x
     );
@@ -1277,7 +1564,7 @@ int VBOX_VM::set_network_max_bytes_sec(double x) {
     //
     fprintf(
         stderr,
-        "%s Setting network throttle for virtual machine.\n",
+        "%s Setting network throttle for VM.\n",
         vboxwrapper_msg_prefix(buf, sizeof(buf))
     );
     sprintf(buf, "%d", (int)(x*8./1000.));
@@ -1333,7 +1620,7 @@ void VBOX_VM::reset_vm_process_priority() {
 
 // If there are errors we can recover from, process them here.
 //
-int VBOX_VM::vbm_popen(string& arguments, string& output, const char* item, bool log_error, bool retry_failures) {
+int VBOX_VM::vbm_popen(string& arguments, string& output, const char* item, bool log_error, bool retry_failures, unsigned int timeout) {
     int retval = 0;
     int retry_count = 0;
     double sleep_interval = 1.0;
@@ -1341,7 +1628,7 @@ int VBOX_VM::vbm_popen(string& arguments, string& output, const char* item, bool
     string retry_notes;
 
     do {
-        retval = vbm_popen_raw(arguments, output);
+        retval = vbm_popen_raw(arguments, output, timeout);
         if (retval) {
 
             // VirtualBox designed the concept of sessions to prevent multiple applications using
@@ -1368,7 +1655,7 @@ int VBOX_VM::vbm_popen(string& arguments, string& output, const char* item, bool
             if (0x80bb0007 == retval) {
                 if (retry_notes.find("Another VirtualBox management") == string::npos) {
                     retry_notes += "Another VirtualBox management application has locked the session for\n";
-                    retry_notes += "this virtual machine. BOINC cannot properly monitor this virtual machine\n";
+                    retry_notes += "this VM. BOINC cannot properly monitor this VM\n";
                     retry_notes += "and so this job will be aborted.\n\n";
                 }
                 if (retry_count) {
@@ -1423,7 +1710,7 @@ int VBOX_VM::vbm_popen(string& arguments, string& output, const char* item, bool
 
 // Execute the vbox manage application and copy the output to the buffer.
 //
-int VBOX_VM::vbm_popen_raw(string& arguments, string& output) {
+int VBOX_VM::vbm_popen_raw(string& arguments, string& output, unsigned int timeout) {
     char buf[256];
     string command;
     size_t errcode_start;
@@ -1502,6 +1789,10 @@ int VBOX_VM::vbm_popen_raw(string& arguments, string& output) {
 
     // Wait until process has completed
     while(1) {
+        if (timeout == 0) {
+            WaitForSingleObject(pi.hProcess, INFINITE);
+        }
+
         GetExitCodeProcess(pi.hProcess, &ulExitCode);
 
         // Copy stdout/stderr to output buffer, handle in the loop so that we can
@@ -1522,14 +1813,10 @@ int VBOX_VM::vbm_popen_raw(string& arguments, string& output) {
         if (ulExitCode != STILL_ACTIVE) break;
 
         // Timeout?
-        if (ulExitTimeout >= 60000) {
-            fprintf(
-                stderr,
-                "%s Process Timeout!.\n",
-                vboxwrapper_msg_prefix(buf, sizeof(buf))
-            );
-
+        if (ulExitTimeout >= (timeout * 1000)) {
             TerminateProcess(pi.hProcess, EXIT_FAILURE);
+            ulExitCode = 0;
+            retval = ERR_TIMEOUT;
             Sleep(1000);
         }
 
@@ -1557,6 +1844,7 @@ CLEANUP:
 
         // If something couldn't be found, just return ERR_FOPEN
         if (!retval) retval = ERR_FOPEN;
+
     }
 
 #else
