@@ -557,7 +557,7 @@ bool do_pass() {
         if (config.enable_assignment) {
             DB_ASSIGNMENT asg;
             char buf2[256];
-            sprintf(buf, "workunitid=%d", wu.id);
+            sprintf(buf2, "workunitid=%d", wu.id);
             asg.delete_from_db_multi(buf2);
         }
 
@@ -738,7 +738,7 @@ int main(int argc, char** argv) {
         }
         if (!do_pass() && !one_pass) {
             log_messages.printf(MSG_NORMAL, "Sleeping....\n");
-            sleep(sleep_sec);
+            daemon_sleep(sleep_sec);
         }
         if (one_pass) {
             break;
@@ -749,4 +749,4 @@ int main(int argc, char** argv) {
     exit(0);
 }
 
-const char *BOINC_RCSID_0c1c4336f1 = "$Id: db_purge.cpp 25344 2012-02-27 11:08:14Z bema $";
+const char *BOINC_RCSID_0c1c4336f1 = "$Id$";
