@@ -50,6 +50,7 @@
 #include "sched_vda.h"
 
 #include "credit.h"
+#include "sched_files.h"
 #include "sched_main.h"
 #include "sched_types.h"
 #include "sched_util.h"
@@ -72,11 +73,15 @@ static bool find_host_by_other(DB_USER& user, HOST req_host, DB_HOST& host) {
 
     // don't dig through hosts of these users
     // prevents flooding the DB with slow queries from users with many hosts
-    for(unsigned int i=0; i < config.dont_search_host_for_userid.size(); i++)
-      if (user.id == config.dont_search_host_for_userid[i])
-        return false;
+    //
+    for (unsigned int i=0; i < config.dont_search_host_for_userid.size(); i++) {
+        if (user.id == config.dont_search_host_for_userid[i]) {
+            return false;
+        }    
+    }
 
-    // Only check if the fields are populated
+    // Only check if all the fields are populated
+    //
     if (strlen(req_host.domain_name) && strlen(req_host.last_ip_addr) && strlen(req_host.os_name) && strlen(req_host.p_model)) {
         strcpy(dn, req_host.domain_name);
         escape_string(dn, 512);
@@ -167,7 +172,7 @@ void unlock_sched() {
 // find the user's most recently-created host with given host CPID
 //
 static bool find_host_by_cpid(DB_USER& user, char* host_cpid, DB_HOST& host) {
-    char buf[256], buf2[256];
+    char buf[1024], buf2[256];
     sprintf(buf, "%s%s", host_cpid, user.email_addr);
     md5_block((const unsigned char*)buf, strlen(buf), buf2);
 
@@ -236,7 +241,7 @@ static void mark_results_over(DB_HOST& host) {
 //
 int authenticate_user() {
     int retval;
-    char buf[256];
+    char buf[1024];
     DB_HOST host;
     DB_USER user;
     DB_TEAM team;
@@ -294,6 +299,7 @@ int authenticate_user() {
                 strlcpy(
                     user.authenticator, g_request->authenticator, sizeof(user.authenticator)
                 );
+                escape_string(user.authenticator, sizeof(user.authenticator));
                 sprintf(buf, "where authenticator='%s'", user.authenticator);
                 retval = user.lookup(buf);
                 if (retval) {
@@ -360,6 +366,7 @@ lookup_user_and_make_new_host:
                 user.authenticator, g_request->authenticator,
                 sizeof(user.authenticator)
             );
+            escape_string(user.authenticator, sizeof(user.authenticator));
             sprintf(buf, "where authenticator='%s'", user.authenticator);
             retval = user.lookup(buf);
         }
@@ -409,7 +416,7 @@ make_new_host:
         //
         // NOTE: If the client was run with --allow_multiple_clients, skip this.
         //
-        if ((g_request->allow_multiple_clients==1)
+        if ((g_request->allow_multiple_clients != 1)
             && find_host_by_other(user, g_request->host, host)
         ) {
             log_messages.printf(MSG_NORMAL,
@@ -492,7 +499,7 @@ got_host:
 static int modify_host_struct(HOST& host) {
     host.timezone = g_request->host.timezone;
     strncpy(host.domain_name, g_request->host.domain_name, sizeof(host.domain_name));
-    char buf[256], buf2[256];
+    char buf[1024], buf2[1024];
     sprintf(buf, "[BOINC|%d.%d.%d]",
         g_request->core_client_major_version,
         g_request->core_client_minor_version,
@@ -506,13 +513,21 @@ static int modify_host_struct(HOST& host) {
         strlcat(host.serialnum, buf2, sizeof(host.serialnum));
     }
     if (strcmp(host.last_ip_addr, g_request->host.last_ip_addr)) {
-        strncpy(host.last_ip_addr, g_request->host.last_ip_addr, sizeof(host.last_ip_addr));
+        strncpy(
+            host.last_ip_addr, g_request->host.last_ip_addr,
+            sizeof(host.last_ip_addr)
+        );
+        host.nsame_ip_addr = 0;
     } else {
         host.nsame_ip_addr++;
     }
     host.on_frac = g_request->host.on_frac;
     host.connected_frac = g_request->host.connected_frac;
     host.active_frac = g_request->host.active_frac;
+    host.gpu_active_frac = g_request->host.gpu_active_frac;
+    host.cpu_and_network_available_frac = g_request->host.cpu_and_network_available_frac;
+    host.client_start_time = g_request->host.client_start_time;
+    host.previous_uptime = g_request->host.previous_uptime;
     host.duration_correction_factor = g_request->host.duration_correction_factor;
     host.p_ncpus = g_request->host.p_ncpus;
     strncpy(host.p_vendor, g_request->host.p_vendor, sizeof(host.p_vendor));
@@ -691,7 +706,7 @@ int send_result_abort() {
 // 2) send global prefs in reply msg if needed
 //
 int handle_global_prefs() {
-    char buf[BLOB_SIZE];
+    char buf[BLOB_SIZE+256];
     g_reply->send_global_prefs = false;
     bool have_working_prefs = (strlen(g_request->working_global_prefs_xml)>0);
     bool have_master_prefs = (strlen(g_request->global_prefs_xml)>0);
@@ -817,7 +832,7 @@ int handle_global_prefs() {
 bool send_code_sign_key(char* code_sign_key) {
     char* oldkey, *signature;
     int i, retval;
-    char path[256];
+    char path[MAXPATHLEN];
 
     if (strlen(g_request->code_sign_key)) {
         if (strcmp(g_request->code_sign_key, code_sign_key)) {
@@ -1071,6 +1086,7 @@ static inline bool requesting_work() {
     if (g_request->cpu_req_secs > 0) return true;
     if (g_request->coprocs.nvidia.count && g_request->coprocs.nvidia.req_secs) return true;
     if (g_request->coprocs.ati.count && g_request->coprocs.ati.req_secs) return true;
+    if (ssp->have_nci_app) return true;
     return false;
 }
 
@@ -1088,6 +1104,10 @@ void process_request(char* code_sign_key) {
 
     memset(&g_reply->wreq, 0, sizeof(g_reply->wreq));
 
+    // if client has sticky files we don't need any more, tell it
+    //
+    do_file_delete_regex();
+
     // if different major version of BOINC, just send a message
     //
     if (wrong_core_client_version()
@@ -1113,11 +1133,6 @@ void process_request(char* code_sign_key) {
             have_no_work = ssp->no_work(g_pid);
             if (have_no_work) {
                 g_wreq->no_jobs_available = true;
-                if (config.debug_send) {
-                    log_messages.printf(MSG_NORMAL,
-                        "[send] No jobs in shmem\n"
-                    );
-                }
             }
             unlock_sema();
         }
@@ -1206,13 +1221,13 @@ void process_request(char* code_sign_key) {
     x = drand()*86400;
     srand(retval);
     last_rpc_time = g_reply->host.rpc_time;
-    t = g_reply->host.rpc_time + x;
+    t = (time_t)(g_reply->host.rpc_time + x);
     rpc_time_tm = localtime(&t);
     g_request->last_rpc_dayofyear = rpc_time_tm->tm_yday;
 
     t = time(0);
     g_reply->host.rpc_time = t;
-    t += x;
+    t += (time_t)x;
     rpc_time_tm = localtime(&t);
     g_request->current_rpc_dayofyear = rpc_time_tm->tm_yday;
 
@@ -1228,9 +1243,14 @@ void process_request(char* code_sign_key) {
     //
     platform = ssp->lookup_platform(g_request->platform.name);
     if (platform) g_request->platforms.list.push_back(platform);
-    for (i=0; i<g_request->alt_platforms.size(); i++) {
-        platform = ssp->lookup_platform(g_request->alt_platforms[i].name);
-        if (platform) g_request->platforms.list.push_back(platform);
+
+    // if primary platform is anonymous, ignore alternate platforms
+    //
+    if (strcmp(g_request->platform.name, "anonymous")) {
+        for (i=0; i<g_request->alt_platforms.size(); i++) {
+            platform = ssp->lookup_platform(g_request->alt_platforms[i].name);
+            if (platform) g_request->platforms.list.push_back(platform);
+        }
     }
     if (g_request->platforms.list.size() == 0) {
         sprintf(buf, "%s %s",
@@ -1270,6 +1290,7 @@ void process_request(char* code_sign_key) {
     if (g_request->have_other_results_list) {
         if (ok_to_send_work
             && (config.resend_lost_results || g_wreq->resend_lost_results)
+            && !g_request->results_truncated
         ) {
             if (resend_lost_work()) {
                 ok_to_send_work = false;
@@ -1285,6 +1306,14 @@ void process_request(char* code_sign_key) {
             ok_to_send_work = false;
         }
 
+        if (have_no_work) {
+            if (config.debug_send) {
+                log_messages.printf(MSG_NORMAL,
+                    "[send] No jobs in shmem cache\n"
+                );
+            }
+        }
+
         // if last RPC was within config.min_sendwork_interval, don't send work
         //
         if (!have_no_work && ok_to_send_work) {
@@ -1378,16 +1407,16 @@ void handle_request(FILE* fin, FILE* fout, char* code_sign_key) {
     double start_time = dtime();
     if (!p){
         process_request(code_sign_key);
+
+        if ((config.locality_scheduling || config.locality_scheduler_fraction) && !sreply.nucleus_only) {
+            send_file_deletes();
+        }
     } else {
         sprintf(buf, "Error in request message: %s", p);
         log_incomplete_request();
         sreply.insert_message(buf, "low");
     }
 
-    if ((config.locality_scheduling || config.locality_scheduler_fraction) && !sreply.nucleus_only) {
-        send_file_deletes();
-    }
-
     if (config.debug_user_messages) {
         log_user_messages();
     }
@@ -1402,4 +1431,4 @@ void handle_request(FILE* fin, FILE* fout, char* code_sign_key) {
     }
 }
 
-const char *BOINC_RCSID_2ac231f9de = "$Id: handle_request.cpp 25355 2012-02-29 07:22:59Z davea $";
+const char *BOINC_RCSID_2ac231f9de = "$Id$";
