@@ -55,9 +55,26 @@ struct GPU_REQUIREMENTS {
     }
 };
 
-extern GPU_REQUIREMENTS cuda_requirements;
-extern GPU_REQUIREMENTS ati_requirements;
+extern GPU_REQUIREMENTS gpu_requirements[NPROC_TYPES];
 
 extern bool wu_is_infeasible_custom(WORKUNIT&, APP&, BEST_APP_VERSION&);
 extern bool app_plan(SCHEDULER_REQUEST&, char* plan_class, HOST_USAGE&);
 extern void handle_file_xfer_results();
+
+// Suppose we have a computation that uses two devices alternately.
+// The devices have speeds s1 and s2.
+// The fraction of work done on device 1 is frac.
+//
+// This function returns:
+// 1) the overall speed
+// 2) the utilization of device 1, which is always in (0, 1).
+//
+static inline void coproc_perf(
+    double s1, double s2, double frac,
+    double& speed, double& u1
+) {
+    double y = (frac*s2 + (1-frac)*s1);
+    speed = s1*s2/y;
+        // do the math
+    u1 = frac*s2/y;
+}
