@@ -23,6 +23,11 @@
 #include "boinc_db.h"
 #include "sched_types.h"
 
+const int MAX_GPUS = 64;
+    // don't believe clients who claim they have more GPUs than this
+const int MAX_CPUS = 4096;
+    // don't believe clients who claim they have more CPUs than this
+
 extern void send_work();
 
 extern int add_result_to_reply(
@@ -74,5 +79,6 @@ extern void send_work_setup();
 extern int effective_ncpus();
 extern int preferred_app_message_index;
 extern void update_n_jobs_today();
+extern int nfiles_on_host(WORKUNIT&);
 
 #endif
