@@ -52,7 +52,7 @@ struct RESOURCE {
     }
 };
 
-// message intended for human eyes
+// a message for the volunteer
 //
 struct USER_MESSAGE {
     std::string message;
@@ -61,8 +61,8 @@ struct USER_MESSAGE {
 };
 
 struct HOST_USAGE {
-    double ncudas;
-    double natis;
+    int proc_type;
+    double gpu_usage;
     double gpu_ram;
     double avg_ncpus;
     double max_ncpus;
@@ -71,12 +71,12 @@ struct HOST_USAGE {
         // Taken from host_app_version elapsed time statistics if available,
         // else on estimate provided by app_plan()
     double peak_flops;
-        // stored in result.estimated_flops, and used for credit calculations
+        // stored in result.flops_estimate, and used for credit calculations
     char cmdline[256];
 
     HOST_USAGE() {
-        ncudas = 0;
-        natis = 0;
+        proc_type = PROC_TYPE_CPU;
+        gpu_usage = 0;
         gpu_ram = 0;
         avg_ncpus = 1;
         max_ncpus = 1;
@@ -85,8 +85,8 @@ struct HOST_USAGE {
         strcpy(cmdline, "");
     }
     void sequential_app(double flops) {
-        ncudas = 0;
-        natis = 0;
+        proc_type = PROC_TYPE_CPU;
+        gpu_usage = 0;
         gpu_ram = 0;
         avg_ncpus = 1;
         max_ncpus = 1;
@@ -96,38 +96,30 @@ struct HOST_USAGE {
         strcpy(cmdline, "");
     }
     inline bool is_sequential_app() {
-         if (ncudas) return false;
-         if (natis) return false;
+         if (proc_type != PROC_TYPE_CPU) return false;
          if (avg_ncpus != 1) return false;
          return true;
     }
     inline int resource_type() {
-        if (ncudas) {
-            return ANON_PLATFORM_NVIDIA;
-        } else if (natis) {
-            return ANON_PLATFORM_ATI;
+        switch (proc_type) {
+        case PROC_TYPE_NVIDIA_GPU: return ANON_PLATFORM_NVIDIA;
+        case PROC_TYPE_AMD_GPU: return ANON_PLATFORM_ATI;
+        case PROC_TYPE_INTEL_GPU: return ANON_PLATFORM_INTEL;
+        default: return ANON_PLATFORM_CPU;
         }
-        return ANON_PLATFORM_CPU;
-    }
-    inline const char* resource_name() {
-        if (ncudas) {
-            return "nvidia GPU";
-        } else if (natis) {
-            return "ATI GPU";
-        }
-        return "CPU";
     }
     inline bool uses_gpu() {
-        if (ncudas) return true;
-        if (natis) return true;
-        return false;
+        return (proc_type != PROC_TYPE_CPU);
     }
 };
 
-// a description of a sticky file on host.
+// a description of a sticky file on host, or a job input file
 //
 struct FILE_INFO {
     char name[256];
+    double nbytes;
+    int status;
+    bool sticky;
 
     int parse(XML_PARSER&);
 };
@@ -289,6 +281,8 @@ struct SCHEDULER_REQUEST {
         // currently queued jobs saturate the CPU for this long;
         // used for crude deadline check
     double duration_correction_factor;
+    double uptime;
+    double previous_uptime;
     char global_prefs_xml[BLOB_SIZE];
     char working_global_prefs_xml[BLOB_SIZE];
     char code_sign_key[4096];
@@ -303,6 +297,10 @@ struct SCHEDULER_REQUEST {
     COPROCS coprocs;
     std::vector<SCHED_DB_RESULT> results;
         // completed results being reported
+    bool results_truncated;
+        // set if (to limit memory usage) we capped this size of "results"
+        // In this case, don't resend lost results
+        // since we don't know what was lost.
     std::vector<RESULT> file_xfer_results;
     std::vector<MSG_FROM_HOST_DESC> msgs_from_host;
     std::vector<FILE_INFO> file_infos;
@@ -355,16 +353,17 @@ struct WORK_REQ {
 
     // the following defined if anonymous platform
     //
-    bool have_cpu_apps;
-    bool have_cuda_apps;
-    bool have_ati_apps;
+    bool client_has_apps_for_proc_type[NPROC_TYPES];
 
     // Flags used by old-style scheduling,
     // while making multiple passes through the work array
+    //
     bool infeasible_only;
     bool reliable_only;
     bool user_apps_only;
     bool beta_only;
+    bool locality_sched_lite;
+        // for LSL apps, send only jobs where client has > 0 files
 
     bool resend_lost_results;
         // this is set if the request is reporting a result
@@ -374,9 +373,8 @@ struct WORK_REQ {
         // so check and resend just in case.
 
     // user preferences
-    bool no_cuda;
-    bool no_ati;
-    bool no_cpu;
+    //
+    bool dont_use_proc_type[NPROC_TYPES];
     bool allow_non_preferred_apps;
     bool allow_beta_work;
     std::vector<APP_INFO> preferred_apps;
@@ -389,30 +387,22 @@ struct WORK_REQ {
 
     // 6.7+ clients send separate requests for different resource types:
     //
-    double cpu_req_secs;        // instance-seconds requested
-    double cpu_req_instances;   // number of idle instances, use if possible
-    double cuda_req_secs;
-    double cuda_req_instances;
-    double ati_req_secs;
-    double ati_req_instances;
-    inline bool need_cpu() {
-        return (cpu_req_secs>0) || (cpu_req_instances>0);
-    }
-    inline bool need_cuda() {
-        return (cuda_req_secs>0) || (cuda_req_instances>0);
-    }
-    inline bool need_ati() {
-        return (ati_req_secs>0) || (ati_req_instances>0);
+    double req_secs[NPROC_TYPES];
+        // instance-seconds requested
+    double req_instances[NPROC_TYPES];
+        // number of idle instances, use if possible
+    inline bool need_proc_type(int t) {
+        return (req_secs[t]>0) || (req_instances[t]>0);
     }
     inline void clear_cpu_req() {
-        cpu_req_secs = 0;
-        cpu_req_instances = 0;
+        req_secs[PROC_TYPE_CPU] = 0;
+        req_instances[PROC_TYPE_CPU] = 0;
     }
     inline void clear_gpu_req() {
-        cuda_req_secs = 0;
-        cuda_req_instances = 0;
-        ati_req_secs = 0;
-        ati_req_instances = 0;
+        for (int i=1; i<NPROC_TYPES; i++) {
+            req_secs[i] = 0;
+            req_instances[i] = 0;
+        }
     }
 
     // older clients send send a single number, the requested duration of jobs
@@ -468,15 +458,11 @@ struct WORK_REQ {
     bool hr_reject_temp;
     bool hr_reject_perm;
     bool outdated_client;
-    bool no_cuda_prefs;
-    bool no_ati_prefs;
-    bool no_cpu_prefs;
     bool max_jobs_on_host_exceeded;
     bool max_jobs_on_host_cpu_exceeded;
     bool max_jobs_on_host_gpu_exceeded;
     bool no_jobs_available;     // project has no work right now
     int max_jobs_per_rpc;
-    void update_for_result(double seconds_filled);
     void add_no_work_message(const char*);
     void get_job_limits();
 
