@@ -35,6 +35,7 @@
 #include <string>
 #include <signal.h>
 #include <sys/time.h>
+#include <sys/param.h>
 
 #include "boinc_db.h"
 #include "util.h"
@@ -509,7 +510,7 @@ int handle_wu(
         }
     }
 
-    // If we are deferring assimilation until all results are over and validated,
+    // If we're deferring assimilation until all results are over and validated,
     // when that happens make sure that WU state is advanced to assimilate ready
     // the items.size is a kludge
     //
@@ -631,7 +632,7 @@ int handle_wu(
     if (deferred_file_delete_time
         && deferred_file_delete_time < wu_item.transition_time
     ) {
-        wu_item.transition_time = deferred_file_delete_time;
+        wu_item.transition_time = (int)deferred_file_delete_time;
     }
 
     // Handle transitioner overload.
@@ -727,7 +728,7 @@ void main_loop() {
             pause();
 #else
             log_messages.printf(MSG_DEBUG, "sleeping %d\n", sleep_interval);
-            sleep(sleep_interval);
+            daemon_sleep(sleep_interval);
 #endif
         }
     }
@@ -753,7 +754,7 @@ void usage(char *name) {
 
 int main(int argc, char** argv) {
     int i, retval;
-    char path[256];
+    char path[MAXPATHLEN];
 
     startup_time = time(0);
     for (i=1; i<argc; i++) {
@@ -818,4 +819,4 @@ int main(int argc, char** argv) {
     main_loop();
 }
 
-const char *BOINC_RCSID_be98c91511 = "$Id: transitioner.cpp 25435 2012-03-16 17:23:55Z romw $";
+const char *BOINC_RCSID_be98c91511 = "$Id$";
