@@ -26,6 +26,7 @@
 #include <cstdlib>
 #include <cerrno>
 
+#include "error_numbers.h"
 #include "filesys.h"
 #include "util.h"
 #include "sched_util.h"
@@ -38,35 +39,35 @@ int main(int argc, char** argv) {
     char* src_dir, *dst_dir;
     int fanout=0;
     std::string filename;
-    char dst_path[256], src_path[256];
+    char dst_path[MAXPATHLEN], src_path[MAXPATHLEN];
     int retval;
     
     if ( (argc == 1) || !strcmp(argv[1], "-h")  || !strcmp(argv[1],"--help") || (argc != 4) ) {
-      printf (usage);
-      exit(1);
+        fprintf(stderr, usage);
+        exit(1);
     }
     src_dir = argv[1];
     dst_dir = argv[2];
     fanout = atoi(argv[3]);
     if (!fanout) {
-      printf ( usage );
-      exit(1);
+        fprintf(stderr, usage);
+        exit(1);
     }
 
     DirScanner scanner(src_dir);
     while (scanner.scan(filename)) {
         retval = dir_hier_path(filename.c_str(), dst_dir, fanout, dst_path, true);
         if (retval) {
-            fprintf(stderr, "dir_hier_path: %d\n", retval);
+            fprintf(stderr, "dir_hier_path: %s\n", boincerror(retval));
             exit(1);
         }
         sprintf(src_path, "%s/%s", src_dir, filename.c_str());
         retval = rename(src_path, dst_path);
         if (retval) {
-            fprintf(stderr, "rename: %d, errno is %d\n", retval, errno);
+            perror("rename");
             exit(1);
         }
     }
 }
 
-const char *BOINC_RCSID_d6492ba662 = "$Id: dir_hier_move.cpp 17388 2009-02-26 00:23:23Z korpela $";
+const char *BOINC_RCSID_d6492ba662 = "$Id$";
